@@ -5,7 +5,7 @@
 MOZ_APP_BASENAME=B2G
 MOZ_APP_VENDOR=Mozilla
 
-MOZ_APP_VERSION=42.0
+MOZ_APP_VERSION=$FIREFOX_VERSION
 MOZ_APP_UA_NAME=Firefox
 
 MOZ_UA_OS_AGNOSTIC=1
@@ -55,17 +55,11 @@ MOZ_TIME_MANAGER=1
 MOZ_SIMPLEPUSH=1
 MOZ_PAY=1
 MOZ_TOOLKIT_SEARCH=
-MOZ_PLACES=
 MOZ_B2G=1
 
 if test "$OS_TARGET" = "Android"; then
-MOZ_NUWA_PROCESS=1
-MOZ_B2G_LOADER=1
-# Warnings-as-errors cannot be enabled on Lollipop until bug 1119980 is fixed.
-if test "$PLATFORM_SDK_VERSION" -lt 21; then
 MOZ_ENABLE_WARNINGS_AS_ERRORS=1
 fi
-fi
 
 MOZ_JSDOWNLOADS=1
 
