@@ -214,15 +214,6 @@ FeedWriter.prototype = {
     element.setAttribute(attribute, uri);
   },
 
-  __faviconService: null,
-  get _faviconService() {
-    if (!this.__faviconService)
-      this.__faviconService = Cc["@mozilla.org/browser/favicon-service;1"].
-                              getService(Ci.nsIFaviconService);
-
-    return this.__faviconService;
-  },
-
   __bundle: null,
   get _bundle() {
     if (!this.__bundle) {
@@ -934,7 +925,7 @@ FeedWriter.prototype = {
 
     let secman = Cc["@mozilla.org/scriptsecuritymanager;1"].
                  getService(Ci.nsIScriptSecurityManager);
-    this._feedPrincipal = secman.getSimpleCodebasePrincipal(this._feedURI);
+    this._feedPrincipal = secman.createCodebasePrincipal(this._feedURI, {});
 
     LOG("Subscribe Preview: feed uri = " + this._window.location.href);
 
@@ -1027,7 +1018,6 @@ FeedWriter.prototype = {
     prefs.removeObserver(PREF_AUDIO_SELECTED_APP, this);
 
     this._removeFeedFromCache();
-    this.__faviconService = null;
     this.__bundle = null;
     this._feedURI = null;
 
@@ -1168,52 +1158,6 @@ FeedWriter.prototype = {
     }
   },
 
-  /**
-   * Sets the icon for the given web-reader item in the readers menu.
-   * The icon is fetched and stored through the favicon service.
-   *
-   * @param aReaderUrl
-   *        the reader url.
-   * @param aMenuItem
-   *        the reader item in the readers menulist.
-   *
-   * @note For privacy reasons we cannot set the image attribute directly
-   *       to the icon url.  See Bug 358878 for details.
-   */
-  _setFaviconForWebReader:
-  function FW__setFaviconForWebReader(aReaderUrl, aMenuItem) {
-    let readerURI = makeURI(aReaderUrl);
-    if (!/^https?$/.test(readerURI.scheme)) {
-      // Don't try to get a favicon for non http(s) URIs.
-      return;
-    }
-    let faviconURI = makeURI(readerURI.prePath + "/favicon.ico");
-    let self = this;
-
-    let docShell = this._window.QueryInterface(Ci.nsIInterfaceRequestor)
-                               .getInterface(Ci.nsIWebNavigation)
-                               .QueryInterface(Ci.nsIDocShell);
-    let usePrivateBrowsing = docShell.QueryInterface(Ci.nsILoadContext)
-                                     .usePrivateBrowsing;
-
-    // We probably need to call InheritFromDocShellToDoc for this, but right now
-    // we can't call it from JS.
-    let attrs = docShell.getOriginAttributes();
-    let ssm = Services.scriptSecurityManager;
-    let nullPrincipal = ssm.createNullPrincipal(attrs);
-
-    this._faviconService.setAndFetchFaviconForPage(readerURI, faviconURI, false,
-      usePrivateBrowsing ? this._faviconService.FAVICON_LOAD_PRIVATE
-                         : this._faviconService.FAVICON_LOAD_NON_PRIVATE,
-      function (aURI, aDataLen, aData, aMimeType) {
-        if (aDataLen > 0) {
-          let dataURL = "data:" + aMimeType + ";base64," +
-                        btoa(String.fromCharCode.apply(null, aData));
-          aMenuItem.setAttribute('image', dataURL);
-        }
-      }, nullPrincipal);
-  },
-
   get _mm() {
     let mm = this._window.QueryInterface(Ci.nsIInterfaceRequestor).
                           getInterface(Ci.nsIDocShell).
