@@ -1,23 +1,33 @@
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-/* import-globals-from ../../debugger-controller.js */
+
 "use strict";
 
-const utils = require('../utils');
+/* import-globals-from ../../debugger-controller.js */
+
+const utils = require("../utils");
 const {
   getSelectedSource,
   getSourceByURL,
   getBreakpoint,
   getBreakpoints,
   makeLocationId
-} = require('../queries');
+} = require("../queries");
 const actions = Object.assign(
   {},
-  require('../actions/sources'),
-  require('../actions/breakpoints')
+  require("../actions/sources"),
+  require("../actions/breakpoints")
 );
-const { bindActionCreators } = require('devtools/client/shared/vendor/redux');
+const { bindActionCreators } = require("devtools/client/shared/vendor/redux");
+const {
+  Heritage,
+  WidgetMethods,
+  setNamedTimeout
+} = require("devtools/client/shared/widgets/view-helpers");
+const { Task } = require("devtools/shared/task");
+const { SideMenuWidget } = require("resource://devtools/client/shared/widgets/SideMenuWidget.jsm");
+const { gDevTools } = require("devtools/client/framework/devtools");
 
 const NEW_SOURCE_DISPLAY_DELAY = 200; // ms
 const FUNCTION_SEARCH_POPUP_POSITION = "topcenter bottomleft";
@@ -63,15 +73,17 @@ function SourcesView(controller, DebuggerView) {
   this._onConditionalPopupShown = this._onConditionalPopupShown.bind(this);
   this._onConditionalPopupHiding = this._onConditionalPopupHiding.bind(this);
   this._onConditionalTextboxKeyPress = this._onConditionalTextboxKeyPress.bind(this);
+  this._onEditorContextMenuOpen = this._onEditorContextMenuOpen.bind(this);
   this._onCopyUrlCommand = this._onCopyUrlCommand.bind(this);
   this._onNewTabCommand = this._onNewTabCommand.bind(this);
+  this._onConditionalPopupHidden = this._onConditionalPopupHidden.bind(this);
 }
 
 SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Initialization function, called when the debugger is started.
    */
-  initialize: function() {
+  initialize: function (isWorker) {
     dumpn("Initializing the SourcesView");
 
     this.widget = new SideMenuWidget(document.getElementById("sources"), {
@@ -99,7 +111,9 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     this._noResultsFoundToolTip = new Tooltip(document);
     this._noResultsFoundToolTip.defaultPosition = FUNCTION_SEARCH_POPUP_POSITION;
 
-    if (Prefs.prettyPrintEnabled) {
+    // We don't show the pretty print button if debugger a worker
+    // because it simply doesn't work yet. (bug 1273730)
+    if (Prefs.prettyPrintEnabled && !isWorker) {
       this._prettyPrintButton.removeAttribute("hidden");
     }
 
@@ -112,10 +126,12 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     this._cbPanel.addEventListener("popupshowing", this._onConditionalPopupShowing, false);
     this._cbPanel.addEventListener("popupshown", this._onConditionalPopupShown, false);
     this._cbPanel.addEventListener("popuphiding", this._onConditionalPopupHiding, false);
+    this._cbPanel.addEventListener("popuphidden", this._onConditionalPopupHidden, false);
     this._cbTextbox.addEventListener("keypress", this._onConditionalTextboxKeyPress, false);
     this._copyUrlMenuItem.addEventListener("command", this._onCopyUrlCommand, false);
     this._newTabMenuItem.addEventListener("command", this._onNewTabCommand, false);
 
+    this._cbPanel.hidden = true;
     this.allowFocusOnRightClick = true;
     this.autoFocusOnSelection = false;
     this.autoFocusOnFirstItem = false;
@@ -127,33 +143,37 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     });
 
     // Sort known source groups towards the end of the list
-    this.widget.groupSortPredicate = function(a, b) {
+    this.widget.groupSortPredicate = function (a, b) {
       if ((a in KNOWN_SOURCE_GROUPS) == (b in KNOWN_SOURCE_GROUPS)) {
         return a.localeCompare(b);
       }
       return (a in KNOWN_SOURCE_GROUPS) ? 1 : -1;
     };
 
+    this.DebuggerView.editor.on("popupOpen", this._onEditorContextMenuOpen);
+
     this._addCommands();
   },
 
   /**
    * Destruction function, called when the debugger is closed.
    */
-  destroy: function() {
+  destroy: function () {
     dumpn("Destroying the SourcesView");
 
     this.widget.removeEventListener("select", this._onSourceSelect, false);
     this._stopBlackBoxButton.removeEventListener("click", this._onStopBlackBoxing, false);
     this._cbPanel.removeEventListener("popupshowing", this._onConditionalPopupShowing, false);
-    this._cbPanel.removeEventListener("popupshowing", this._onConditionalPopupShown, false);
+    this._cbPanel.removeEventListener("popupshown", this._onConditionalPopupShown, false);
     this._cbPanel.removeEventListener("popuphiding", this._onConditionalPopupHiding, false);
+    this._cbPanel.removeEventListener("popuphidden", this._onConditionalPopupHidden, false);
     this._cbTextbox.removeEventListener("keypress", this._onConditionalTextboxKeyPress, false);
     this._copyUrlMenuItem.removeEventListener("command", this._onCopyUrlCommand, false);
     this._newTabMenuItem.removeEventListener("command", this._onNewTabCommand, false);
+    this.DebuggerView.editor.off("popupOpen", this._onEditorContextMenuOpen, false);
   },
 
-  empty: function() {
+  empty: function () {
     WidgetMethods.empty.call(this);
     this._unnamedSourceIndex = 0;
     this._selectedBreakpoint = null;
@@ -162,7 +182,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Add commands that XUL can fire.
    */
-  _addCommands: function() {
+  _addCommands: function () {
     XULUtils.addCommands(this._commandset, {
       addBreakpointCommand: e => this._onCmdAddBreakpoint(e),
       addConditionalBreakpointCommand: e => this._onCmdAddConditionalBreakpoint(e),
@@ -190,7 +210,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     }
   },
 
-  sourcesDidUpdate: function() {
+  sourcesDidUpdate: function () {
     if (!getSelectedSource(this.getState())) {
       let url = this._preferredSourceURL;
       let source = url && getSourceByURL(this.getState(), url);
@@ -207,9 +227,9 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     }
   },
 
-  renderSource: function(source) {
+  renderSource: function (source) {
     this.addSource(source, { staged: false });
-    for(let bp of getBreakpoints(this.getState())) {
+    for (let bp of getBreakpoints(this.getState())) {
       if (bp.location.actor === source.actor) {
         this.renderBreakpoint(bp);
       }
@@ -226,7 +246,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    *        Additional options for adding the source. Supported options:
    *        - staged: true to stage the item to be appended later
    */
-  addSource: function(aSource, aOptions = {}) {
+  addSource: function (aSource, aOptions = {}) {
     if (!aSource.url && !aOptions.force) {
       // We don't show any unnamed eval scripts yet (see bug 1124106)
       return;
@@ -271,12 +291,12 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     });
   },
 
-  _parseUrl: function(aSource) {
+  _parseUrl: function (aSource) {
     let fullUrl = aSource.url;
     let url, unicodeUrl, label, group;
 
     if (!fullUrl) {
-      unicodeUrl = 'SCRIPT' + this._unnamedSourceIndex++;
+      unicodeUrl = "SCRIPT" + this._unnamedSourceIndex++;
       label = unicodeUrl;
       group = L10N.getStr("anonymousSourcesLabel");
     }
@@ -294,11 +314,11 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     };
   },
 
-  renderBreakpoint: function(breakpoint, removed) {
+  renderBreakpoint: function (breakpoint, removed) {
     if (removed) {
       // Be defensive about the breakpoint not existing.
       if (this._getBreakpoint(breakpoint)) {
-        this._removeBreakpoint(breakpoint)
+        this._removeBreakpoint(breakpoint);
       }
     }
     else {
@@ -319,7 +339,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aOptions [optional]
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _addBreakpoint: function(breakpoint, options = {}) {
+  _addBreakpoint: function (breakpoint, options = {}) {
     let disabled = breakpoint.disabled;
     let location = breakpoint.location;
 
@@ -367,7 +387,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _removeBreakpoint: function(breakpoint) {
+  _removeBreakpoint: function (breakpoint) {
     // When a parent source item is removed, all the child breakpoint items are
     // also automagically removed.
     let sourceItem = this.getItemByValue(breakpoint.location.actor);
@@ -387,7 +407,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     window.emit(EVENTS.BREAKPOINT_HIDDEN_IN_PANE);
   },
 
-  _getBreakpoint: function(bp) {
+  _getBreakpoint: function (bp) {
     return this.getItemForPredicate(item => {
       return item.attachment.actor === bp.location.actor &&
         item.attachment.line === bp.location.line;
@@ -399,7 +419,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    *
    * @param object breakpoint
    */
-  _updateBreakpointStatus: function(breakpoint) {
+  _updateBreakpointStatus: function (breakpoint) {
     let location = breakpoint.location;
     let breakpointItem = this._getBreakpoint(getBreakpoint(this.getState(), location));
     if (!breakpointItem) {
@@ -443,7 +463,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    *          - openPopup: tells if the expression popup should be shown.
    *          - noEditorUpdate: tells if you want to skip editor updates.
    */
-  highlightBreakpoint: function(aLocation, aOptions = {}) {
+  highlightBreakpoint: function (aLocation, aOptions = {}) {
     let breakpoint = getBreakpoint(this.getState(), aLocation);
     if (!breakpoint) {
       return;
@@ -470,7 +490,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * Highlight the breakpoint on the current currently focused line/column
    * if it exists.
    */
-  highlightBreakpointAtCursor: function() {
+  highlightBreakpointAtCursor: function () {
     let actor = this.selectedValue;
     let line = this.DebuggerView.editor.getCursor().line + 1;
 
@@ -481,7 +501,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Unhighlights the current breakpoint in this sources container.
    */
-  unhighlightBreakpoint: function() {
+  unhighlightBreakpoint: function () {
     this._hideConditionalPopup();
     this._unselectBreakpoint();
   },
@@ -489,7 +509,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    /**
     * Display the message thrown on breakpoint condition
     */
-  showBreakpointConditionThrownMessage: function(aLocation, aMessage = "") {
+  showBreakpointConditionThrownMessage: function (aLocation, aMessage = "") {
     let breakpointItem = this._getBreakpoint(getBreakpoint(this.getState(), aLocation));
     if (!breakpointItem) {
       return;
@@ -503,7 +523,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * Update the checked/unchecked and enabled/disabled states of the buttons in
    * the sources toolbar based on the currently selected source's state.
    */
-  updateToolbarButtonsState: function(source) {
+  updateToolbarButtonsState: function (source) {
     if (source.isBlackBoxed) {
       this._blackBoxButton.setAttribute("checked", true);
       this._prettyPrintButton.setAttribute("checked", true);
@@ -522,7 +542,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Toggle the pretty printing of the selected source.
    */
-  togglePrettyPrint: function() {
+  togglePrettyPrint: function () {
     if (this._prettyPrintButton.hasAttribute("disabled")) {
       return;
     }
@@ -546,7 +566,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Toggle the black boxed state of the selected source.
    */
-  toggleBlackBoxing: Task.async(function*() {
+  toggleBlackBoxing: Task.async(function* () {
     const source = getSelectedSource(this.getState());
     const shouldBlackBox = !source.isBlackBoxed;
 
@@ -564,7 +584,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     this.actions.blackbox(source, shouldBlackBox);
   }),
 
-  renderBlackBoxed: function(source) {
+  renderBlackBoxed: function (source) {
     const sourceItem = this.getItemByValue(source.actor);
     sourceItem.prebuiltNode.classList.toggle(
       "black-boxed",
@@ -579,7 +599,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Toggles all breakpoints enabled/disabled.
    */
-  toggleBreakpoints: function() {
+  toggleBreakpoints: function () {
     let breakpoints = getBreakpoints(this.getState());
     let hasBreakpoints = breakpoints.length > 0;
     let hasEnabledBreakpoints = breakpoints.some(bp => !bp.disabled);
@@ -593,7 +613,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     }
   },
 
-  togglePromiseDebugger: function() {
+  togglePromiseDebugger: function () {
     if (Prefs.promiseDebuggerEnabled) {
       let promisePane = this.DebuggerView._promisePane;
       promisePane.hidden = !promisePane.hidden;
@@ -604,29 +624,29 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     }
   },
 
-  hidePrettyPrinting: function() {
-    this._prettyPrintButton.style.display = 'none';
+  hidePrettyPrinting: function () {
+    this._prettyPrintButton.style.display = "none";
 
-    if (this._blackBoxButton.style.display === 'none') {
-      let sep = document.querySelector('#sources-toolbar .devtools-separator');
-      sep.style.display = 'none';
+    if (this._blackBoxButton.style.display === "none") {
+      let sep = document.querySelector("#sources-toolbar .devtools-separator");
+      sep.style.display = "none";
     }
   },
 
-  hideBlackBoxing: function() {
-    this._blackBoxButton.style.display = 'none';
+  hideBlackBoxing: function () {
+    this._blackBoxButton.style.display = "none";
 
-    if (this._prettyPrintButton.style.display === 'none') {
-      let sep = document.querySelector('#sources-toolbar .devtools-separator');
-      sep.style.display = 'none';
+    if (this._prettyPrintButton.style.display === "none") {
+      let sep = document.querySelector("#sources-toolbar .devtools-separator");
+      sep.style.display = "none";
     }
   },
 
-  getDisplayURL: function(source) {
+  getDisplayURL: function (source) {
     if (!source.url) {
       return this.getItemByValue(source.actor).attachment.label;
     }
-    return NetworkHelper.convertToUnicode(unescape(source.url))
+    return NetworkHelper.convertToUnicode(unescape(source.url));
   },
 
   /**
@@ -635,7 +655,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aItem
    *        The breakpoint item to select.
    */
-  _selectBreakpoint: function(bp) {
+  _selectBreakpoint: function (bp) {
     if (this._selectedBreakpoint === bp) {
       return;
     }
@@ -652,7 +672,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Marks the current breakpoint as unselected in this sources container.
    */
-  _unselectBreakpoint: function() {
+  _unselectBreakpoint: function () {
     if (!this._selectedBreakpoint) {
       return;
     }
@@ -666,33 +686,43 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Opens a conditional breakpoint's expression input popup.
    */
-  _openConditionalPopup: function() {
+  _openConditionalPopup: function () {
     let breakpointItem = this._getBreakpoint(this._selectedBreakpoint);
     let attachment = breakpointItem.attachment;
     // Check if this is an enabled conditional breakpoint, and if so,
     // retrieve the current conditional epression.
     let bp = getBreakpoint(this.getState(), attachment);
     let expr = (bp ? (bp.condition || "") : "");
+    let cbPanel = this._cbPanel;
 
     // Update the conditional expression textbox. If no expression was
     // previously set, revert to using an empty string by default.
     this._cbTextbox.value = expr;
 
-    // Show the conditional expression panel. The popup arrow should be pointing
-    // at the line number node in the breakpoint item view.
-    this._cbPanel.hidden = false;
-    this._cbPanel.openPopup(breakpointItem.attachment.view.lineNumber,
-                            BREAKPOINT_CONDITIONAL_POPUP_POSITION,
-                            BREAKPOINT_CONDITIONAL_POPUP_OFFSET_X,
-                            BREAKPOINT_CONDITIONAL_POPUP_OFFSET_Y);
+    function openPopup() {
+      // Show the conditional expression panel. The popup arrow should be pointing
+      // at the line number node in the breakpoint item view.
+      cbPanel.hidden = false;
+      cbPanel.openPopup(breakpointItem.attachment.view.lineNumber,
+                              BREAKPOINT_CONDITIONAL_POPUP_POSITION,
+                              BREAKPOINT_CONDITIONAL_POPUP_OFFSET_X,
+                              BREAKPOINT_CONDITIONAL_POPUP_OFFSET_Y);
+
+      cbPanel.removeEventListener("popuphidden", openPopup, false);
+    }
+
+    // Wait until the other cb panel is closed
+    if (!this._cbPanel.hidden) {
+      this._cbPanel.addEventListener("popuphidden", openPopup, false);
+    } else {
+      openPopup();
+    }
   },
 
   /**
    * Hides a conditional breakpoint's expression input popup.
    */
-  _hideConditionalPopup: function() {
-    this._cbPanel.hidden = true;
-
+  _hideConditionalPopup: function () {
     // Sometimes this._cbPanel doesn't have hidePopup method which doesn't
     // break anything but simply outputs an exception to the console.
     if (this._cbPanel.hidePopup) {
@@ -713,7 +743,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    *         An object containing the breakpoint container, checkbox,
    *         line number and line text nodes.
    */
-  _createBreakpointView: function(aOptions) {
+  _createBreakpointView: function (aOptions) {
     let { location, disabled, text, message } = aOptions;
     let identifier = makeLocationId(location);
 
@@ -788,7 +818,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @return object
    *         An object containing the breakpoint commandset and menu popup ids.
    */
-  _createContextMenu: function(aOptions) {
+  _createContextMenu: function (aOptions) {
     let { location, disabled } = aOptions;
     let identifier = makeLocationId(location);
 
@@ -865,7 +895,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Copy the source url from the currently selected item.
    */
-  _onCopyUrlCommand: function() {
+  _onCopyUrlCommand: function () {
     let selected = this.selectedItem && this.selectedItem.attachment;
     if (!selected) {
       return;
@@ -876,8 +906,8 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Opens selected item source in a new tab.
    */
-  _onNewTabCommand: function() {
-    let win = Services.wm.getMostRecentWindow("navigator:browser");
+  _onNewTabCommand: function () {
+    let win = Services.wm.getMostRecentWindow(gDevTools.chromeWindowType);
     let selected = this.selectedItem.attachment;
     win.openUILinkIn(selected.source.url, "tab", { relatedToCurrent: true });
   },
@@ -888,7 +918,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aItem
    *        The corresponding item.
    */
-  _onBreakpointRemoved: function(aItem) {
+  _onBreakpointRemoved: function (aItem) {
     dumpn("Finalizing breakpoint item: " + aItem.stringify());
 
     // Destroy the context menu for the breakpoint.
@@ -897,7 +927,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     document.getElementById(contextMenu.menupopupId).remove();
   },
 
-  _onMouseDown: function(e) {
+  _onMouseDown: function (e) {
     this.hideNoResultsTooltip();
 
     if (!e.metaKey) {
@@ -908,7 +938,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     let identifier = this._findIdentifier(e.clientX, e.clientY);
 
     if (!identifier) {
-        return;
+      return;
     }
 
     let foundDefinitions = this._getFunctionDefinitions(identifier);
@@ -924,7 +954,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * Searches for function definition of a function in a given source file
    */
 
-  _findDefinition: function(parsedSource, aName) {
+  _findDefinition: function (parsedSource, aName) {
     let functionDefinitions = parsedSource.getNamedFunctionDefinitions(aName);
 
     let resultList = [];
@@ -944,13 +974,13 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
         startLine: functionDefinitions[i][0].functionLocation.start.line,
         startColumn: functionDefinitions[i][0].functionLocation.start.column,
         name: functionDefinitions[i][0].functionName
-      }
+      };
 
-      resultList.push(functionDefinition)
+      resultList.push(functionDefinition);
     }
 
     return {
-     definitions: resultList
+      definitions: resultList
     };
   },
 
@@ -961,13 +991,13 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param number x, y
    *        The left/top coordinates where to look for an identifier.
    */
-  _findIdentifier: function(x, y) {
+  _findIdentifier: function (x, y) {
     let parsedSource = SourceUtils.parseSource(this.DebuggerView, this.Parser);
     let identifierInfo = SourceUtils.findIdentifier(this.DebuggerView.editor, parsedSource, x, y);
 
     // Not hovering over an identifier
     if (!identifierInfo) {
-        return;
+      return;
     }
 
     return identifierInfo;
@@ -976,7 +1006,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The selection listener for the source editor.
    */
-  _onEditorCursorActivity: function(e) {
+  _onEditorCursorActivity: function (e) {
     let editor = this.DebuggerView.editor;
     let start = editor.getCursor("start").line + 1;
     let end = editor.getCursor().line + 1;
@@ -996,14 +1026,14 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * Uses function definition data to perform actions in different
    * cases of how many locations were found: zero, one, or multiple definitions
    */
-  _showFunctionDefinitionResults: function(aHoveredFunction, aDefinitionList, aEditor) {
+  _showFunctionDefinitionResults: function (aHoveredFunction, aDefinitionList, aEditor) {
     let definitions = aDefinitionList;
     let hoveredFunction = aHoveredFunction;
 
-    //show a popup saying no results were found
+    // show a popup saying no results were found
     if (definitions.length == 0) {
       this._noResultsFoundToolTip.setTextContent({
-          messages: [L10N.getStr("noMatchingStringsText")]
+        messages: [L10N.getStr("noMatchingStringsText")]
       });
 
       this._markedIdentifier = aEditor.markText(
@@ -1018,12 +1048,12 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
       // TODO: multiple definitions found, do something else
       this.DebuggerView.setEditorLocation(definitions[0].source, definitions[0].startLine);
     }
-},
+  },
 
   /**
    * Hides the tooltip and clear marked text popup.
    */
-  hideNoResultsTooltip: function() {
+  hideNoResultsTooltip: function () {
     this._noResultsFoundToolTip.hide();
     if (this._markedIdentifier) {
       this._markedIdentifier.clear();
@@ -1034,11 +1064,11 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /*
    * Gets the definition locations from function metadata
    */
-  _getFunctionDefinitions: function(aIdentifierInfo) {
+  _getFunctionDefinitions: function (aIdentifierInfo) {
     let parsedSource = SourceUtils.parseSource(this.DebuggerView, this.Parser);
     let definition_info = this._findDefinition(parsedSource, aIdentifierInfo.name);
 
-    //Did not find any definitions for the identifier
+    // Did not find any definitions for the identifier
     if (!definition_info) {
       return;
     }
@@ -1049,7 +1079,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The select listener for the sources container.
    */
-  _onSourceSelect: function({ detail: sourceItem }) {
+  _onSourceSelect: function ({ detail: sourceItem }) {
     if (!sourceItem) {
       return;
     }
@@ -1058,11 +1088,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     this.actions.selectSource(source);
   },
 
-  renderSourceSelected: function(source) {
-    // Set window title. No need to split the url by " -> " here,
-    // because it was already sanitized when the source was added.
-    document.title = L10N.getFormatStr("DebuggerWindowScriptTitle", source.url);
-
+  renderSourceSelected: function (source) {
     if (source.url) {
       this._preferredSourceURL = source.url;
     }
@@ -1073,14 +1099,37 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The click listener for the "stop black boxing" button.
    */
-  _onStopBlackBoxing: Task.async(function*() {
+  _onStopBlackBoxing: Task.async(function* () {
     this.actions.blackbox(getSelectedSource(this.getState()), false);
   }),
 
+  /**
+   * The source editor's contextmenu handler.
+   * - Toggles "Add Conditional Breakpoint" and "Edit Conditional Breakpoint" items
+   */
+  _onEditorContextMenuOpen: function (message, ev, popup) {
+    let actor = this.selectedValue;
+    let line = this.DebuggerView.editor.getCursor().line + 1;
+    let location = { actor, line };
+
+    let breakpoint = getBreakpoint(this.getState(), location);
+    let addConditionalBreakpointMenuItem = popup.querySelector("#se-dbg-cMenu-addConditionalBreakpoint");
+    let editConditionalBreakpointMenuItem = popup.querySelector("#se-dbg-cMenu-editConditionalBreakpoint");
+
+    if (breakpoint && !!breakpoint.condition) {
+      editConditionalBreakpointMenuItem.removeAttribute("hidden");
+      addConditionalBreakpointMenuItem.setAttribute("hidden", true);
+    }
+    else {
+      addConditionalBreakpointMenuItem.removeAttribute("hidden");
+      editConditionalBreakpointMenuItem.setAttribute("hidden", true);
+    }
+  },
+
   /**
    * The click listener for a breakpoint container.
    */
-  _onBreakpointClick: function(e) {
+  _onBreakpointClick: function (e) {
     let sourceItem = this.getItemForElement(e.target);
     let breakpointItem = this.getItemForElement.call(sourceItem, e.target);
     let attachment = breakpointItem.attachment;
@@ -1097,7 +1146,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The click listener for a breakpoint checkbox.
    */
-  _onBreakpointCheckboxClick: function(e) {
+  _onBreakpointCheckboxClick: function (e) {
     let sourceItem = this.getItemForElement(e.target);
     let breakpointItem = this.getItemForElement.call(sourceItem, e.target);
     let bp = getBreakpoint(this.getState(), breakpointItem.attachment);
@@ -1117,7 +1166,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The popup showing listener for the breakpoints conditional expression panel.
    */
-  _onConditionalPopupShowing: function() {
+  _onConditionalPopupShowing: function () {
     this._conditionalPopupVisible = true; // Used in tests.
     window.emit(EVENTS.CONDITIONAL_BREAKPOINT_POPUP_SHOWING);
   },
@@ -1125,7 +1174,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The popup shown listener for the breakpoints conditional expression panel.
    */
-  _onConditionalPopupShown: function() {
+  _onConditionalPopupShown: function () {
     this._cbTextbox.focus();
     this._cbTextbox.select();
   },
@@ -1133,7 +1182,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * The popup hiding listener for the breakpoints conditional expression panel.
    */
-  _onConditionalPopupHiding: function() {
+  _onConditionalPopupHiding: function () {
     this._conditionalPopupVisible = false; // Used in tests.
 
     // Check if this is an enabled conditional breakpoint, and if so,
@@ -1145,10 +1194,17 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     }
   },
 
+  /**
+   * The popup hidden listener for the breakpoints conditional expression panel.
+   */
+  _onConditionalPopupHidden: function () {
+    this._cbPanel.hidden = true;
+  },
+
   /**
    * The keypress listener for the breakpoints conditional expression textbox.
    */
-  _onConditionalTextboxKeyPress: function(e) {
+  _onConditionalTextboxKeyPress: function (e) {
     if (e.keyCode == e.DOM_VK_RETURN) {
       this._hideConditionalPopup();
     }
@@ -1157,9 +1213,9 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Called when the add breakpoint key sequence was pressed.
    */
-  _onCmdAddBreakpoint: function(e) {
+  _onCmdAddBreakpoint: function (e) {
     let actor = this.selectedValue;
-    let line = (e && e.sourceEvent.target.tagName == 'menuitem' ?
+    let line = (this.DebuggerView.clickedLine ?
                 this.DebuggerView.clickedLine + 1 :
                 this.DebuggerView.editor.getCursor().line + 1);
     let location = { actor, line };
@@ -1178,11 +1234,12 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Called when the add conditional breakpoint key sequence was pressed.
    */
-  _onCmdAddConditionalBreakpoint: function(e) {
+  _onCmdAddConditionalBreakpoint: function (e) {
     let actor = this.selectedValue;
-    let line = (e && e.sourceEvent.target.tagName == 'menuitem' ?
+    let line = (this.DebuggerView.clickedLine ?
                 this.DebuggerView.clickedLine + 1 :
                 this.DebuggerView.editor.getCursor().line + 1);
+
     let location = { actor, line };
     let bp = getBreakpoint(this.getState(), location);
 
@@ -1196,7 +1253,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
     }
   },
 
-  getOtherBreakpoints: function(location) {
+  getOtherBreakpoints: function (location) {
     const bps = getBreakpoints(this.getState());
     if (location) {
       return bps.filter(bp => {
@@ -1213,7 +1270,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onSetConditional: function(aLocation) {
+  _onSetConditional: function (aLocation) {
     // Highlight the breakpoint and show a conditional expression popup.
     this.highlightBreakpoint(aLocation, { openPopup: true });
   },
@@ -1224,7 +1281,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onEnableSelf: function(aLocation) {
+  _onEnableSelf: function (aLocation) {
     // Enable the breakpoint, in this container and the controller store.
     this.actions.enableBreakpoint(aLocation);
   },
@@ -1235,7 +1292,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onDisableSelf: function(aLocation) {
+  _onDisableSelf: function (aLocation) {
     const bp = getBreakpoint(this.getState(), aLocation);
     if (!bp.disabled) {
       this.actions.disableBreakpoint(aLocation);
@@ -1248,7 +1305,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onDeleteSelf: function(aLocation) {
+  _onDeleteSelf: function (aLocation) {
     this.actions.removeBreakpoint(aLocation);
   },
 
@@ -1258,7 +1315,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onEnableOthers: function(aLocation) {
+  _onEnableOthers: function (aLocation) {
     let other = this.getOtherBreakpoints(aLocation);
     // TODO(jwl): batch these and interrupt the thread for all of them
     other.forEach(bp => this._onEnableSelf(bp.location));
@@ -1270,7 +1327,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onDisableOthers: function(aLocation) {
+  _onDisableOthers: function (aLocation) {
     let other = this.getOtherBreakpoints(aLocation);
     other.forEach(bp => this._onDisableSelf(bp.location));
   },
@@ -1281,7 +1338,7 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
    * @param object aLocation
    *        @see DebuggerController.Breakpoints.addBreakpoint
    */
-  _onDeleteOthers: function(aLocation) {
+  _onDeleteOthers: function (aLocation) {
     let other = this.getOtherBreakpoints(aLocation);
     other.forEach(bp => this._onDeleteSelf(bp.location));
   },
@@ -1289,21 +1346,21 @@ SourcesView.prototype = Heritage.extend(WidgetMethods, {
   /**
    * Function invoked on the "enableAll" menuitem command.
    */
-  _onEnableAll: function() {
+  _onEnableAll: function () {
     this._onEnableOthers(undefined);
   },
 
   /**
    * Function invoked on the "disableAll" menuitem command.
    */
-  _onDisableAll: function() {
+  _onDisableAll: function () {
     this._onDisableOthers(undefined);
   },
 
   /**
    * Function invoked on the "deleteAll" menuitem command.
    */
-  _onDeleteAll: function() {
+  _onDeleteAll: function () {
     this._onDeleteOthers(undefined);
   },
 
