@@ -6,24 +6,17 @@
 
 "use strict";
 
-const {Cc, Ci, Cu} = require("chrome");
+/* eslint-disable mozilla/reject-some-requires */
+const {Ci} = require("chrome");
+/* eslint-enable mozilla/reject-some-requires */
 const promise = require("promise");
-const {CssLogic} = require("devtools/shared/inspector/css-logic");
-const {ELEMENT_STYLE} = require("devtools/server/actors/styles");
+const CssLogic = require("devtools/shared/inspector/css-logic");
+const {ELEMENT_STYLE} = require("devtools/shared/specs/styles");
 const {TextProperty} =
       require("devtools/client/inspector/rules/models/text-property");
 const {promiseWarn} = require("devtools/client/inspector/shared/utils");
-const {parseDeclarations} = require("devtools/client/shared/css-parsing-utils");
-
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-
-XPCOMUtils.defineLazyGetter(this, "osString", function() {
-  return Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime).OS;
-});
-
-XPCOMUtils.defineLazyGetter(this, "domUtils", function() {
-  return Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
-});
+const {parseDeclarations} = require("devtools/shared/css-parsing-utils");
+const Services = require("Services");
 
 /**
  * Rule is responsible for the following:
@@ -59,6 +52,8 @@ function Rule(elementStyle, options) {
     this.mediaText = this.domRule.mediaText;
   }
 
+  this.cssProperties = this.elementStyle.ruleView.cssProperties;
+
   // Populate the text properties with the style's current authoredText
   // value, and add in any disabled properties from the store.
   this.textProps = this._getTextProperties();
@@ -83,7 +78,7 @@ Rule.prototype = {
     }
     this._inheritedSource = "";
     if (this.inherited) {
-      let eltText = this.inherited.tagName.toLowerCase();
+      let eltText = this.inherited.displayName;
       if (this.inherited.id) {
         eltText += "#" + this.inherited.id;
       }
@@ -141,7 +136,7 @@ Rule.prototype = {
    *         Promise which resolves with location as an object containing
    *         both the full and short version of the source string.
    */
-  getOriginalSourceStrings: function() {
+  getOriginalSourceStrings: function () {
     return this.domRule.getOriginalLocation().then(({href,
                                                      line, mediaText}) => {
       let mediaString = mediaText ? " @" + mediaText : "";
@@ -164,7 +159,7 @@ Rule.prototype = {
    * @param {Object} options
    *        Creation options. See the Rule constructor for documentation.
    */
-  matches: function(options) {
+  matches: function (options) {
     return this.style === options.rule;
   },
 
@@ -182,7 +177,7 @@ Rule.prototype = {
    * @param {TextProperty} siblingProp
    *        Optional, property next to which the new property will be added.
    */
-  createProperty: function(name, value, priority, enabled, siblingProp) {
+  createProperty: function (name, value, priority, enabled, siblingProp) {
     let prop = new TextProperty(this, name, value, priority, enabled);
 
     let ind;
@@ -196,7 +191,11 @@ Rule.prototype = {
 
     this.applyProperties((modifications) => {
       modifications.createProperty(ind, name, value, priority);
+      // Now that the rule has been updated, the server might have given us data
+      // that changes the state of the property. Update it now.
+      prop.updateEditor();
     });
+
     return prop;
   },
 
@@ -205,7 +204,7 @@ Rule.prototype = {
    * does not support as-authored styles.  Store disabled properties
    * in the element style's store.
    */
-  _applyPropertiesNoAuthored: function(modifications) {
+  _applyPropertiesNoAuthored: function (modifications) {
     this.elementStyle.markOverriddenAll();
 
     let disabledProps = [];
@@ -241,7 +240,11 @@ Rule.prototype = {
 
     return modifications.apply().then(() => {
       let cssProps = {};
-      for (let cssProp of parseDeclarations(this.style.authoredText)) {
+      // Note that even though StyleRuleActors normally provide parsed
+      // declarations already, _applyPropertiesNoAuthored is only used when
+      // connected to older backend that do not provide them. So parse here.
+      for (let cssProp of parseDeclarations(this.cssProperties.isKnown,
+                                            this.style.authoredText)) {
         cssProps[cssProp.name] = cssProp;
       }
 
@@ -269,7 +272,7 @@ Rule.prototype = {
    * authored" case; that is, when the StyleRuleActor supports
    * setRuleText.
    */
-  _applyPropertiesAuthored: function(modifications) {
+  _applyPropertiesAuthored: function (modifications) {
     return modifications.apply().then(() => {
       // The rewriting may have required some other property values to
       // change, e.g., to insert some needed terminators.  Update the
@@ -300,12 +303,13 @@ Rule.prototype = {
    * @return {Promise} a promise which will resolve when the edit
    *        is complete
    */
-  applyProperties: function(modifier) {
+  applyProperties: function (modifier) {
     // If there is already a pending modification, we have to wait
     // until it settles before applying the next modification.
     let resultPromise =
         promise.resolve(this._applyingModifications).then(() => {
-          let modifications = this.style.startModifyingProperties();
+          let modifications = this.style.startModifyingProperties(
+            this.cssProperties);
           modifier(modifications);
           if (this.style.canSetRuleText) {
             return this._applyPropertiesAuthored(modifications);
@@ -332,7 +336,7 @@ Rule.prototype = {
    * @param {String} name
    *        The new property name (such as "background" or "border-top").
    */
-  setPropertyName: function(property, name) {
+  setPropertyName: function (property, name) {
     if (name === property.name) {
       return;
     }
@@ -355,7 +359,7 @@ Rule.prototype = {
    * @param {String} priority
    *        The property's priority (either "important" or an empty string).
    */
-  setPropertyValue: function(property, value, priority) {
+  setPropertyValue: function (property, value, priority) {
     if (value === property.value && priority === property.priority) {
       return;
     }
@@ -380,8 +384,8 @@ Rule.prototype = {
    * @param {String} priority
    *        The property's priority (either "important" or an empty string).
    */
-  previewPropertyValue: function(property, value, priority) {
-    let modifications = this.style.startModifyingProperties();
+  previewPropertyValue: function (property, value, priority) {
+    let modifications = this.style.startModifyingProperties(this.cssProperties);
     modifications.setProperty(this.textProps.indexOf(property),
                               property.name, value, priority);
     modifications.apply().then(() => {
@@ -398,7 +402,7 @@ Rule.prototype = {
    *        The property to enable/disable
    * @param {Boolean} value
    */
-  setPropertyEnabled: function(property, value) {
+  setPropertyEnabled: function (property, value) {
     if (property.enabled === !!value) {
       return;
     }
@@ -416,7 +420,7 @@ Rule.prototype = {
    * @param {TextProperty} property
    *        The property to be removed
    */
-  removeProperty: function(property) {
+  removeProperty: function (property) {
     let index = this.textProps.indexOf(property);
     this.textProps.splice(index, 1);
     // Need to re-apply properties in case removing this TextProperty
@@ -430,17 +434,30 @@ Rule.prototype = {
    * Get the list of TextProperties from the style. Needs
    * to parse the style's authoredText.
    */
-  _getTextProperties: function() {
+  _getTextProperties: function () {
     let textProps = [];
     let store = this.elementStyle.store;
-    let props = parseDeclarations(this.style.authoredText, true);
+
+    // Starting with FF49, StyleRuleActors provide parsed declarations.
+    let props = this.style.declarations;
+    if (!props.length) {
+      props = parseDeclarations(this.cssProperties.isKnown,
+                                this.style.authoredText, true);
+    }
+
     for (let prop of props) {
       let name = prop.name;
+      // If the authored text has an invalid property, it will show up
+      // as nameless.  Skip these as we don't currently have a good
+      // way to display them.
+      if (!name) {
+        continue;
+      }
       // In an inherited rule, we only show inherited properties.
       // However, we must keep all properties in order for rule
       // rewriting to work properly.  So, compute the "invisible"
       // property here.
-      let invisible = this.inherited && !domUtils.isInheritedProperty(name);
+      let invisible = this.inherited && !this.cssProperties.isInherited(name);
       let value = store.userProperties.getProperty(this.style, name,
                                                    prop.value);
       let textProp = new TextProperty(this, name, value, prop.priority,
@@ -455,7 +472,7 @@ Rule.prototype = {
   /**
    * Return the list of disabled properties from the store for this rule.
    */
-  _getDisabledProperties: function() {
+  _getDisabledProperties: function () {
     let store = this.elementStyle.store;
 
     // Include properties from the disabled property store, if any.
@@ -481,7 +498,7 @@ Rule.prototype = {
    * Reread the current state of the rules and rebuild text
    * properties as needed.
    */
-  refresh: function(options) {
+  refresh: function (options) {
     this.matchedSelectors = options.matchedSelectors || [];
     let newTextProps = this._getTextProperties();
 
@@ -542,7 +559,7 @@ Rule.prototype = {
    * @return {Boolean} true if a property was updated, false if no properties
    *         were updated.
    */
-  _updateTextProperty: function(newProp) {
+  _updateTextProperty: function (newProp) {
     let match = { rank: 0, prop: null };
 
     for (let prop of this.textProps) {
@@ -607,7 +624,7 @@ Rule.prototype = {
    * @param {Number} direction
    *        The move focus direction number.
    */
-  editClosestTextProperty: function(textProperty, direction) {
+  editClosestTextProperty: function (textProperty, direction) {
     let index = this.textProps.indexOf(textProperty);
 
     if (direction === Ci.nsIFocusManager.MOVEFOCUS_FORWARD) {
@@ -638,10 +655,10 @@ Rule.prototype = {
   /**
    * Return a string representation of the rule.
    */
-  stringifyRule: function() {
+  stringifyRule: function () {
     let selectorText = this.selectorText;
     let cssText = "";
-    let terminator = osString === "WINNT" ? "\r\n" : "\n";
+    let terminator = Services.appinfo.OS === "WINNT" ? "\r\n" : "\n";
 
     for (let textProp of this.textProps) {
       if (!textProp.invisible) {
@@ -657,7 +674,7 @@ Rule.prototype = {
    * @return {Boolean} true if there is any visible property, or false
    *         if all properties are invisible
    */
-  hasAnyVisibleProperties: function() {
+  hasAnyVisibleProperties: function () {
     for (let prop of this.textProps) {
       if (!prop.invisible) {
         return true;
