@@ -6,19 +6,18 @@
 
 const { Cc, Ci, Cu } = require("chrome");
 const promise = require("promise");
+const defer = require("devtools/shared/defer");
 const Services = require("Services");
 const { TargetFactory } = require("devtools/client/framework/target");
 const Telemetry = require("devtools/client/shared/telemetry");
+const {ViewHelpers} = require("devtools/client/shared/widgets/view-helpers");
 
 const NS_XHTML = "http://www.w3.org/1999/xhtml";
 const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
-const Node = Ci.nsIDOMNode;
 
-loader.lazyImporter(this, "console", "resource://gre/modules/Console.jsm");
 loader.lazyImporter(this, "PluralForm", "resource://gre/modules/PluralForm.jsm");
-loader.lazyImporter(this, "EventEmitter", "resource://devtools/shared/event-emitter.js");
 
-loader.lazyGetter(this, "prefBranch", function() {
+loader.lazyGetter(this, "prefBranch", function () {
   return Services.prefs.getBranch(null)
                     .QueryInterface(Ci.nsIPrefBranch2);
 });
@@ -28,7 +27,11 @@ loader.lazyGetter(this, "toolboxStrings", function () {
 
 loader.lazyRequireGetter(this, "gcliInit", "devtools/shared/gcli/commands/index");
 loader.lazyRequireGetter(this, "util", "gcli/util/util");
-loader.lazyRequireGetter(this, "ConsoleServiceListener", "devtools/shared/webconsole/utils", true);
+loader.lazyRequireGetter(this, "ConsoleServiceListener", "devtools/server/actors/utils/webconsole-utils", true);
+loader.lazyRequireGetter(this, "gDevTools", "devtools/client/framework/devtools", true);
+loader.lazyRequireGetter(this, "gDevToolsBrowser", "devtools/client/framework/devtools-browser", true);
+loader.lazyRequireGetter(this, "nodeConstants", "devtools/shared/dom-node-constants", true);
+loader.lazyRequireGetter(this, "EventEmitter", "devtools/shared/event-emitter");
 
 /**
  * A collection of utilities to help working with commands
@@ -37,7 +40,7 @@ var CommandUtils = {
   /**
    * Utility to ensure that things are loaded in the correct order
    */
-  createRequisition: function(target, options) {
+  createRequisition: function (target, options) {
     if (!gcliInit) {
       return promise.reject("Unable to load gcli");
     }
@@ -50,7 +53,7 @@ var CommandUtils = {
   /**
    * Destroy the remote side of the requisition as well as the local side
    */
-  destroyRequisition: function(requisition, target) {
+  destroyRequisition: function (requisition, target) {
     requisition.destroy();
     gcliInit.releaseSystem(target);
   },
@@ -59,7 +62,7 @@ var CommandUtils = {
    * Read a toolbarSpec from preferences
    * @param pref The name of the preference to read
    */
-  getCommandbarSpec: function(pref) {
+  getCommandbarSpec: function (pref) {
     let value = prefBranch.getComplexValue(pref, Ci.nsISupportsString).data;
     return JSON.parse(value);
   },
@@ -71,11 +74,11 @@ var CommandUtils = {
    * buttons that are of type checkbox. this means that we don't properly
    * unregister event handlers until the window is destroyed.
    */
-  createButtons: function(toolbarSpec, target, document, requisition) {
+  createButtons: function (toolbarSpec, target, document, requisition) {
     return util.promiseEach(toolbarSpec, typed => {
       // Ask GCLI to parse the typed string (doesn't execute it)
       return requisition.update(typed).then(() => {
-        let button = document.createElement("toolbarbutton");
+        let button = document.createElementNS(NS_XHTML, "button");
 
         // Ignore invalid commands
         let command = requisition.commandAssignment.value;
@@ -92,19 +95,28 @@ var CommandUtils = {
         else {
           button.setAttribute("text-as-image", "true");
           button.setAttribute("label", command.name);
-          button.className = "devtools-toolbarbutton";
         }
+
+        button.classList.add("devtools-button");
+
         if (command.tooltipText != null) {
-          button.setAttribute("tooltiptext", command.tooltipText);
+          button.setAttribute("title", command.tooltipText);
         }
         else if (command.description != null) {
-          button.setAttribute("tooltiptext", command.description);
+          button.setAttribute("title", command.description);
         }
 
         button.addEventListener("click", () => {
           requisition.updateExec(typed);
         }, false);
 
+        button.addEventListener("keypress", (event) => {
+          if (ViewHelpers.isSpaceOrReturn(event)) {
+            event.preventDefault();
+            requisition.updateExec(typed);
+          }
+        }, false);
+
         // Allow the command button to be toggleable
         if (command.state) {
           button.setAttribute("autocheck", false);
@@ -164,7 +176,7 @@ var CommandUtils = {
    * @param targetContainer An object containing a 'target' property which
    * reflects the current debug target
    */
-  createEnvironment: function(container, targetProperty="target") {
+  createEnvironment: function (container, targetProperty = "target") {
     if (!container[targetProperty].toString ||
         !/TabTarget/.test(container[targetProperty].toString())) {
       throw new Error("Missing target");
@@ -210,16 +222,11 @@ exports.CommandUtils = CommandUtils;
  * When bug 780102 is fixed all isLinux checks can be removed and we can revert
  * to using panels.
  */
-loader.lazyGetter(this, "isLinux", function() {
-  return OS == "Linux";
+loader.lazyGetter(this, "isLinux", function () {
+  return Services.appinfo.OS == "Linux";
 });
-loader.lazyGetter(this, "isMac", function() {
-  return OS == "Darwin";
-});
-
-loader.lazyGetter(this, "OS", function() {
-  let os = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime).OS;
-  return os;
+loader.lazyGetter(this, "isMac", function () {
+  return Services.appinfo.OS == "Darwin";
 });
 
 /**
@@ -240,6 +247,9 @@ function DeveloperToolbar(aChromeWindow)
   this._warningsCount = {};
   this._errorListeners = {};
 
+  this._onToolboxReady = this._onToolboxReady.bind(this);
+  this._onToolboxDestroyed = this._onToolboxDestroyed.bind(this);
+
   EventEmitter.decorate(this);
 }
 exports.DeveloperToolbar = DeveloperToolbar;
@@ -268,7 +278,7 @@ DeveloperToolbar.prototype.NOTIFICATIONS = NOTIFICATIONS;
  * Is the toolbar open?
  */
 Object.defineProperty(DeveloperToolbar.prototype, "visible", {
-  get: function() {
+  get: function () {
     return this._element && !this._element.hidden;
   },
   enumerable: true
@@ -280,7 +290,7 @@ var _gSequenceId = 0;
  * Getter for a unique ID.
  */
 Object.defineProperty(DeveloperToolbar.prototype, "sequenceId", {
-  get: function() {
+  get: function () {
     return _gSequenceId++;
   },
   enumerable: true
@@ -289,7 +299,7 @@ Object.defineProperty(DeveloperToolbar.prototype, "sequenceId", {
 /**
  * Create the <toolbar> element to insert within browser UI
  */
-DeveloperToolbar.prototype.createToolbar = function() {
+DeveloperToolbar.prototype.createToolbar = function () {
   if (this._element) {
     return;
   }
@@ -301,7 +311,8 @@ DeveloperToolbar.prototype.createToolbar = function() {
   close.setAttribute("id", "developer-toolbar-closebutton");
   close.setAttribute("class", "close-icon");
   close.setAttribute("oncommand", "DeveloperToolbar.hide();");
-  close.setAttribute("tooltiptext", "developerToolbarCloseButton.tooltiptext");
+  let closeTooltip = toolboxStrings.GetStringFromName("toolbar.closeButton.tooltip");
+  close.setAttribute("tooltiptext", closeTooltip);
 
   let stack = this._doc.createElement("stack");
   stack.setAttribute("flex", "1");
@@ -318,8 +329,14 @@ DeveloperToolbar.prototype.createToolbar = function() {
   let toolboxBtn = this._doc.createElement("toolbarbutton");
   toolboxBtn.setAttribute("id", "developer-toolbar-toolbox-button");
   toolboxBtn.setAttribute("class", "developer-toolbar-button");
-  toolboxBtn.setAttribute("observes", "devtoolsMenuBroadcaster_DevToolbox");
-  toolboxBtn.setAttribute("tooltiptext", "devToolbarToolsButton.tooltip");
+  let toolboxTooltip = toolboxStrings.GetStringFromName("toolbar.toolsButton.tooltip");
+  toolboxBtn.setAttribute("tooltiptext", toolboxTooltip);
+  toolboxBtn.addEventListener("command", function (event) {
+    let window = event.target.ownerDocument.defaultView;
+    gDevToolsBrowser.toggleToolboxCommand(window.gBrowser);
+  });
+  this._errorCounterButton = toolboxBtn;
+  this._errorCounterButton._defaultTooltipText = toolboxTooltip;
 
   // On Mac, the close button is on the left,
   // while it is on the right on every other platforms.
@@ -342,16 +359,13 @@ DeveloperToolbar.prototype.createToolbar = function() {
     if (statusBar)
       statusBar.parentNode.insertBefore(this._element, statusBar);
   }
-  this._errorCounterButton = toolboxBtn
-  this._errorCounterButton._defaultTooltipText =
-      this._errorCounterButton.getAttribute("tooltiptext");
 };
 
 /**
  * Called from browser.xul in response to menu-click or keyboard shortcut to
  * toggle the toolbar
  */
-DeveloperToolbar.prototype.toggle = function() {
+DeveloperToolbar.prototype.toggle = function () {
   if (this.visible) {
     return this.hide().catch(console.error);
   } else {
@@ -363,7 +377,7 @@ DeveloperToolbar.prototype.toggle = function() {
  * Called from browser.xul in response to menu-click or keyboard shortcut to
  * toggle the toolbar
  */
-DeveloperToolbar.prototype.focus = function() {
+DeveloperToolbar.prototype.focus = function () {
   if (this.visible) {
     this._input.focus();
     return promise.resolve();
@@ -376,13 +390,13 @@ DeveloperToolbar.prototype.focus = function() {
  * Called from browser.xul in response to menu-click or keyboard shortcut to
  * toggle the toolbar
  */
-DeveloperToolbar.prototype.focusToggle = function() {
+DeveloperToolbar.prototype.focusToggle = function () {
   if (this.visible) {
     // If we have focus then the active element is the HTML input contained
     // inside the xul input element
     let active = this._chromeWindow.document.activeElement;
     let position = this._input.compareDocumentPosition(active);
-    if (position & Node.DOCUMENT_POSITION_CONTAINED_BY) {
+    if (position & nodeConstants.DOCUMENT_POSITION_CONTAINED_BY) {
       this.hide();
     }
     else {
@@ -404,7 +418,7 @@ DeveloperToolbar.introShownThisSession = false;
 /**
  * Show the developer toolbar
  */
-DeveloperToolbar.prototype.show = function(focus) {
+DeveloperToolbar.prototype.show = function (focus) {
   if (this._showPromise != null) {
     return this._showPromise;
   }
@@ -432,7 +446,7 @@ DeveloperToolbar.prototype.show = function(focus) {
     return promise.all(panelPromises).then(panels => {
       [ this.tooltipPanel, this.outputPanel ] = panels;
 
-      this._doc.getElementById("Tools:DevToolbar").setAttribute("checked", "true");
+      this._doc.getElementById("menu_devToolbar").setAttribute("checked", "true");
 
       this.target = TargetFactory.forTab(this._chromeWindow.gBrowser.selectedTab);
       const options = {
@@ -446,10 +460,10 @@ DeveloperToolbar.prototype.show = function(focus) {
         // we fetch it early
         let value = this._input.value || "";
         return this.requisition.update(value).then(() => {
-          const Inputter = require('gcli/mozui/inputter').Inputter;
-          const Completer = require('gcli/mozui/completer').Completer;
-          const Tooltip = require('gcli/mozui/tooltip').Tooltip;
-          const FocusManager = require('gcli/ui/focus').FocusManager;
+          const Inputter = require("gcli/mozui/inputter").Inputter;
+          const Completer = require("gcli/mozui/completer").Completer;
+          const Tooltip = require("gcli/mozui/tooltip").Tooltip;
+          const FocusManager = require("gcli/ui/focus").FocusManager;
 
           this.onOutput = this.requisition.commandOutputManager.onOutput;
 
@@ -492,6 +506,9 @@ DeveloperToolbar.prototype.show = function(focus) {
           tabbrowser.addEventListener("load", this, true);
           tabbrowser.addEventListener("beforeunload", this, true);
 
+          gDevTools.on("toolbox-ready", this._onToolboxReady);
+          gDevTools.on("toolbox-destroyed", this._onToolboxDestroyed);
+
           this._initErrorsCount(tabbrowser.selectedTab);
 
           this._element.hidden = false;
@@ -520,7 +537,8 @@ DeveloperToolbar.prototype.show = function(focus) {
           if (!DeveloperToolbar.introShownThisSession) {
             let intro = require("gcli/ui/intro");
             intro.maybeShowIntro(this.requisition.commandOutputManager,
-                                 this.requisition.conversionContext);
+                                 this.requisition.conversionContext,
+                                 this.outputPanel);
             DeveloperToolbar.introShownThisSession = true;
           }
 
@@ -536,7 +554,7 @@ DeveloperToolbar.prototype.show = function(focus) {
 /**
  * Hide the developer toolbar.
  */
-DeveloperToolbar.prototype.hide = function() {
+DeveloperToolbar.prototype.hide = function () {
   // If we're already in the process of hiding, just use the other promise
   if (this._hidePromise != null) {
     return this._hidePromise;
@@ -550,7 +568,7 @@ DeveloperToolbar.prototype.hide = function() {
 
     Services.prefs.setBoolPref("devtools.toolbar.visible", false);
 
-    this._doc.getElementById("Tools:DevToolbar").setAttribute("checked", "false");
+    this._doc.getElementById("menu_devToolbar").setAttribute("checked", "false");
     this.destroy();
 
     this._telemetry.toolClosed("developertoolbar");
@@ -570,7 +588,7 @@ DeveloperToolbar.prototype.hide = function() {
  * @param nsIDOMNode tab the xul:tab for which you want to track the number of
  * errors.
  */
-DeveloperToolbar.prototype._initErrorsCount = function(tab) {
+DeveloperToolbar.prototype._initErrorsCount = function (tab) {
   let tabId = tab.linkedPanel;
   if (tabId in this._errorsCount) {
     this._updateErrorsCount();
@@ -601,7 +619,7 @@ DeveloperToolbar.prototype._initErrorsCount = function(tab) {
  * @param nsIDOMNode tab the xul:tab for which you want to stop tracking the
  * number of errors.
  */
-DeveloperToolbar.prototype._stopErrorsCount = function(tab) {
+DeveloperToolbar.prototype._stopErrorsCount = function (tab) {
   let tabId = tab.linkedPanel;
   if (!(tabId in this._errorsCount) || !(tabId in this._warningsCount)) {
     this._updateErrorsCount();
@@ -619,7 +637,7 @@ DeveloperToolbar.prototype._stopErrorsCount = function(tab) {
 /**
  * Hide the developer toolbar
  */
-DeveloperToolbar.prototype.destroy = function() {
+DeveloperToolbar.prototype.destroy = function () {
   if (this._input == null) {
     return; // Already destroyed
   }
@@ -630,6 +648,9 @@ DeveloperToolbar.prototype.destroy = function() {
   tabbrowser.removeEventListener("load", this, true);
   tabbrowser.removeEventListener("beforeunload", this, true);
 
+  gDevTools.off("toolbox-ready", this._onToolboxReady);
+  gDevTools.off("toolbox-destroyed", this._onToolboxDestroyed);
+
   Array.prototype.forEach.call(tabbrowser.tabs, this._stopErrorsCount, this);
 
   this.focusManager.removeMonitoredElement(this.outputPanel._frame);
@@ -661,7 +682,7 @@ DeveloperToolbar.prototype.destroy = function() {
  * Utility for sending notifications
  * @param topic a NOTIFICATION constant
  */
-DeveloperToolbar.prototype._notify = function(topic) {
+DeveloperToolbar.prototype._notify = function (topic) {
   let data = { toolbar: this };
   data.wrappedJSObject = data;
   Services.obs.notifyObservers(data, topic, null);
@@ -670,7 +691,7 @@ DeveloperToolbar.prototype._notify = function(topic) {
 /**
  * Update various parts of the UI when the current tab changes
  */
-DeveloperToolbar.prototype.handleEvent = function(ev) {
+DeveloperToolbar.prototype.handleEvent = function (ev) {
   if (ev.type == "TabSelect" || ev.type == "load") {
     if (this.visible) {
       let tab = this._chromeWindow.gBrowser.selectedTab;
@@ -704,6 +725,16 @@ DeveloperToolbar.prototype.handleEvent = function(ev) {
   }
 };
 
+/**
+ * Update toolbox toggle button when toolbox goes on and off
+ */
+DeveloperToolbar.prototype._onToolboxReady = function () {
+  this._errorCounterButton.setAttribute("checked", "true");
+};
+DeveloperToolbar.prototype._onToolboxDestroyed = function () {
+  this._errorCounterButton.setAttribute("checked", "false");
+};
+
 /**
  * Count a page error received for the currently selected tab. This
  * method counts the JavaScript exceptions received and CSS errors/warnings.
@@ -713,7 +744,7 @@ DeveloperToolbar.prototype.handleEvent = function(ev) {
  * @param object pageError the page error object received from the
  * PageErrorListener.
  */
-DeveloperToolbar.prototype._onPageError = function(tabId, pageError) {
+DeveloperToolbar.prototype._onPageError = function (tabId, pageError) {
   if (pageError.category == "CSS Parser" ||
       pageError.category == "CSS Loader") {
     return;
@@ -734,14 +765,14 @@ DeveloperToolbar.prototype._onPageError = function(tabId, pageError) {
  * @private
  * @param nsIDOMEvent ev the beforeunload DOM event.
  */
-DeveloperToolbar.prototype._onPageBeforeUnload = function(ev) {
+DeveloperToolbar.prototype._onPageBeforeUnload = function (ev) {
   let window = ev.target.defaultView;
   if (window.top !== window) {
     return;
   }
 
   let tabs = this._chromeWindow.gBrowser.tabs;
-  Array.prototype.some.call(tabs, function(tab) {
+  Array.prototype.some.call(tabs, function (tab) {
     if (tab.linkedBrowser.contentWindow === window) {
       let tabId = tab.linkedPanel;
       if (tabId in this._errorsCount || tabId in this._warningsCount) {
@@ -764,7 +795,7 @@ DeveloperToolbar.prototype._onPageBeforeUnload = function(ev) {
  * count changed. If this is provided and it doesn't match the currently
  * selected tab, then the button is not updated.
  */
-DeveloperToolbar.prototype._updateErrorsCount = function(changedTabId) {
+DeveloperToolbar.prototype._updateErrorsCount = function (changedTabId) {
   let tabId = this._chromeWindow.gBrowser.selectedTab.linkedPanel;
   if (changedTabId && tabId != changedTabId) {
     return;
@@ -802,7 +833,7 @@ DeveloperToolbar.prototype._updateErrorsCount = function(changedTabId) {
  * @param nsIDOMElement tab The xul:tab for which you want to reset the page
  * errors counters.
  */
-DeveloperToolbar.prototype.resetErrorsCount = function(tab) {
+DeveloperToolbar.prototype.resetErrorsCount = function (tab) {
   let tabId = tab.linkedPanel;
   if (tabId in this._errorsCount || tabId in this._warningsCount) {
     this._errorsCount[tabId] = 0;
@@ -834,7 +865,7 @@ function OutputPanel() {
  *
  * @param devtoolbar The parent DeveloperToolbar object
  */
-OutputPanel.create = function(devtoolbar) {
+OutputPanel.create = function (devtoolbar) {
   var outputPanel = Object.create(OutputPanel.prototype);
   return outputPanel._init(devtoolbar);
 };
@@ -842,7 +873,7 @@ OutputPanel.create = function(devtoolbar) {
 /**
  * @private See OutputPanel.create
  */
-OutputPanel.prototype._init = function(devtoolbar) {
+OutputPanel.prototype._init = function (devtoolbar) {
   this._devtoolbar = devtoolbar;
   this._input = this._devtoolbar._input;
   this._toolbar = this._devtoolbar._doc.getElementById("developer-toolbar");
@@ -894,7 +925,7 @@ OutputPanel.prototype._init = function(devtoolbar) {
   this._update = this._update.bind(this);
 
   // Wire up the element from the iframe, and resolve the promise
-  let deferred = promise.defer();
+  let deferred = defer();
   let onload = () => {
     this._frame.removeEventListener("load", onload, true);
 
@@ -914,11 +945,11 @@ OutputPanel.prototype._init = function(devtoolbar) {
   this._frame.addEventListener("load", onload, true);
 
   return deferred.promise;
-}
+};
 
 /* Copy the current devtools theme attribute into the iframe,
    so it can be styled correctly. */
-OutputPanel.prototype._copyTheme = function() {
+OutputPanel.prototype._copyTheme = function () {
   if (this.document) {
     let theme =
       this._devtoolbar._doc.documentElement.getAttribute("devtoolstheme");
@@ -929,7 +960,7 @@ OutputPanel.prototype._copyTheme = function() {
 /**
  * Prevent the popup from hiding if it is not permitted via this.canHide.
  */
-OutputPanel.prototype._onpopuphiding = function(ev) {
+OutputPanel.prototype._onpopuphiding = function (ev) {
   // TODO: When we switch back from tooltip to panel we can remove this hack:
   // https://bugzilla.mozilla.org/show_bug.cgi?id=780102
   if (isLinux && !this.canHide) {
@@ -940,7 +971,7 @@ OutputPanel.prototype._onpopuphiding = function(ev) {
 /**
  * Display the OutputPanel.
  */
-OutputPanel.prototype.show = function() {
+OutputPanel.prototype.show = function () {
   if (isLinux) {
     this.canHide = false;
   }
@@ -961,9 +992,9 @@ OutputPanel.prototype.show = function() {
  * Internal helper to set the height of the output panel to fit the available
  * content;
  */
-OutputPanel.prototype._resize = function() {
+OutputPanel.prototype._resize = function () {
   if (this._panel == null || this.document == null || !this._panel.state == "closed") {
-    return
+    return;
   }
 
   // Set max panel width to match any content with a max of the width of the
@@ -974,7 +1005,7 @@ OutputPanel.prototype._resize = function() {
   // We'd like to put this in CSS but we can't:
   //   body { width: calc(min(-5px, max-content)); }
   //   #_panel { max-width: -5px; }
-  switch(OS) {
+  switch (Services.appinfo.OS) {
     case "Linux":
       maxWidth -= 5;
       break;
@@ -1022,7 +1053,7 @@ OutputPanel.prototype._resize = function() {
 /**
  * Called by GCLI when a command is executed.
  */
-OutputPanel.prototype._outputChanged = function(ev) {
+OutputPanel.prototype._outputChanged = function (ev) {
   if (ev.output.hidden) {
     return;
   }
@@ -1044,7 +1075,7 @@ OutputPanel.prototype._outputChanged = function(ev) {
  * Called when displayed Output says it's changed or from outputChanged, which
  * happens when there is a new displayed Output.
  */
-OutputPanel.prototype._update = function() {
+OutputPanel.prototype._update = function () {
   // destroy has been called, bail out
   if (this._div == null) {
     return;
@@ -1080,7 +1111,7 @@ OutputPanel.prototype._update = function() {
 /**
  * Detach listeners from the currently displayed Output.
  */
-OutputPanel.prototype.remove = function() {
+OutputPanel.prototype.remove = function () {
   if (isLinux) {
     this.canHide = true;
   }
@@ -1097,7 +1128,7 @@ OutputPanel.prototype.remove = function() {
 /**
  * Detach listeners from the currently displayed Output.
  */
-OutputPanel.prototype.destroy = function() {
+OutputPanel.prototype.destroy = function () {
   this.remove();
 
   this._panel.removeEventListener("popuphiding", this._onpopuphiding, true);
@@ -1120,7 +1151,7 @@ OutputPanel.prototype.destroy = function() {
  * Called by GCLI to indicate that we should show or hide one either the
  * tooltip panel or the output panel.
  */
-OutputPanel.prototype._visibilityChanged = function(ev) {
+OutputPanel.prototype._visibilityChanged = function (ev) {
   if (ev.outputVisible === true) {
     // this.show is called by _outputChanged
   } else {
@@ -1154,7 +1185,7 @@ function TooltipPanel() {
  *
  * @param devtoolbar The parent DeveloperToolbar object
  */
-TooltipPanel.create = function(devtoolbar) {
+TooltipPanel.create = function (devtoolbar) {
   var tooltipPanel = Object.create(TooltipPanel.prototype);
   return tooltipPanel._init(devtoolbar);
 };
@@ -1162,8 +1193,8 @@ TooltipPanel.create = function(devtoolbar) {
 /**
  * @private See TooltipPanel.create
  */
-TooltipPanel.prototype._init = function(devtoolbar) {
-  let deferred = promise.defer();
+TooltipPanel.prototype._init = function (devtoolbar) {
+  let deferred = defer();
 
   let chromeDocument = devtoolbar._doc;
   this._devtoolbar = devtoolbar;
@@ -1240,7 +1271,7 @@ TooltipPanel.prototype._init = function(devtoolbar) {
 
 /* Copy the current devtools theme attribute into the iframe,
    so it can be styled correctly. */
-TooltipPanel.prototype._copyTheme = function() {
+TooltipPanel.prototype._copyTheme = function () {
   if (this.document) {
     let theme =
       this._devtoolbar._doc.documentElement.getAttribute("devtoolstheme");
@@ -1251,7 +1282,7 @@ TooltipPanel.prototype._copyTheme = function() {
 /**
  * Prevent the popup from hiding if it is not permitted via this.canHide.
  */
-TooltipPanel.prototype._onpopuphiding = function(ev) {
+TooltipPanel.prototype._onpopuphiding = function (ev) {
   // TODO: When we switch back from tooltip to panel we can remove this hack:
   // https://bugzilla.mozilla.org/show_bug.cgi?id=780102
   if (isLinux && !this.canHide) {
@@ -1262,7 +1293,7 @@ TooltipPanel.prototype._onpopuphiding = function(ev) {
 /**
  * Display the TooltipPanel.
  */
-TooltipPanel.prototype.show = function(dimensions) {
+TooltipPanel.prototype.show = function (dimensions) {
   if (!dimensions) {
     dimensions = { start: 0, end: 0 };
   }
@@ -1298,9 +1329,9 @@ const AVE_CHAR_WIDTH = 4.5;
 /**
  * Display the TooltipPanel.
  */
-TooltipPanel.prototype._resize = function() {
+TooltipPanel.prototype._resize = function () {
   if (this._panel == null || this.document == null || !this._panel.state == "closed") {
-    return
+    return;
   }
 
   let offset = 10 + Math.floor(this._dimensions.start * AVE_CHAR_WIDTH);
@@ -1321,7 +1352,7 @@ TooltipPanel.prototype._resize = function() {
 /**
  * Hide the TooltipPanel.
  */
-TooltipPanel.prototype.remove = function() {
+TooltipPanel.prototype.remove = function () {
   if (isLinux) {
     this.canHide = true;
   }
@@ -1333,7 +1364,7 @@ TooltipPanel.prototype.remove = function() {
 /**
  * Hide the TooltipPanel.
  */
-TooltipPanel.prototype.destroy = function() {
+TooltipPanel.prototype.destroy = function () {
   this.remove();
 
   this._panel.removeEventListener("popuphiding", this._onpopuphiding, true);
@@ -1357,7 +1388,7 @@ TooltipPanel.prototype.destroy = function() {
  * Called by GCLI to indicate that we should show or hide one either the
  * tooltip panel or the output panel.
  */
-TooltipPanel.prototype._visibilityChanged = function(ev) {
+TooltipPanel.prototype._visibilityChanged = function (ev) {
   if (ev.tooltipVisible === true) {
     this.show(ev.dimensions);
   } else {
