@@ -6,16 +6,16 @@
 
 // Tests that the CubicBezierWidget generates content in a given parent node
 
-const TEST_URI = "chrome://devtools/content/shared/widgets/cubic-bezier-frame.xhtml";
 const {CubicBezierWidget} =
   require("devtools/client/shared/widgets/CubicBezierWidget");
 
-add_task(function*() {
-  yield addTab("about:blank");
+const TEST_URI = `data:text/html,<div id="cubic-bezier-container" />`;
+
+add_task(function* () {
   let [host, win, doc] = yield createHost("bottom", TEST_URI);
 
   info("Checking that the graph markup is created in the parent");
-  let container = doc.querySelector("#container");
+  let container = doc.querySelector("#cubic-bezier-container");
   let w = new CubicBezierWidget(container);
 
   ok(container.querySelector(".display-wrap"),
@@ -27,9 +27,7 @@ add_task(function*() {
   is(buttons.length, 2,
     "The 2 control points have been added");
   is(buttons[0].className, "control-point");
-  is(buttons[0].id, "P1");
   is(buttons[1].className, "control-point");
-  is(buttons[1].id, "P2");
   ok(container.querySelector("canvas"), "The curve canvas has been added");
 
   info("Destroying the widget");
@@ -37,5 +35,4 @@ add_task(function*() {
   is(container.children.length, 0, "All nodes have been removed");
 
   host.destroy();
-  gBrowser.removeCurrentTab();
 });
