@@ -8,20 +8,28 @@
 
 var { Cc, Ci, Cu, Cr } = require("chrome");
 const {getRect, getElementFromPoint, getAdjustedQuads} = require("devtools/shared/layout/utils");
-const promise = require("promise");
-const {Task} = Cu.import("resource://gre/modules/Task.jsm", {});
+const defer = require("devtools/shared/defer");
+const {Task} = require("devtools/shared/task");
 var DOMUtils = Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
 var loader = Cc["@mozilla.org/moz/jssubscript-loader;1"]
             .getService(Ci.mozIJSSubScriptLoader);
-let EventUtils = {};
-loader.loadSubScript("chrome://marionette/content/EventUtils.js", EventUtils);
 
-const protocol = require("devtools/server/protocol");
+// Set up a dummy environment so that EventUtils works. We need to be careful to
+// pass a window object into each EventUtils method we call rather than having
+// it rely on the |window| global.
+let EventUtils = {};
+EventUtils.window = {};
+EventUtils.parent = {};
+EventUtils._EU_Ci = Components.interfaces;
+EventUtils._EU_Cc = Components.classes;
+loader.loadSubScript("chrome://mochikit/content/tests/SimpleTest/EventUtils.js", EventUtils);
+
+const protocol = require("devtools/shared/protocol");
 const {Arg, Option, method, RetVal, types} = protocol;
 
 var dumpn = msg => {
   dump(msg + "\n");
-}
+};
 
 /**
  * Get the instance of CanvasFrameAnonymousContentHelper used by a given
@@ -41,7 +49,7 @@ function getHighlighterCanvasFrameHelper(conn, actorID) {
 var TestActor = exports.TestActor = protocol.ActorClass({
   typeName: "testActor",
 
-  initialize: function(conn, tabActor, options) {
+  initialize: function (conn, tabActor, options) {
     this.conn = conn;
     this.tabActor = tabActor;
   },
@@ -61,7 +69,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
     let document = this.content.document;
     if (Array.isArray(selector)) {
       let fullSelector = selector.join(" >> ");
-      while(selector.length > 1) {
+      while (selector.length > 1) {
         let str = selector.shift();
         let iframe = document.querySelector(str);
         if (!iframe) {
@@ -82,6 +90,21 @@ var TestActor = exports.TestActor = protocol.ActorClass({
     }
     return node;
   },
+  /**
+   * Helper to get the number of elements matching a selector
+   * @param {string} CSS selector.
+   */
+  getNumberOfElementMatches: protocol.method(function (selector,
+                                                       root = this.content.document) {
+    return root.querySelectorAll(selector).length;
+  }, {
+    request: {
+      selector: Arg(0, "string"),
+    },
+    response: {
+      value: RetVal("number")
+    }
+  }),
 
   /**
    * Get a value for a given attribute name, on one of the elements of the box
@@ -164,7 +187,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
    * @param {String} actorID The highlighter actor ID
    */
   changeHighlightedNodeWaitForUpdate: protocol.method(function (name, value, actorID) {
-    let deferred = promise.defer();
+    let deferred = defer();
 
     let highlighter = this.conn.getActor(actorID);
     let {_highlighter: h} = highlighter;
@@ -234,7 +257,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
    */
   changeZoomLevel: protocol.method(function (level, actorID) {
     dumpn("Zooming page to " + level);
-    let deferred = promise.defer();
+    let deferred = defer();
 
     if (actorID) {
       let actor = this.conn.getActor(actorID);
@@ -285,7 +308,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
    * @return {Object} An object with each property being a box-model region, each
    * of them being an object with the p1/p2/p3/p4 properties
    */
-  getAllAdjustedQuads: protocol.method(function(selector) {
+  getAllAdjustedQuads: protocol.method(function (selector) {
     let regions = {};
     let node = this._querySelector(selector);
     for (let boxType of ["content", "padding", "border", "margin"]) {
@@ -303,9 +326,10 @@ var TestActor = exports.TestActor = protocol.ActorClass({
   }),
 
   /**
-   * Synthesize a mouse event on an element. This handler doesn't send a message
-   * back. Consumers should listen to specific events on the inspector/highlighter
-   * to know when the event got synthesized.
+   * Synthesize a mouse event on an element, after ensuring that it is visible
+   * in the viewport. This handler doesn't send a message back. Consumers
+   * should listen to specific events on the inspector/highlighter to know when
+   * the event got synthesized.
    * @param {String} selector The node selector to get the node target for the event
    * @param {Number} x
    * @param {Number} y
@@ -313,9 +337,9 @@ var TestActor = exports.TestActor = protocol.ActorClass({
    *                  synthesizeMouseAtCenter will be used instead
    * @param {Object} options Other event options
    */
-  synthesizeMouse: protocol.method(function({ selector, x, y, center, options }) {
+  synthesizeMouse: protocol.method(function ({ selector, x, y, center, options }) {
     let node = this._querySelector(selector);
-
+    node.scrollIntoView();
     if (center) {
       EventUtils.synthesizeMouseAtCenter(node, options, node.ownerDocument.defaultView);
     } else {
@@ -362,7 +386,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
   }),
 
   loadAndWaitForCustomEvent: protocol.method(function (url) {
-    let deferred = promise.defer();
+    let deferred = defer();
     let self = this;
     // Wait for DOMWindowCreated first, as listening on the current outerwindow
     // doesn't allow receiving test-page-processing-done.
@@ -388,7 +412,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
       // _querySelector throws if the node doesn't exists
       this._querySelector(selector);
       return true;
-    } catch(e) {
+    } catch (e) {
       return false;
     }
   }, {
@@ -524,7 +548,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
   reloadFrame: protocol.method(function (selector) {
     let node = this._querySelector(selector);
 
-    let deferred = promise.defer();
+    let deferred = defer();
 
     let onLoad = function () {
       node.removeEventListener("load", onLoad);
@@ -575,7 +599,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
       return {};
     }
 
-    let deferred = promise.defer();
+    let deferred = defer();
     this.content.addEventListener("scroll", function onScroll(event) {
       this.removeEventListener("scroll", onScroll);
 
@@ -597,6 +621,17 @@ var TestActor = exports.TestActor = protocol.ActorClass({
     }
   }),
 
+  /**
+   * Forces the reflow and waits for the next repaint.
+   */
+  reflow: protocol.method(function () {
+    let deferred = defer();
+    this.content.document.documentElement.offsetWidth;
+    this.content.requestAnimationFrame(deferred.resolve);
+
+    return deferred.promise;
+  }),
+
   getNodeRect: protocol.method(Task.async(function* (selector) {
     let node = this._querySelector(selector);
     return getRect(this.content, node, this.content);
@@ -622,7 +657,7 @@ var TestActor = exports.TestActor = protocol.ActorClass({
    * - {String} innerHTML.
    * - {String} textContent.
    */
-  getNodeInfo: protocol.method(function(selector) {
+  getNodeInfo: protocol.method(function (selector) {
     let node = this._querySelector(selector);
     let info = null;
 
@@ -651,8 +686,8 @@ var TestActor = exports.TestActor = protocol.ActorClass({
   })
 });
 
-const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
-  initialize: function(client, { testActor }, toolbox) {
+var TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
+  initialize: function (client, { testActor }, toolbox) {
     protocol.Front.prototype.initialize.call(this, client, { actor: testActor });
     this.manage(this);
     this.toolbox = toolbox;
@@ -664,11 +699,11 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * @return {Promise} The returned promise will only resolve when the
    * highlighter has updated to the new zoom level.
    */
-  zoomPageTo: function(level) {
+  zoomPageTo: function (level) {
     return this.changeZoomLevel(level, this.toolbox.highlighter.actorID);
   },
 
-  changeHighlightedNodeWaitForUpdate: protocol.custom(function(name, value, highlighter) {
+  changeHighlightedNodeWaitForUpdate: protocol.custom(function (name, value, highlighter) {
     return this._changeHighlightedNodeWaitForUpdate(name, value, (highlighter || this.toolbox.highlighter).actorID);
   }, {
     impl: "_changeHighlightedNodeWaitForUpdate"
@@ -681,11 +716,11 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * @param {Object} highlighter Optional custom highlither to target
    * @return {String} value
    */
-  getHighlighterNodeAttribute: function(nodeID, name, highlighter) {
+  getHighlighterNodeAttribute: function (nodeID, name, highlighter) {
     return this.getHighlighterAttribute(nodeID, name, (highlighter || this.toolbox.highlighter).actorID);
   },
 
-  getHighlighterNodeTextContent: protocol.custom(function(nodeID, highlighter) {
+  getHighlighterNodeTextContent: protocol.custom(function (nodeID, highlighter) {
     return this._getHighlighterNodeTextContent(nodeID, (highlighter || this.toolbox.highlighter).actorID);
   }, {
     impl: "_getHighlighterNodeTextContent"
@@ -694,7 +729,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
   /**
    * Is the highlighter currently visible on the page?
    */
-  isHighlighting: function() {
+  isHighlighting: function () {
     return this.getHighlighterNodeAttribute("box-model-elements", "hidden")
       .then(value => value === null);
   },
@@ -707,7 +742,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * @param {String} prefix An optional prefix for logging information to the
    * console.
    */
-  isNodeCorrectlyHighlighted: Task.async(function*(selector, is, prefix="") {
+  isNodeCorrectlyHighlighted: Task.async(function* (selector, is, prefix = "") {
     prefix += (prefix ? " " : "") + selector + " ";
 
     let boxModel = yield this._getBoxModelStatus();
@@ -727,7 +762,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
   /**
    * Get the current rect of the border region of the box-model highlighter
    */
-  getSimpleBorderRect: Task.async(function*(toolbox) {
+  getSimpleBorderRect: Task.async(function* (toolbox) {
     let {border} = yield this._getBoxModelStatus(toolbox);
     let {p1, p2, p3, p4} = border.points;
 
@@ -743,7 +778,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * Get the current positions and visibility of the various box-model highlighter
    * elements.
    */
-  _getBoxModelStatus: Task.async(function*() {
+  _getBoxModelStatus: Task.async(function* () {
     let isVisible = yield this.isHighlighting();
 
     let ret = {
@@ -768,8 +803,8 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
     // Taken and tweaked from:
     // https://github.com/iominh/point-in-polygon-extended/blob/master/src/index.js#L30-L85
     function isLeft(p0, p1, p2) {
-      let l = ( (p1[0] - p0[0]) * (p2[1] - p0[1]) ) -
-              ( (p2[0] - p0[0]) * (p1[1] - p0[1]) );
+      let l = ((p1[0] - p0[0]) * (p2[1] - p0[1])) -
+              ((p2[0] - p0[0]) * (p1[1] - p0[1]));
       return l;
     }
     function isInside(point, polygon) {
@@ -816,7 +851,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
 
       // Converts points dictionnary into an array
       let list = [];
-      for(var i = 1; i <= 4; i++) {
+      for (var i = 1; i <= 4; i++) {
         let p = points["p" + i];
         list.push([p.x, p.y]);
       }
@@ -839,7 +874,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * Get the coordinate (points attribute) from one of the polygon elements in the
    * box model highlighter.
    */
-  _getPointsForRegion: Task.async(function*(region) {
+  _getPointsForRegion: Task.async(function* (region) {
     let d = yield this.getHighlighterNodeAttribute("box-model-" + region, "d");
 
     let polygons = d.match(/M[^M]+/g);
@@ -848,7 +883,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
     }
 
     let points = polygons[0].trim().split(" ").map(i => {
-      return i.replace(/M|L/, "").split(",")
+      return i.replace(/M|L/, "").split(",");
     });
 
     return {
@@ -875,12 +910,12 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * Is a given region polygon element of the box-model highlighter currently
    * hidden?
    */
-  _isRegionHidden: Task.async(function*(region) {
+  _isRegionHidden: Task.async(function* (region) {
     let value = yield this.getHighlighterNodeAttribute("box-model-" + region, "hidden");
     return value !== null;
   }),
 
-  _getGuideStatus: Task.async(function*(location) {
+  _getGuideStatus: Task.async(function* (location) {
     let id = "box-model-guide-" + location;
 
     let hidden = yield this.getHighlighterNodeAttribute(id, "hidden");
@@ -904,7 +939,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * @return {Object} Null if at least one guide is hidden. Otherwise an object
    * with p1, p2, p3, p4 properties being {x, y} objects.
    */
-  getGuidesRectangle: Task.async(function*() {
+  getGuidesRectangle: Task.async(function* () {
     let tGuide = yield this._getGuideStatus("top");
     let rGuide = yield this._getGuideStatus("right");
     let bGuide = yield this._getGuideStatus("bottom");
@@ -922,7 +957,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
     };
   }),
 
-  waitForHighlighterEvent: protocol.custom(function(event) {
+  waitForHighlighterEvent: protocol.custom(function (event) {
     return this._waitForHighlighterEvent(event, this.toolbox.highlighter.actorID);
   }, {
     impl: "_waitForHighlighterEvent"
@@ -938,7 +973,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
    * - points {Array} an array of all the polygons defined by the path. Each box
    *   is itself an Array of points, themselves being [x,y] coordinates arrays.
    */
-  getHighlighterRegionPath: Task.async(function*(region, highlighter) {
+  getHighlighterRegionPath: Task.async(function* (region, highlighter) {
     let d = yield this.getHighlighterNodeAttribute("box-model-" + region, "d", highlighter);
     if (!d) {
       return {d: null};
@@ -952,7 +987,7 @@ const TestActorFront = exports.TestActorFront = protocol.FrontClass(TestActor, {
     let points = [];
     for (let polygon of polygons) {
       points.push(polygon.trim().split(" ").map(i => {
-        return i.replace(/M|L/, "").split(",")
+        return i.replace(/M|L/, "").split(",");
       }));
     }
 
