@@ -26,12 +26,13 @@
 "use strict";
 
 const EventEmitter = require("devtools/shared/event-emitter");
-const {setTimeout, clearTimeout} = require("sdk/timers");
-const {PREDEFINED, PRESETS, DEFAULT_PRESET_CATEGORY} = require("devtools/client/shared/widgets/CubicBezierPresets");
-const {Cc, Ci} = require('chrome');
-loader.lazyGetter(this, "DOMUtils", () => {
-  return Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
-});
+const {
+  PREDEFINED,
+  PRESETS,
+  DEFAULT_PRESET_CATEGORY
+} = require("devtools/client/shared/widgets/CubicBezierPresets");
+const {getCSSLexer} = require("devtools/shared/css-lexer");
+const XHTML_NS = "http://www.w3.org/1999/xhtml";
 
 /**
  * CubicBezier data structure helper
@@ -40,21 +41,21 @@ loader.lazyGetter(this, "DOMUtils", () => {
  */
 function CubicBezier(coordinates) {
   if (!coordinates) {
-    throw "No offsets were defined";
+    throw new Error("No offsets were defined");
   }
 
   this.coordinates = coordinates.map(n => +n);
 
   for (let i = 4; i--;) {
     let xy = this.coordinates[i];
-    if (isNaN(xy) || (!(i%2) && (xy < 0 || xy > 1))) {
-      throw "Wrong coordinate at " + i + "(" + xy + ")";
+    if (isNaN(xy) || (!(i % 2) && (xy < 0 || xy > 1))) {
+      throw new Error(`Wrong coordinate at ${i}(${xy})`);
     }
   }
 
-  this.coordinates.toString = function() {
+  this.coordinates.toString = function () {
     return this.map(n => {
-      return (Math.round(n * 100)/100 + '').replace(/^0\./, '.');
+      return (Math.round(n * 100) / 100 + "").replace(/^0\./, ".");
     }) + "";
   };
 }
@@ -70,12 +71,13 @@ CubicBezier.prototype = {
     return this.coordinates.slice(2);
   },
 
-  toString: function() {
+  toString: function () {
     // Check first if current coords are one of css predefined functions
     let predefName = Object.keys(PREDEFINED)
-                           .find(key => coordsAreEqual(PREDEFINED[key], this.coordinates));
+                           .find(key => coordsAreEqual(PREDEFINED[key],
+                                                       this.coordinates));
 
-    return predefName || 'cubic-bezier(' + this.coordinates + ')';
+    return predefName || "cubic-bezier(" + this.coordinates + ")";
   }
 };
 
@@ -91,7 +93,7 @@ function BezierCanvas(canvas, bezier, padding) {
   this.padding = getPadding(padding);
 
   // Convert to a cartesian coordinate system with axes from 0 to 1
-  this.ctx = this.canvas.getContext('2d');
+  this.ctx = this.canvas.getContext("2d");
   let p = this.padding;
 
   this.ctx.scale(canvas.width * (1 - p[1] - p[3]),
@@ -111,22 +113,24 @@ BezierCanvas.prototype = {
     let p = this.padding, w = this.canvas.width, h = this.canvas.height;
 
     return [{
-      left: w * (this.bezier.coordinates[0] * (1 - p[3] - p[1]) - p[3]) + 'px',
-      top: h * (1 - this.bezier.coordinates[1] * (1 - p[0] - p[2]) - p[0]) + 'px'
+      left: w * (this.bezier.coordinates[0] * (1 - p[3] - p[1]) - p[3]) + "px",
+      top: h * (1 - this.bezier.coordinates[1] * (1 - p[0] - p[2]) - p[0])
+           + "px"
     }, {
-      left: w * (this.bezier.coordinates[2] * (1 - p[3] - p[1]) - p[3]) + 'px',
-      top: h * (1 - this.bezier.coordinates[3] * (1 - p[0] - p[2]) - p[0]) + 'px'
+      left: w * (this.bezier.coordinates[2] * (1 - p[3] - p[1]) - p[3]) + "px",
+      top: h * (1 - this.bezier.coordinates[3] * (1 - p[0] - p[2]) - p[0])
+           + "px"
     }];
   },
 
   /**
    * Convert an element's left/top offsets into coordinates
    */
-  offsetsToCoordinates: function(element) {
+  offsetsToCoordinates: function (element) {
     let p = this.padding, w = this.canvas.width, h = this.canvas.height;
 
     // Convert padding percentage to actual padding
-    p = p.map(function(a, i) { return a * (i % 2? w : h)});
+    p = p.map((a, i) => a * (i % 2 ? w : h));
 
     return [
       (parseFloat(element.style.left) - p[3]) / (w + p[1] + p[3]),
@@ -137,13 +141,13 @@ BezierCanvas.prototype = {
   /**
    * Draw the cubic bezier curve for the current coordinates
    */
-  plot: function(settings={}) {
+  plot: function (settings = {}) {
     let xy = this.bezier.coordinates;
 
     let defaultSettings = {
-      handleColor: '#666',
+      handleColor: "#666",
       handleThickness: .008,
-      bezierColor: '#4C9ED9',
+      bezierColor: "#4C9ED9",
       bezierThickness: .015,
       drawHandles: true
     };
@@ -168,15 +172,15 @@ BezierCanvas.prototype = {
 
       this.ctx.moveTo(0, 0);
       this.ctx.lineTo(xy[0], xy[1]);
-      this.ctx.moveTo(1,1);
+      this.ctx.moveTo(1, 1);
       this.ctx.lineTo(xy[2], xy[3]);
 
       this.ctx.stroke();
       this.ctx.closePath();
 
-      var circle = function(ctx, cx, cy, r) {
+      let circle = (ctx, cx, cy, r) => {
         ctx.beginPath();
-        ctx.arc(cx, cy, r, 0, 2*Math.PI, !1);
+        ctx.arc(cx, cy, r, 0, 2 * Math.PI, !1);
         ctx.closePath();
       };
 
@@ -190,8 +194,8 @@ BezierCanvas.prototype = {
     this.ctx.beginPath();
     this.ctx.lineWidth = defaultSettings.bezierThickness;
     this.ctx.strokeStyle = defaultSettings.bezierColor;
-    this.ctx.moveTo(0,0);
-    this.ctx.bezierCurveTo(xy[0], xy[1], xy[2], xy[3], 1,1);
+    this.ctx.moveTo(0, 0);
+    this.ctx.bezierCurveTo(xy[0], xy[1], xy[2], xy[3], 1, 1);
     this.ctx.stroke();
     this.ctx.closePath();
   }
@@ -206,7 +210,8 @@ BezierCanvas.prototype = {
  * Emits "updated" events whenever the curve is changed. Along with the event is
  * sent a CubicBezier object
  */
-function CubicBezierWidget(parent, coordinates=PRESETS["ease-in"]["ease-in-sine"]) {
+function CubicBezierWidget(parent,
+                           coordinates = PRESETS["ease-in"]["ease-in-sine"]) {
   EventEmitter.decorate(this);
 
   this.parent = parent;
@@ -246,29 +251,27 @@ function CubicBezierWidget(parent, coordinates=PRESETS["ease-in"]["ease-in-sine"
 exports.CubicBezierWidget = CubicBezierWidget;
 
 CubicBezierWidget.prototype = {
-  _initMarkup: function() {
+  _initMarkup: function () {
     let doc = this.parent.ownerDocument;
 
-    let wrap = doc.createElement("div");
+    let wrap = doc.createElementNS(XHTML_NS, "div");
     wrap.className = "display-wrap";
 
-    let plane = doc.createElement("div");
+    let plane = doc.createElementNS(XHTML_NS, "div");
     plane.className = "coordinate-plane";
 
-    let p1 = doc.createElement("button");
+    let p1 = doc.createElementNS(XHTML_NS, "button");
     p1.className = "control-point";
-    p1.id = "P1";
     plane.appendChild(p1);
 
-    let p2 = doc.createElement("button");
+    let p2 = doc.createElementNS(XHTML_NS, "button");
     p2.className = "control-point";
-    p2.id = "P2";
     plane.appendChild(p2);
 
-    let curve = doc.createElement("canvas");
+    let curve = doc.createElementNS(XHTML_NS, "canvas");
     curve.setAttribute("width", 150);
     curve.setAttribute("height", 370);
-    curve.id = "curve";
+    curve.className = "curve";
 
     plane.appendChild(curve);
     wrap.appendChild(plane);
@@ -276,17 +279,17 @@ CubicBezierWidget.prototype = {
     this.parent.appendChild(wrap);
 
     return {
-      p1: p1,
-      p2: p2,
-      curve: curve
+      p1,
+      p2,
+      curve
     };
   },
 
-  _removeMarkup: function() {
-    this.parent.ownerDocument.querySelector(".display-wrap").remove();
+  _removeMarkup: function () {
+    this.parent.querySelector(".display-wrap").remove();
   },
 
-  _initEvents: function() {
+  _initEvents: function () {
     this.p1.addEventListener("mousedown", this._onPointMouseDown);
     this.p2.addEventListener("mousedown", this._onPointMouseDown);
 
@@ -298,7 +301,7 @@ CubicBezierWidget.prototype = {
     this.presets.on("new-coordinates", this._onNewCoordinates);
   },
 
-  _removeEvents: function() {
+  _removeEvents: function () {
     this.p1.removeEventListener("mousedown", this._onPointMouseDown);
     this.p2.removeEventListener("mousedown", this._onPointMouseDown);
 
@@ -310,7 +313,7 @@ CubicBezierWidget.prototype = {
     this.presets.off("new-coordinates", this._onNewCoordinates);
   },
 
-  _onPointMouseDown: function(event) {
+  _onPointMouseDown: function (event) {
     // Updating the boundingbox in case it has changed
     this.curveBoundingBox = this.curve.getBoundingClientRect();
 
@@ -343,7 +346,7 @@ CubicBezierWidget.prototype = {
     };
   },
 
-  _onPointKeyDown: function(event) {
+  _onPointKeyDown: function (event) {
     let point = event.target;
     let code = event.keyCode;
 
@@ -351,22 +354,22 @@ CubicBezierWidget.prototype = {
       event.preventDefault();
 
       // Arrow keys pressed
-      let left = parseInt(point.style.left);
-      let top = parseInt(point.style.top);
+      let left = parseInt(point.style.left, 10);
+      let top = parseInt(point.style.top, 10);
       let offset = 3 * (event.shiftKey ? 10 : 1);
 
       switch (code) {
-        case 37: point.style.left = left - offset + 'px'; break;
-        case 38: point.style.top = top - offset + 'px'; break;
-        case 39: point.style.left = left + offset + 'px'; break;
-        case 40: point.style.top = top + offset + 'px'; break;
+        case 37: point.style.left = left - offset + "px"; break;
+        case 38: point.style.top = top - offset + "px"; break;
+        case 39: point.style.left = left + offset + "px"; break;
+        case 40: point.style.top = top + offset + "px"; break;
       }
 
       this._updateFromPoints();
     }
   },
 
-  _onCurveClick: function(event) {
+  _onCurveClick: function (event) {
     this.curveBoundingBox = this.curve.getBoundingClientRect();
 
     let left = this.curveBoundingBox.left;
@@ -376,9 +379,9 @@ CubicBezierWidget.prototype = {
 
     // Find which point is closer
     let distP1 = distance(x, y,
-      parseInt(this.p1.style.left), parseInt(this.p1.style.top));
+      parseInt(this.p1.style.left, 10), parseInt(this.p1.style.top, 10));
     let distP2 = distance(x, y,
-      parseInt(this.p2.style.left), parseInt(this.p2.style.top));
+      parseInt(this.p2.style.left, 10), parseInt(this.p2.style.top, 10));
 
     let point = distP1 < distP2 ? this.p1 : this.p2;
     point.style.left = x + "px";
@@ -387,17 +390,19 @@ CubicBezierWidget.prototype = {
     this._updateFromPoints();
   },
 
-  _onNewCoordinates: function(event, coordinates) {
+  _onNewCoordinates: function (event, coordinates) {
     this.coordinates = coordinates;
   },
 
   /**
    * Get the current point coordinates and redraw the curve to match
    */
-  _updateFromPoints: function() {
+  _updateFromPoints: function () {
     // Get the new coordinates from the point's offsets
     let coordinates = this.bezierCanvas.offsetsToCoordinates(this.p1);
-    coordinates = coordinates.concat(this.bezierCanvas.offsetsToCoordinates(this.p2));
+    coordinates = coordinates.concat(
+      this.bezierCanvas.offsetsToCoordinates(this.p2)
+    );
 
     this.presets.refreshMenu(coordinates);
     this._redraw(coordinates);
@@ -407,7 +412,7 @@ CubicBezierWidget.prototype = {
    * Redraw the curve
    * @param {Array} coordinates The array of control point coordinates
    */
-  _redraw: function(coordinates) {
+  _redraw: function (coordinates) {
     // Provide a new CubicBezier to the canvas and plot the curve
     this.bezierCanvas.bezier = new CubicBezier(coordinates);
     this.bezierCanvas.plot();
@@ -433,7 +438,8 @@ CubicBezierWidget.prototype = {
 
   /**
    * Set new coordinates for the control point and redraw the curve
-   * @param {String} value A string value. E.g. "linear", "cubic-bezier(0,0,1,1)"
+   * @param {String} value A string value. E.g. "linear",
+   * "cubic-bezier(0,0,1,1)"
    */
   set cssCubicBezierValue(value) {
     if (!value) {
@@ -449,7 +455,7 @@ CubicBezierWidget.prototype = {
     this.coordinates = coordinates;
   },
 
-  destroy: function() {
+  destroy: function () {
     this._removeEvents();
     this._removeMarkup();
 
@@ -463,7 +469,8 @@ CubicBezierWidget.prototype = {
 /**
  * CubicBezierPreset widget.
  * Builds a menu of presets from CubicBezierPresets
- * @param {DOMNode} parent The container where the preset panel should be created
+ * @param {DOMNode} parent The container where the preset panel should be
+ * created
  *
  * Emits "new-coordinate" event along with the coordinates
  * whenever a preset is selected.
@@ -508,16 +515,16 @@ CubicBezierPresetWidget.prototype = {
    *        div .preset
    *        ...
    */
-  _initMarkup: function() {
+  _initMarkup: function () {
     let doc = this.parent.ownerDocument;
 
-    let presetPane = doc.createElement("div");
+    let presetPane = doc.createElementNS(XHTML_NS, "div");
     presetPane.className = "preset-pane";
 
-    let categoryList = doc.createElement("div");
+    let categoryList = doc.createElementNS(XHTML_NS, "div");
     categoryList.id = "preset-categories";
 
-    let presetContainer = doc.createElement("div");
+    let presetContainer = doc.createElementNS(XHTML_NS, "div");
     presetContainer.id = "preset-container";
 
     Object.keys(PRESETS).forEach(categoryLabel => {
@@ -543,27 +550,28 @@ CubicBezierPresetWidget.prototype = {
     };
   },
 
-  _createCategory: function(categoryLabel) {
+  _createCategory: function (categoryLabel) {
     let doc = this.parent.ownerDocument;
 
-    let category = doc.createElement("div");
+    let category = doc.createElementNS(XHTML_NS, "div");
     category.id = categoryLabel;
     category.classList.add("category");
 
     let categoryDisplayLabel = this._normalizeCategoryLabel(categoryLabel);
     category.textContent = categoryDisplayLabel;
+    category.setAttribute("title", categoryDisplayLabel);
 
     return category;
   },
 
-  _normalizeCategoryLabel: function(categoryLabel) {
+  _normalizeCategoryLabel: function (categoryLabel) {
     return categoryLabel.replace("/-/g", " ");
   },
 
-  _createPresetList: function(categoryLabel) {
+  _createPresetList: function (categoryLabel) {
     let doc = this.parent.ownerDocument;
 
-    let presetList = doc.createElement("div");
+    let presetList = doc.createElementNS(XHTML_NS, "div");
     presetList.id = "preset-category-" + categoryLabel;
     presetList.classList.add("preset-list");
 
@@ -575,15 +583,15 @@ CubicBezierPresetWidget.prototype = {
     return presetList;
   },
 
-  _createPreset: function(categoryLabel, presetLabel) {
+  _createPreset: function (categoryLabel, presetLabel) {
     let doc = this.parent.ownerDocument;
 
-    let preset = doc.createElement("div");
+    let preset = doc.createElementNS(XHTML_NS, "div");
     preset.classList.add("preset");
     preset.id = presetLabel;
     preset.coordinates = PRESETS[categoryLabel][presetLabel];
     // Create preset preview
-    let curve = doc.createElement("canvas");
+    let curve = doc.createElementNS(XHTML_NS, "canvas");
     let bezier = new CubicBezier(preset.coordinates);
     curve.setAttribute("height", 50);
     curve.setAttribute("width", 50);
@@ -595,19 +603,21 @@ CubicBezierPresetWidget.prototype = {
     preset.appendChild(curve);
 
     // Create preset label
-    let presetLabelElem = doc.createElement("p");
-    let presetDisplayLabel = this._normalizePresetLabel(categoryLabel, presetLabel);
+    let presetLabelElem = doc.createElementNS(XHTML_NS, "p");
+    let presetDisplayLabel = this._normalizePresetLabel(categoryLabel,
+                                                        presetLabel);
     presetLabelElem.textContent = presetDisplayLabel;
     preset.appendChild(presetLabelElem);
+    preset.setAttribute("title", presetDisplayLabel);
 
     return preset;
   },
 
-  _normalizePresetLabel: function(categoryLabel, presetLabel) {
+  _normalizePresetLabel: function (categoryLabel, presetLabel) {
     return presetLabel.replace(categoryLabel + "-", "").replace("/-/g", " ");
   },
 
-  _initEvents: function() {
+  _initEvents: function () {
     for (let category of this.categories) {
       category.addEventListener("click", this._onCategoryClick);
     }
@@ -617,7 +627,7 @@ CubicBezierPresetWidget.prototype = {
     }
   },
 
-  _removeEvents: function() {
+  _removeEvents: function () {
     for (let category of this.categories) {
       category.removeEventListener("click", this._onCategoryClick);
     }
@@ -627,16 +637,16 @@ CubicBezierPresetWidget.prototype = {
     }
   },
 
-  _onPresetClick: function(event) {
+  _onPresetClick: function (event) {
     this.emit("new-coordinates", event.currentTarget.coordinates);
     this.activePreset = event.currentTarget;
   },
 
-  _onCategoryClick: function(event) {
+  _onCategoryClick: function (event) {
     this.activeCategory = event.target;
   },
 
-  _setActivePresetList: function(presetListId) {
+  _setActivePresetList: function (presetListId) {
     let presetList = this.presetPane.querySelector("#" + presetListId);
     swapClassName("active-preset-list", this._activePresetList, presetList);
     this._activePresetList = presetList;
@@ -668,7 +678,7 @@ CubicBezierPresetWidget.prototype = {
    * existing preset.
    * @param {Array} coordinates new coords [i, j, k, l]
    */
-  refreshMenu: function(coordinates) {
+  refreshMenu: function (coordinates) {
     // If we cannot find a matching preset, keep
     // menu on last known preset category.
     let category = this._activeCategory;
@@ -686,21 +696,19 @@ CubicBezierPresetWidget.prototype = {
     // If the new coordinates do match a preset,
     // set its category and preset button as active.
     Object.keys(PRESETS).forEach(categoryLabel => {
-
       Object.keys(PRESETS[categoryLabel]).forEach(presetLabel => {
         if (coordsAreEqual(PRESETS[categoryLabel][presetLabel], coordinates)) {
           category = this.parent.querySelector("#" + categoryLabel);
           preset = this.parent.querySelector("#" + presetLabel);
         }
       });
-
     });
 
     this.activeCategory = category;
     this.activePreset = preset;
   },
 
-  destroy: function() {
+  destroy: function () {
     this._removeEvents();
     this.parent.querySelector(".preset-pane").remove();
   }
@@ -722,24 +730,24 @@ function TimingFunctionPreviewWidget(parent) {
 TimingFunctionPreviewWidget.prototype = {
   PREVIEW_DURATION: 1000,
 
-  _initMarkup: function() {
+  _initMarkup: function () {
     let doc = this.parent.ownerDocument;
 
-    let container = doc.createElement("div");
+    let container = doc.createElementNS(XHTML_NS, "div");
     container.className = "timing-function-preview";
 
-    this.dot = doc.createElement("div");
+    this.dot = doc.createElementNS(XHTML_NS, "div");
     this.dot.className = "dot";
     container.appendChild(this.dot);
 
-    let scale = doc.createElement("div");
+    let scale = doc.createElementNS(XHTML_NS, "div");
     scale.className = "scale";
     container.appendChild(scale);
 
     this.parent.appendChild(container);
   },
 
-  destroy: function() {
+  destroy: function () {
     clearTimeout(this.autoRestartAnimation);
     this.parent.querySelector(".timing-function-preview").remove();
     this.parent = this.dot = null;
@@ -751,10 +759,10 @@ TimingFunctionPreviewWidget.prototype = {
    * supplied function is invalid, the preview will stop.
    * @param {String} value
    */
-  preview: function(value) {
+  preview: function (value) {
     // Don't restart the preview animation if the value is the same
     if (value === this.previousValue) {
-      return false;
+      return;
     }
 
     clearTimeout(this.autoRestartAnimation);
@@ -770,14 +778,18 @@ TimingFunctionPreviewWidget.prototype = {
   /**
    * Re-start the preview animation from the beginning
    */
-  restartAnimation: function() {
-    // Reset the animation duration in case it was changed
-    this.dot.style.animationDuration = (this.PREVIEW_DURATION * 2) + "ms";
-
+  restartAnimation: function () {
     // Just toggling the class won't do it unless there's a sync reflow
-    this.dot.classList.remove("animate");
-    let w = this.dot.offsetWidth;
-    this.dot.classList.add("animate");
+    this.dot.animate([
+      { left: "-7px", offset: 0 },
+      { left: "143px", offset: 0.25 },
+      { left: "143px", offset: 0.5 },
+      { left: "-7px", offset: 0.75 },
+      { left: "-7px", offset: 1 }
+    ], {
+      duration: (this.PREVIEW_DURATION * 2),
+      fill: "forwards"
+    });
 
     // Restart it again after a while
     this.autoRestartAnimation = setTimeout(this.restartAnimation.bind(this),
@@ -788,7 +800,7 @@ TimingFunctionPreviewWidget.prototype = {
 // Helpers
 
 function getPadding(padding) {
-  let p = typeof padding === 'number'? [padding] : padding;
+  let p = typeof padding === "number" ? [padding] : padding;
 
   if (p.length === 1) {
     p[1] = p[0];
@@ -821,7 +833,7 @@ function parseTimingFunction(value) {
     return PREDEFINED[value];
   }
 
-  let tokenStream = DOMUtils.getCSSLexer(value);
+  let tokenStream = getCSSLexer(value);
   let getNextToken = () => {
     while (true) {
       let token = tokenStream.nextToken();
