@@ -3,12 +3,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const {Ci, Cu} = require("chrome");
+const {Ci} = require("chrome");
 const EventEmitter = require("devtools/shared/event-emitter");
-loader.lazyImporter(this, "setNamedTimeout",
-  "resource://devtools/client/shared/widgets/ViewHelpers.jsm");
-loader.lazyImporter(this, "clearNamedTimeout",
-  "resource://devtools/client/shared/widgets/ViewHelpers.jsm");
+loader.lazyRequireGetter(this, "setNamedTimeout",
+  "devtools/client/shared/widgets/view-helpers", true);
+loader.lazyRequireGetter(this, "clearNamedTimeout",
+  "devtools/client/shared/widgets/view-helpers", true);
 
 const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 const HTML_NS = "http://www.w3.org/1999/xhtml";
@@ -52,6 +52,8 @@ const MAX_VISIBLE_STRING_SIZE = 100;
  *                          the table. See @setupColumns for more info.
  *        - uniqueId: the column which will be the unique identifier of each
  *                    entry in the table. Default: name.
+ *        - wrapTextInElements: Don't ever use 'value' attribute on labels.
+ *                              Default: false.
  *        - emptyText: text to display when no entries in the table to display.
  *        - highlightUpdated: true to highlight the changed/added row.
  *        - removableColumns: Whether columns are removeable. If set to false,
@@ -68,9 +70,10 @@ function TableWidget(node, options = {}) {
   this._parent = node;
 
   let {initialColumns, emptyText, uniqueId, highlightUpdated, removableColumns,
-       firstColumn, cellContextMenuId} = options;
+       firstColumn, wrapTextInElements, cellContextMenuId} = options;
   this.emptyText = emptyText || "";
   this.uniqueId = uniqueId || "name";
+  this.wrapTextInElements = wrapTextInElements || false;
   this.firstColumn = firstColumn || "";
   this.highlightUpdated = highlightUpdated || false;
   this.removableColumns = removableColumns !== false;
@@ -213,7 +216,7 @@ TableWidget.prototype = {
   /**
    * Emit all cell edit events.
    */
-  onChange: function(type, data) {
+  onChange: function (type, data) {
     let changedField = data.change.field;
     let colName = changedField.parentNode.id;
     let column = this.columns.get(colName);
@@ -242,7 +245,7 @@ TableWidget.prototype = {
     this.emit(EVENTS.CELL_EDIT, change);
   },
 
-  onEditorDestroyed: function() {
+  onEditorDestroyed: function () {
     this._editableFieldsEngine = null;
   },
 
@@ -257,7 +260,7 @@ TableWidget.prototype = {
    * @param  {Event} event
    *         Keydown event
    */
-  onEditorTab: function(event) {
+  onEditorTab: function (event) {
     let textbox = event.target;
     let editor = this._editableFieldsEngine;
 
@@ -355,7 +358,7 @@ TableWidget.prototype = {
    * @param  {Event} event
    *         Keydown event
    */
-  getEditedCellOnTab: function(event, column) {
+  getEditedCellOnTab: function (event, column) {
     let cell = null;
     let cols = this.editableColumns;
     let rowIndex = this.visibleSelectedIndex;
@@ -415,7 +418,7 @@ TableWidget.prototype = {
    * @param  {Object} row
    *         The values from the removed row.
    */
-  onRowRemoved: function(event, row) {
+  onRowRemoved: function (event, row) {
     if (!this._editableFieldsEngine || !this._editableFieldsEngine.isEditing) {
       return;
     }
@@ -435,7 +438,7 @@ TableWidget.prototype = {
   /**
    * Cancel an edit because the edit target has been lost.
    */
-  onEditorTargetLost: function() {
+  onEditorTargetLost: function () {
     let editor = this._editableFieldsEngine;
 
     if (!editor || !editor.isEditing) {
@@ -449,7 +452,7 @@ TableWidget.prototype = {
    * Keydown event handler for the table. Used for keyboard navigation amongst
    * rows.
    */
-  onKeydown: function(event) {
+  onKeydown: function (event) {
     // If we are in edit mode bail out.
     if (this._editableFieldsEngine && this._editableFieldsEngine.isEditing) {
       return;
@@ -512,7 +515,7 @@ TableWidget.prototype = {
    * rows. This method clears any inline editors if an area outside a textbox or
    * label is clicked.
    */
-  onMousedown: function({target}) {
+  onMousedown: function ({target}) {
     let nodeName = target.nodeName;
 
     if (nodeName === "textbox" || !this._editableFieldsEngine) {
@@ -529,7 +532,7 @@ TableWidget.prototype = {
    * @param  {String|Array} editableColumns
    *         An array or comma separated list of editable column names.
    */
-  makeFieldsEditable: function(editableColumns) {
+  makeFieldsEditable: function (editableColumns) {
     let selectors = [];
 
     if (typeof editableColumns === "string") {
@@ -566,7 +569,7 @@ TableWidget.prototype = {
     }
   },
 
-  destroy: function() {
+  destroy: function () {
     this.off(EVENTS.ROW_SELECTED, this.bindSelectedRow);
     this.off(EVENTS.ROW_REMOVED, this.onRowRemoved);
 
@@ -590,7 +593,7 @@ TableWidget.prototype = {
   /**
    * Sets the text to be shown when the table is empty.
    */
-  setPlaceholderText: function(text) {
+  setPlaceholderText: function (text) {
     this.placeholder.setAttribute("value", text);
   },
 
@@ -599,7 +602,7 @@ TableWidget.prototype = {
    * context menu allows users to toggle various columns, only with an exception
    * of the unique columns and when only two columns are visible in the table.
    */
-  setupHeadersContextMenu: function() {
+  setupHeadersContextMenu: function () {
     let popupset = this.document.getElementsByTagName("popupset")[0];
     if (!popupset) {
       popupset = this.document.createElementNS(XUL_NS, "popupset");
@@ -617,7 +620,7 @@ TableWidget.prototype = {
    * Populates the header context menu with the names of the columns along with
    * displaying which columns are hidden or visible.
    */
-  populateMenuPopup: function() {
+  populateMenuPopup: function () {
     if (!this.menupopup) {
       return;
     }
@@ -646,7 +649,7 @@ TableWidget.prototype = {
   /**
    * Event handler for the `command` event on the column headers context menu
    */
-  onPopupCommand: function(event) {
+  onPopupCommand: function (event) {
     let item = event.originalTarget;
     let checked = !!item.getAttribute("checked");
     let id = item.getAttribute("data-id");
@@ -673,7 +676,7 @@ TableWidget.prototype = {
    * @param {array} hiddenColumns
    *        Ids of all the columns that are hidden by default.
    */
-  setColumns: function(columns, sortOn = this.sortedOn, hiddenColumns = []) {
+  setColumns: function (columns, sortOn = this.sortedOn, hiddenColumns = []) {
     for (let column of this.columns.values()) {
       column.destroy();
     }
@@ -716,7 +719,7 @@ TableWidget.prototype = {
    * Returns true if the passed string or the row json object corresponds to the
    * selected item in the table.
    */
-  isSelected: function(item) {
+  isSelected: function (item) {
     if (typeof item == "object") {
       item = item[this.uniqueId];
     }
@@ -727,14 +730,14 @@ TableWidget.prototype = {
   /**
    * Selects the row corresponding to the `id` json.
    */
-  selectRow: function(id) {
+  selectRow: function (id) {
     this.selectedRow = id;
   },
 
   /**
    * Selects the next row. Cycles over to the first row if last row is selected
    */
-  selectNextRow: function() {
+  selectNextRow: function () {
     for (let column of this.columns.values()) {
       column.selectNextRow();
     }
@@ -744,7 +747,7 @@ TableWidget.prototype = {
    * Selects the previous row. Cycles over to the last row if first row is
    * selected.
    */
-  selectPreviousRow: function() {
+  selectPreviousRow: function () {
     for (let column of this.columns.values()) {
       column.selectPreviousRow();
     }
@@ -753,7 +756,7 @@ TableWidget.prototype = {
   /**
    * Clears any selected row.
    */
-  clearSelection: function() {
+  clearSelection: function () {
     this.selectedIndex = -1;
   },
 
@@ -768,9 +771,9 @@ TableWidget.prototype = {
    * @param {boolean} suppressFlash
    *        true to not flash the row while inserting the row.
    */
-  push: function(item, suppressFlash) {
+  push: function (item, suppressFlash) {
     if (!this.sortedOn || !this.columns) {
-      Cu.reportError("Can't insert item without defining columns first");
+      console.error("Can't insert item without defining columns first");
       return;
     }
 
@@ -799,8 +802,8 @@ TableWidget.prototype = {
   /**
    * Removes the row associated with the `item` object.
    */
-  remove: function(item) {
-    if (typeof item == "string") {
+  remove: function (item) {
+    if (typeof item != "object") {
       item = this.items.get(item);
     }
     if (!item) {
@@ -827,7 +830,7 @@ TableWidget.prototype = {
    * used to insert the row using `push` method. The linking is done via the
    * `uniqueId` key's value.
    */
-  update: function(item) {
+  update: function (item) {
     let oldItem = this.items.get(item[this.uniqueId]);
     if (!oldItem) {
       return;
@@ -850,7 +853,7 @@ TableWidget.prototype = {
   /**
    * Removes all of the rows from the table.
    */
-  clear: function() {
+  clear: function () {
     this.items.clear();
     for (let column of this.columns.values()) {
       column.clear();
@@ -867,7 +870,7 @@ TableWidget.prototype = {
    * @param {string} column
    *        The id of the column on which the table should be sorted.
    */
-  sortBy: function(column) {
+  sortBy: function (column) {
     this.emit(EVENTS.COLUMN_SORTED, column);
     this.sortedOn = column;
 
@@ -925,7 +928,7 @@ TableWidget.prototype = {
   /**
    * Calls the afterScroll function when the user has stopped scrolling
    */
-  onScroll: function() {
+  onScroll: function () {
     clearNamedTimeout("table-scroll");
     setNamedTimeout("table-scroll", AFTER_SCROLL_DELAY, this.afterScroll);
   },
@@ -933,7 +936,7 @@ TableWidget.prototype = {
   /**
    * Emits the "scroll-end" event when the whole table is scrolled
    */
-  afterScroll: function() {
+  afterScroll: function () {
     let scrollHeight = this.tbody.getBoundingClientRect().height -
         this.tbody.querySelector(".table-widget-column-header").clientHeight;
 
@@ -964,6 +967,7 @@ function Column(table, id, header) {
   this.window = table.window;
   this.id = id;
   this.uniqueId = table.uniqueId;
+  this.wrapTextInElements = table.wrapTextInElements;
   this.table = table;
   this.cells = [];
   this.items = {};
@@ -1088,7 +1092,7 @@ Column.prototype = {
    * @param {string} column
    *        The id of the column being sorted by.
    */
-  onColumnSorted: function(event, column) {
+  onColumnSorted: function (event, column) {
     if (column != this.id) {
       this.sorted = 0;
       return;
@@ -1100,7 +1104,7 @@ Column.prototype = {
     this.updateZebra();
   },
 
-  onTableFiltered: function(event, itemsToHide) {
+  onTableFiltered: function (event, itemsToHide) {
     this._updateItems();
     if (!this.cells) {
       return;
@@ -1122,7 +1126,7 @@ Column.prototype = {
    * @param {string} id
    *        The unique id of the object associated with the row.
    */
-  onRowUpdated: function(event, id) {
+  onRowUpdated: function (event, id) {
     this._updateItems();
     if (this.highlightUpdated && this.items[id] != null) {
       if (this.table.editBookmark) {
@@ -1139,7 +1143,7 @@ Column.prototype = {
     this.updateZebra();
   },
 
-  destroy: function() {
+  destroy: function () {
     this.table.off(EVENTS.COLUMN_SORTED, this.onColumnSorted);
     this.table.off(EVENTS.HEADER_CONTEXT_MENU, this.toggleColumn);
     this.table.off(EVENTS.ROW_UPDATED, this.onRowUpdated);
@@ -1158,7 +1162,7 @@ Column.prototype = {
   /**
    * Selects the row at the `index` index
    */
-  selectRowAt: function(index) {
+  selectRowAt: function (index) {
     if (this.selectedRow != null) {
       this.cells[this.items[this.selectedRow]].toggleClass("theme-selected");
     }
@@ -1174,7 +1178,7 @@ Column.prototype = {
   /**
    * Selects the row with the object having the `uniqueId` value as `id`
    */
-  selectRow: function(id) {
+  selectRow: function (id) {
     this._updateItems();
     this.selectRowAt(this.items[id]);
   },
@@ -1182,7 +1186,7 @@ Column.prototype = {
   /**
    * Selects the next row. Cycles to first if last row is selected.
    */
-  selectNextRow: function() {
+  selectNextRow: function () {
     this._updateItems();
     let index = this.items[this.selectedRow] + 1;
     if (index == this.cells.length) {
@@ -1194,7 +1198,7 @@ Column.prototype = {
   /**
    * Selects the previous row. Cycles to last if first row is selected.
    */
-  selectPreviousRow: function() {
+  selectPreviousRow: function () {
     this._updateItems();
     let index = this.items[this.selectedRow] - 1;
     if (index == -1) {
@@ -1211,7 +1215,7 @@ Column.prototype = {
    * @returns {number}
    *          The index of the currently pushed item.
    */
-  push: function(item) {
+  push: function (item) {
     let value = item[this.id];
 
     if (this.sorted) {
@@ -1241,7 +1245,7 @@ Column.prototype = {
   /**
    * Inserts the `item` object at the given `index` index in the table.
    */
-  insertAt: function(item, index) {
+  insertAt: function (item, index) {
     if (index < this.cells.length) {
       this._itemsDirty = true;
     }
@@ -1263,7 +1267,7 @@ Column.prototype = {
    * @param {string} checked
    *        true if the column is visible
    */
-  toggleColumn: function(event, id, checked) {
+  toggleColumn: function (event, id, checked) {
     if (arguments.length == 0) {
       // Act like a toggling method when called with no params
       id = this.id;
@@ -1282,7 +1286,7 @@ Column.prototype = {
   /**
    * Removes the corresponding item from the column.
    */
-  remove: function(item) {
+  remove: function (item) {
     this._updateItems();
     let index = this.items[item[this.uniqueId]];
     if (index == null) {
@@ -1300,7 +1304,7 @@ Column.prototype = {
   /**
    * Updates the corresponding item from the column.
    */
-  update: function(item) {
+  update: function (item) {
     this._updateItems();
 
     let index = this.items[item[this.uniqueId]];
@@ -1315,7 +1319,7 @@ Column.prototype = {
    * Updates the `this.items` cell-id vs cell-index map to be in sync with
    * `this.cells`.
    */
-  _updateItems: function() {
+  _updateItems: function () {
     if (!this._itemsDirty) {
       return;
     }
@@ -1328,7 +1332,7 @@ Column.prototype = {
   /**
    * Clears the current column
    */
-  clear: function() {
+  clear: function () {
     this.cells = [];
     this.items = {};
     this._itemsDirty = false;
@@ -1342,7 +1346,7 @@ Column.prototype = {
    * Sorts the given items and returns the sorted list if the table was sorted
    * by this column.
    */
-  sort: function(items) {
+  sort: function (items) {
     // Only sort the array if we are sorting based on this column
     if (this.sorted == 1) {
       items.sort((a, b) => {
@@ -1395,7 +1399,7 @@ Column.prototype = {
    * Click event handler for the column. Used to detect click on header for
    * for sorting.
    */
-  onClick: function(event) {
+  onClick: function (event) {
     let target = event.originalTarget;
 
     if (target.nodeType !== target.ELEMENT_NODE || target == this.column) {
@@ -1410,7 +1414,7 @@ Column.prototype = {
   /**
    * Mousedown event handler for the column. Used to select rows.
    */
-  onMousedown: function(event) {
+  onMousedown: function (event) {
     let target = event.originalTarget;
 
     if (target.nodeType !== target.ELEMENT_NODE ||
@@ -1446,6 +1450,7 @@ Column.prototype = {
 function Cell(column, item, nextCell) {
   let document = column.document;
 
+  this.wrapTextInElements = column.wrapTextInElements;
   this.label = document.createElementNS(XUL_NS, "label");
   this.label.setAttribute("crop", "end");
   this.label.className = "plain table-widget-cell";
@@ -1499,6 +1504,12 @@ Cell.prototype = {
       return;
     }
 
+    if (this.wrapTextInElements && !(value instanceof Ci.nsIDOMNode)) {
+      let span = this.label.ownerDocument.createElementNS(HTML_NS, "span");
+      span.textContent = value;
+      value = span;
+    }
+
     if (!(value instanceof Ci.nsIDOMNode) &&
         value.length > MAX_VISIBLE_STRING_SIZE) {
       value = value .substr(0, MAX_VISIBLE_STRING_SIZE) + "\u2026";
@@ -1521,7 +1532,7 @@ Cell.prototype = {
     return this._value;
   },
 
-  toggleClass: function(className, condition) {
+  toggleClass: function (className, condition) {
     this.label.classList.toggle(className, condition);
   },
 
@@ -1529,7 +1540,7 @@ Cell.prototype = {
    * Flashes the cell for a brief time. This when done for with cells in all
    * columns, makes it look like the row is being highlighted/flashed.
    */
-  flash: function() {
+  flash: function () {
     if (!this.label.parentNode) {
       return;
     }
@@ -1544,11 +1555,11 @@ Cell.prototype = {
     this.label.classList.add("flash-out");
   },
 
-  focus: function() {
+  focus: function () {
     this.label.focus();
   },
 
-  destroy: function() {
+  destroy: function () {
     this.label.remove();
     this.label = null;
   }
@@ -1634,7 +1645,7 @@ EditableFieldsEngine.prototype = {
    * @param  {EventTarget} target
    *         Calling event's target.
    */
-  onTrigger: function({target}) {
+  onTrigger: function ({target}) {
     this.edit(target);
   },
 
@@ -1648,7 +1659,7 @@ EditableFieldsEngine.prototype = {
    * @param  {Event} event
    *         The calling event.
    */
-  onKeydown: function(event) {
+  onKeydown: function (event) {
     if (!this.textbox) {
       return;
     }
@@ -1675,7 +1686,7 @@ EditableFieldsEngine.prototype = {
    * @param  {Node} target
    *         Dom node to be edited.
    */
-  edit: function(target) {
+  edit: function (target) {
     if (!target) {
       return;
     }
@@ -1704,7 +1715,7 @@ EditableFieldsEngine.prototype = {
     this.textbox.select();
   },
 
-  completeEdit: function() {
+  completeEdit: function () {
     if (!this.isEditing) {
       return;
     }
@@ -1738,7 +1749,7 @@ EditableFieldsEngine.prototype = {
   /**
    * Cancel an edit.
    */
-  cancelEdit: function() {
+  cancelEdit: function () {
     if (!this.isEditing) {
       return;
     }
@@ -1752,7 +1763,7 @@ EditableFieldsEngine.prototype = {
   /**
    * Stop edit mode and apply changes.
    */
-  blur: function() {
+  blur: function () {
     if (this.isEditing) {
       this.completeEdit();
     }
@@ -1766,7 +1777,7 @@ EditableFieldsEngine.prototype = {
    * @param  {Node} destination [description]
    *         The node to copy styles to.
    */
-  copyStyles: function(source, destination) {
+  copyStyles: function (source, destination) {
     let style = source.ownerGlobal.getComputedStyle(source);
     let props = [
       "borderTopWidth",
@@ -1781,8 +1792,8 @@ EditableFieldsEngine.prototype = {
       "marginRight",
       "marginBottom",
       "marginLeft",
-      "MozMarginStart",
-      "MozMarginEnd"
+      "marginInlineStart",
+      "marginInlineEnd"
     ];
 
     for (let prop of props) {
@@ -1796,7 +1807,7 @@ EditableFieldsEngine.prototype = {
   /**
    * Destroys all editors in the current document.
    */
-  destroy: function() {
+  destroy: function () {
     if (this.textbox) {
       this.textbox.removeEventListener("keydown", this.onKeydown);
       this.textbox.remove();
