@@ -1,5 +1,3 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set ft= javascript ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,11 +6,11 @@
 
 const {Cc, Ci, Cu} = require("chrome");
 
-var WebConsoleUtils = require("devtools/shared/webconsole/utils").Utils;
-var Heritage = require("sdk/core/heritage");
+var WebConsoleUtils = require("devtools/client/webconsole/utils").Utils;
+var { extend } = require("sdk/core/heritage");
 var {TargetFactory} = require("devtools/client/framework/target");
 var {Tools} = require("devtools/client/definitions");
-const { Task } = require("resource://gre/modules/Task.jsm");
+const { Task } = require("devtools/shared/task");
 var promise = require("promise");
 var Services = require("Services");
 
@@ -34,8 +32,7 @@ const BROWSER_CONSOLE_FILTER_PREFS_PREFIX = "devtools.browserconsole.filter.";
 
 var gHudId = 0;
 
-///////////////////////////////////////////////////////////////////////////
-//// The HUD service
+// The HUD service
 
 function HUD_SERVICE()
 {
@@ -54,24 +51,7 @@ HUD_SERVICE.prototype =
    */
   consoles: null,
 
-  _browerConsoleSessionState: false,
-  storeBrowserConsoleSessionState() {
-    this._browerConsoleSessionState = !!this.getBrowserConsole();
-  },
-  getBrowserConsoleSessionState() {
-    return this._browerConsoleSessionState;
-  },
-
   /**
-   * Restore the Browser Console as provided by SessionStore.
-   */
-  restoreBrowserConsoleSession: function HS_restoreBrowserConsoleSession() {
-    if (!HUDService.getBrowserConsole()) {
-      HUDService.toggleBrowserConsole();
-    }
-  },
-
-   /**
    * Assign a function to this property to listen for every request that
    * completes. Used by unit tests. The callback takes one argument: the HTTP
    * activity object as received from the remote Web Console.
@@ -83,12 +63,12 @@ HUD_SERVICE.prototype =
   lastFinishedRequest: null,
 
   /**
-   * Firefox-specific current tab getter
+   * Get the current context, which is the main application window.
    *
    * @returns nsIDOMWindow
    */
   currentContext: function HS_currentContext() {
-    return Services.wm.getMostRecentWindow("navigator:browser");
+    return Services.wm.getMostRecentWindow(gDevTools.chromeWindowType);
   },
 
   /**
@@ -251,7 +231,7 @@ HUD_SERVICE.prototype =
         .then((aBrowserConsole) => {
           this._browserConsoleDefer.resolve(aBrowserConsole);
           this._browserConsoleDefer = null;
-        })
+        });
     }, console.error.bind(console));
 
     return this._browserConsoleDefer.promise;
@@ -313,7 +293,7 @@ function WebConsole(aTarget, aIframeWindow, aChromeWindow)
   this.browserWindow = this.chromeWindow.top;
 
   let element = this.browserWindow.document.documentElement;
-  if (element.getAttribute("windowtype") != "navigator:browser") {
+  if (element.getAttribute("windowtype") != gDevTools.chromeWindowType) {
     this.browserWindow = HUDService.currentContext();
   }
 
@@ -456,7 +436,7 @@ WebConsole.prototype = {
   viewSource: function WC_viewSource(aSourceURL, aSourceLine) {
     // Attempt to access view source via a browser first, which may display it in
     // a tab, if enabled.
-    let browserWin = Services.wm.getMostRecentWindow("navigator:browser");
+    let browserWin = Services.wm.getMostRecentWindow(gDevTools.chromeWindowType);
     if (browserWin && browserWin.BrowserViewSourceOfDocument) {
       return browserWin.BrowserViewSourceOfDocument({
         URL: aSourceURL,
@@ -507,7 +487,7 @@ WebConsole.prototype = {
     }
     toolbox.viewSourceInDebugger(aSourceURL, aSourceLine).then(() => {
       this.ui.emit("source-in-debugger-opened");
-    })
+    });
   },
 
   /**
@@ -605,12 +585,14 @@ WebConsole.prototype = {
       }
     }
 
-    let onDestroy = Task.async(function*() {
-      try {
-        yield this.target.activeTab.focus()
-      }
-      catch (ex) {
-        // Tab focus can fail if the tab or target is closed.
+    let onDestroy = Task.async(function* () {
+      if (!this._browserConsole) {
+        try {
+          yield this.target.activeTab.focus();
+        }
+        catch (ex) {
+          // Tab focus can fail if the tab or target is closed.
+        }
       }
 
       let id = WebConsoleUtils.supportsString(this.hudId);
@@ -653,8 +635,7 @@ function BrowserConsole()
   this._telemetry = new Telemetry();
 }
 
-BrowserConsole.prototype = Heritage.extend(WebConsole.prototype,
-{
+BrowserConsole.prototype = extend(WebConsole.prototype, {
   _browserConsole: true,
   _bc_init: null,
   _bc_destroyer: null,
@@ -673,9 +654,6 @@ BrowserConsole.prototype = Heritage.extend(WebConsole.prototype,
       return this._bc_init;
     }
 
-    // Only add the shutdown observer if we've opened a Browser Console window.
-    ShutdownObserver.init();
-
     this.ui._filterPrefsPrefix = BROWSER_CONSOLE_FILTER_PREFS_PREFIX;
 
     let window = this.iframeWindow;
@@ -689,9 +667,6 @@ BrowserConsole.prototype = Heritage.extend(WebConsole.prototype,
     };
     window.addEventListener("unload", onClose);
 
-    // Make sure Ctrl-W closes the Browser Console window.
-    window.document.getElementById("cmd_close").removeAttribute("disabled");
-
     this._telemetry.toolOpened("browserconsole");
 
     // Create an onFocus handler just to display the dev edition promo.
@@ -748,31 +723,3 @@ const HUDService = new HUD_SERVICE();
   exports.consoles = HUDService.consoles;
   exports.lastFinishedRequest = HUDService.lastFinishedRequest;
 })();
-
-/**
- * The ShutdownObserver listens for app shutdown and saves the current state
- * of the Browser Console for session restore.
- */
-var ShutdownObserver = {
-  _initialized: false,
-  init() {
-    if (this._initialized) {
-      return;
-    }
-
-    Services.obs.addObserver(this, "quit-application-granted", false);
-
-    this._initialized = true;
-  },
-
-  observe(message, topic) {
-    if (topic == "quit-application-granted") {
-      HUDService.storeBrowserConsoleSessionState();
-      this.uninit();
-    }
-  },
-
-  uninit() {
-    Services.obs.removeObserver(this, "quit-application-granted");
-  }
-};
