@@ -4,47 +4,42 @@
 
 "use strict";
 
-var { components, Cc, Ci, Cu } = require("chrome");
-var Services = require("Services");
-
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-Cu.import("resource://gre/modules/NetUtil.jsm");
-Cu.import("resource://gre/modules/FileUtils.jsm");
-Cu.import("resource://gre/modules/Task.jsm");
-
+const {Cc, Ci} = require("chrome");
+const Services = require("Services");
+const {XPCOMUtils} = require("resource://gre/modules/XPCOMUtils.jsm");
 const promise = require("promise");
+const {Task} = require("devtools/shared/task");
 const events = require("sdk/event/core");
-const protocol = require("devtools/server/protocol");
-const {Arg, Option, method, RetVal, types} = protocol;
-const {LongStringActor, ShortLongString} = require("devtools/server/actors/string");
+const protocol = require("devtools/shared/protocol");
+const {LongStringActor} = require("devtools/server/actors/string");
 const {fetch} = require("devtools/shared/DevToolsUtils");
 const {listenOnce} = require("devtools/shared/async-utils");
+const {originalSourceSpec, mediaRuleSpec, styleSheetSpec,
+       styleSheetsSpec} = require("devtools/shared/specs/stylesheets");
 const {SourceMapConsumer} = require("source-map");
+const { installHelperSheet,
+  addPseudoClassLock, removePseudoClassLock } = require("devtools/server/actors/highlighters/utils/markup");
 
-loader.lazyGetter(this, "CssLogic", () => require("devtools/shared/inspector/css-logic").CssLogic);
+loader.lazyGetter(this, "CssLogic", () => require("devtools/shared/inspector/css-logic"));
 
-const {
-  getIndentationFromPrefs,
-  getIndentationFromString
-} = require("devtools/shared/indentation");
+XPCOMUtils.defineLazyGetter(this, "DOMUtils", function () {
+  return Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
+});
 
-var TRANSITION_CLASS = "moz-styleeditor-transitioning";
+var TRANSITION_PSEUDO_CLASS = ":-moz-styleeditor-transitioning";
 var TRANSITION_DURATION_MS = 500;
 var TRANSITION_BUFFER_MS = 1000;
 var TRANSITION_RULE_SELECTOR =
-".moz-styleeditor-transitioning:root, .moz-styleeditor-transitioning:root *";
-var TRANSITION_RULE = TRANSITION_RULE_SELECTOR + " {\
-transition-duration: " + TRANSITION_DURATION_MS + "ms !important; \
-transition-delay: 0ms !important;\
-transition-timing-function: ease-out !important;\
-transition-property: all !important;\
-}";
+`:root${TRANSITION_PSEUDO_CLASS}, :root${TRANSITION_PSEUDO_CLASS} *`;
+var TRANSITION_RULE = `${TRANSITION_RULE_SELECTOR} {
+  transition-duration: ${TRANSITION_DURATION_MS}ms !important;
+  transition-delay: 0ms !important;
+  transition-timing-function: ease-out !important;
+  transition-property: all !important;
+}`;
 
 var LOAD_ERROR = "error-load";
 
-types.addActorType("stylesheet");
-types.addActorType("originalsource");
-
 // The possible kinds of style-applied events.
 // UPDATE_PRESERVING_RULES means that the update is guaranteed to
 // preserve the number and order of rules on the style sheet.
@@ -62,210 +57,55 @@ exports.UPDATE_GENERAL = UPDATE_GENERAL;
 let modifiedStyleSheets = new WeakMap();
 
 /**
- * Creates a StyleSheetsActor. StyleSheetsActor provides remote access to the
- * stylesheets of a document.
+ * Actor representing an original source of a style sheet that was specified
+ * in a source map.
  */
-var StyleSheetsActor = exports.StyleSheetsActor = protocol.ActorClass({
-  typeName: "stylesheets",
-
-  /**
-   * The window we work with, taken from the parent actor.
-   */
-  get window() {
-    return this.parentActor.window;
-  },
-
-  /**
-   * The current content document of the window we work with.
-   */
-  get document() {
-    return this.window.document;
-  },
-
-  form: function()
-  {
-    return { actor: this.actorID };
-  },
-
-  initialize: function (conn, tabActor) {
+var OriginalSourceActor = protocol.ActorClassWithSpec(originalSourceSpec, {
+  initialize: function (aUrl, aSourceMap, aParentActor) {
     protocol.Actor.prototype.initialize.call(this, null);
 
-    this.parentActor = tabActor;
+    this.url = aUrl;
+    this.sourceMap = aSourceMap;
+    this.parentActor = aParentActor;
+    this.conn = this.parentActor.conn;
+
+    this.text = null;
   },
 
-  /**
-   * Protocol method for getting a list of StyleSheetActors representing
-   * all the style sheets in this document.
-   */
-  getStyleSheets: method(Task.async(function* () {
-    // Iframe document can change during load (bug 1171919). Track their windows
-    // instead.
-    let windows = [this.window];
-    let actors = [];
+  form: function () {
+    return {
+      actor: this.actorID, // actorID is set when it's added to a pool
+      url: this.url,
+      relatedStyleSheet: this.parentActor.form()
+    };
+  },
 
-    for (let win of windows) {
-      let sheets = yield this._addStyleSheets(win);
-      actors = actors.concat(sheets);
-
-      // Recursively handle style sheets of the documents in iframes.
-      for (let iframe of win.document.querySelectorAll("iframe, browser, frame")) {
-        if (iframe.contentDocument && iframe.contentWindow) {
-          // Sometimes, iframes don't have any document, like the
-          // one that are over deeply nested (bug 285395)
-          windows.push(iframe.contentWindow);
-        }
-      }
+  _getText: function () {
+    if (this.text) {
+      return promise.resolve(this.text);
     }
-    return actors;
-  }), {
-    request: {},
-    response: { styleSheets: RetVal("array:stylesheet") }
-  }),
-
-  /**
-   * Check if we should be showing this stylesheet.
-   *
-   * @param {Document} doc
-   *        Document for which we're checking
-   * @param {DOMCSSStyleSheet} sheet
-   *        Stylesheet we're interested in
-   *
-   * @return boolean
-   *         Whether the stylesheet should be listed.
-   */
-  _shouldListSheet: function(doc, sheet) {
-    // Special case about:PreferenceStyleSheet, as it is generated on the
-    // fly and the URI is not registered with the about: handler.
-    // https://bugzilla.mozilla.org/show_bug.cgi?id=935803#c37
-    if (sheet.href && sheet.href.toLowerCase() == "about:preferencestylesheet") {
-      return false;
+    let content = this.sourceMap.sourceContentFor(this.url);
+    if (content) {
+      this.text = content;
+      return promise.resolve(content);
     }
-
-    return true;
+    let options = {
+      policy: Ci.nsIContentPolicy.TYPE_INTERNAL_STYLESHEET,
+      window: this.window
+    };
+    return fetch(this.url, options).then(({content}) => {
+      this.text = content;
+      return content;
+    });
   },
 
   /**
-   * Add all the stylesheets for the document in this window to the map and
-   * create an actor for each one if not already created.
-   *
-   * @param {Window} win
-   *        Window for which to add stylesheets
-   *
-   * @return {Promise}
-   *         Promise that resolves to an array of StyleSheetActors
+   * Protocol method to get the text of this source.
    */
-  _addStyleSheets: function(win)
-  {
-    return Task.spawn(function*() {
-      let doc = win.document;
-      // readyState can be uninitialized if an iframe has just been created but
-      // it has not started to load yet.
-      if (doc.readyState === "loading" || doc.readyState === "uninitialized") {
-        // Wait for the document to load first.
-        yield listenOnce(win, "DOMContentLoaded", true);
-
-        // Make sure we have the actual document for this window. If the
-        // readyState was initially uninitialized, the initial dummy document
-        // was replaced with the actual document (bug 1171919).
-        doc = win.document;
-      }
-
-      let isChrome = Services.scriptSecurityManager.isSystemPrincipal(doc.nodePrincipal);
-      let styleSheets = isChrome ? DOMUtils.getAllStyleSheets(doc) : doc.styleSheets;
-      let actors = [];
-      for (let i = 0; i < styleSheets.length; i++) {
-        let sheet = styleSheets[i];
-        if (!this._shouldListSheet(doc, sheet)) {
-          continue;
-        }
-
-        let actor = this.parentActor.createStyleSheetActor(sheet);
-        actors.push(actor);
-
-        // Get all sheets, including imported ones
-        let imports = yield this._getImported(doc, actor);
-        actors = actors.concat(imports);
-      }
-      return actors;
-    }.bind(this));
-  },
-
-  /**
-   * Get all the stylesheets @imported from a stylesheet.
-   *
-   * @param  {Document} doc
-   *         The document including the stylesheet
-   * @param  {DOMStyleSheet} styleSheet
-   *         Style sheet to search
-   * @return {Promise}
-   *         A promise that resolves with an array of StyleSheetActors
-   */
-  _getImported: function(doc, styleSheet) {
-    return Task.spawn(function*() {
-      let rules = yield styleSheet.getCSSRules();
-      let imported = [];
-
-      for (let i = 0; i < rules.length; i++) {
-        let rule = rules[i];
-        if (rule.type == Ci.nsIDOMCSSRule.IMPORT_RULE) {
-          // Associated styleSheet may be null if it has already been seen due
-          // to duplicate @imports for the same URL.
-          if (!rule.styleSheet || !this._shouldListSheet(doc, rule.styleSheet)) {
-            continue;
-          }
-          let actor = this.parentActor.createStyleSheetActor(rule.styleSheet);
-          imported.push(actor);
-
-          // recurse imports in this stylesheet as well
-          let children = yield this._getImported(doc, actor);
-          imported = imported.concat(children);
-        }
-        else if (rule.type != Ci.nsIDOMCSSRule.CHARSET_RULE) {
-          // @import rules must precede all others except @charset
-          break;
-        }
-      }
-
-      return imported;
-    }.bind(this));
-  },
-
-
-  /**
-   * Create a new style sheet in the document with the given text.
-   * Return an actor for it.
-   *
-   * @param  {object} request
-   *         Debugging protocol request object, with 'text property'
-   * @return {object}
-   *         Object with 'styelSheet' property for form on new actor.
-   */
-  addStyleSheet: method(function(text) {
-    let parent = this.document.documentElement;
-    let style = this.document.createElementNS("http://www.w3.org/1999/xhtml", "style");
-    style.setAttribute("type", "text/css");
-
-    if (text) {
-      style.appendChild(this.document.createTextNode(text));
-    }
-    parent.appendChild(style);
-
-    let actor = this.parentActor.createStyleSheetActor(style.sheet);
-    return actor;
-  }, {
-    request: { text: Arg(0, "string") },
-    response: { styleSheet: RetVal("stylesheet") }
-  })
-});
-
-/**
- * The corresponding Front object for the StyleSheetsActor.
- */
-var StyleSheetsFront = protocol.FrontClass(StyleSheetsActor, {
-  initialize: function(client, tabForm) {
-    protocol.Front.prototype.initialize.call(this, client);
-    this.actorID = tabForm.styleSheetsActor;
-    this.manage(this);
+  getText: function () {
+    return this._getText().then((text) => {
+      return new LongStringActor(this.conn, text || "");
+    });
   }
 });
 
@@ -273,16 +113,7 @@ var StyleSheetsFront = protocol.FrontClass(StyleSheetsActor, {
  * A MediaRuleActor lives on the server and provides access to properties
  * of a DOM @media rule and emits events when it changes.
  */
-var MediaRuleActor = protocol.ActorClass({
-  typeName: "mediarule",
-
-  events: {
-    "matches-change" : {
-      type: "matchesChange",
-      matches: Arg(0, "boolean"),
-    }
-  },
-
+var MediaRuleActor = protocol.ActorClassWithSpec(mediaRuleSpec, {
   get window() {
     return this.parentActor.window;
   },
@@ -295,7 +126,7 @@ var MediaRuleActor = protocol.ActorClass({
     return this.mql ? this.mql.matches : null;
   },
 
-  initialize: function(aMediaRule, aParentActor) {
+  initialize: function (aMediaRule, aParentActor) {
     protocol.Actor.prototype.initialize.call(this, null);
 
     this.rawRule = aMediaRule;
@@ -309,7 +140,7 @@ var MediaRuleActor = protocol.ActorClass({
 
     try {
       this.mql = this.window.matchMedia(aMediaRule.media.mediaText);
-    } catch(e) {
+    } catch (e) {
     }
 
     if (this.mql) {
@@ -317,7 +148,7 @@ var MediaRuleActor = protocol.ActorClass({
     }
   },
 
-  destroy: function()
+  destroy: function ()
   {
     if (this.mql) {
       this.mql.removeListener(this._matchesChange);
@@ -326,7 +157,7 @@ var MediaRuleActor = protocol.ActorClass({
     protocol.Actor.prototype.destroy.call(this);
   },
 
-  form: function(detail) {
+  form: function (detail) {
     if (detail === "actorid") {
       return this.actorID;
     }
@@ -344,82 +175,19 @@ var MediaRuleActor = protocol.ActorClass({
     return form;
   },
 
-  _matchesChange: function() {
+  _matchesChange: function () {
     events.emit(this, "matches-change", this.matches);
   }
 });
 
-/**
- * Cooresponding client-side front for a MediaRuleActor.
- */
-var MediaRuleFront = protocol.FrontClass(MediaRuleActor, {
-  initialize: function(client, form) {
-    protocol.Front.prototype.initialize.call(this, client, form);
-
-    this._onMatchesChange = this._onMatchesChange.bind(this);
-    events.on(this, "matches-change", this._onMatchesChange);
-  },
-
-  _onMatchesChange: function(matches) {
-    this._form.matches = matches;
-  },
-
-  form: function(form, detail) {
-    if (detail === "actorid") {
-      this.actorID = form;
-      return;
-    }
-    this.actorID = form.actor;
-    this._form = form;
-  },
-
-  get mediaText() {
-    return this._form.mediaText;
-  },
-  get conditionText() {
-    return this._form.conditionText;
-  },
-  get matches() {
-    return this._form.matches;
-  },
-  get line() {
-    return this._form.line || -1;
-  },
-  get column() {
-    return this._form.column || -1;
-  },
-  get parentStyleSheet() {
-    return this.conn.getActor(this._form.parentStyleSheet);
-  }
-});
-
 /**
  * A StyleSheetActor represents a stylesheet on the server.
  */
-var StyleSheetActor = protocol.ActorClass({
-  typeName: "stylesheet",
-
-  events: {
-    "property-change" : {
-      type: "propertyChange",
-      property: Arg(0, "string"),
-      value: Arg(1, "json")
-    },
-    "style-applied" : {
-      type: "styleApplied",
-      kind: Arg(0, "number"),
-      styleSheet: Arg(1, "stylesheet")
-    },
-    "media-rules-changed" : {
-      type: "mediaRulesChanged",
-      rules: Arg(0, "array:mediarule")
-    }
-  },
-
+var StyleSheetActor = protocol.ActorClassWithSpec(styleSheetSpec, {
   /* List of original sources that generated this stylesheet */
   _originalSources: null,
 
-  toString: function() {
+  toString: function () {
     return "[StyleSheetActor " + this.actorID + "]";
   },
 
@@ -482,7 +250,23 @@ var StyleSheetActor = protocol.ActorClass({
     return this._styleSheetIndex;
   },
 
-  initialize: function(aStyleSheet, aParentActor, aWindow) {
+  destroy: function () {
+    if (this._transitionTimeout) {
+      this.window.clearTimeout(this._transitionTimeout);
+      removePseudoClassLock(
+                   this.document.documentElement, TRANSITION_PSEUDO_CLASS);
+    }
+  },
+
+  /**
+   * Since StyleSheetActor doesn't have a protocol.js parent actor that take
+   * care of its lifetime, implementing disconnect is required to cleanup.
+   */
+  disconnect: function () {
+    this.destroy();
+  },
+
+  initialize: function (aStyleSheet, aParentActor, aWindow) {
     protocol.Actor.prototype.initialize.call(this, null);
 
     this.rawSheet = aStyleSheet;
@@ -494,8 +278,6 @@ var StyleSheetActor = protocol.ActorClass({
     // text and index are unknown until source load
     this.text = null;
     this._styleSheetIndex = -1;
-
-    this._transitionRefCount = 0;
   },
 
   /**
@@ -503,7 +285,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {Boolean} true if all the rules have source; false if
    *         some rule was created via CSSOM.
    */
-  allRulesHaveSource: function() {
+  allRulesHaveSource: function () {
     let rules;
     try {
       rules = this.rawSheet.cssRules;
@@ -528,7 +310,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {Promise}
    *         Promise that resolves with a CSSRuleList
    */
-  getCSSRules: function() {
+  getCSSRules: function () {
     let rules;
     try {
       rules = this.rawSheet.cssRules;
@@ -572,7 +354,7 @@ var StyleSheetActor = protocol.ActorClass({
    *         With properties of the underlying stylesheet, plus 'text',
    *        'styleSheetIndex' and 'parentActor' if it's @imported
    */
-  form: function(detail) {
+  form: function (detail) {
     if (detail === "actorid") {
       return this.actorID;
     }
@@ -595,12 +377,12 @@ var StyleSheetActor = protocol.ActorClass({
       title: this.rawSheet.title,
       system: !CssLogic.isContentStylesheet(this.rawSheet),
       styleSheetIndex: this.styleSheetIndex
-    }
+    };
 
     try {
       form.ruleCount = this.rawSheet.cssRules.length;
     }
-    catch(e) {
+    catch (e) {
       // stylesheet had an @import rule that wasn't loaded yet
       this.getCSSRules().then(() => {
         this._notifyPropertyChanged("ruleCount");
@@ -615,14 +397,12 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {object}
    *         'disabled' - the disabled state after toggling.
    */
-  toggleDisabled: method(function() {
+  toggleDisabled: function () {
     this.rawSheet.disabled = !this.rawSheet.disabled;
     this._notifyPropertyChanged("disabled");
 
     return this.rawSheet.disabled;
-  }, {
-    response: { disabled: RetVal("boolean")}
-  }),
+  },
 
   /**
    * Send an event notifying that a property of the stylesheet
@@ -631,22 +411,18 @@ var StyleSheetActor = protocol.ActorClass({
    * @param  {string} property
    *         Name of the changed property
    */
-  _notifyPropertyChanged: function(property) {
+  _notifyPropertyChanged: function (property) {
     events.emit(this, "property-change", property, this.form()[property]);
   },
 
   /**
    * Protocol method to get the text of this stylesheet.
    */
-  getText: method(function() {
+  getText: function () {
     return this._getText().then((text) => {
       return new LongStringActor(this.conn, text || "");
     });
-  }, {
-    response: {
-      text: RetVal("longstring")
-    }
-  }),
+  },
 
   /**
    * Fetch the text for this stylesheet from the cache or network. Return
@@ -655,7 +431,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {Promise}
    *         Promise that resolves with a string text of the stylesheet.
    */
-  _getText: function() {
+  _getText: function () {
     if (typeof this.text === "string") {
       return promise.resolve(this.text);
     }
@@ -676,10 +452,22 @@ var StyleSheetActor = protocol.ActorClass({
     let options = {
       loadFromCache: true,
       policy: Ci.nsIContentPolicy.TYPE_INTERNAL_STYLESHEET,
-      window: this.window,
       charset: this._getCSSCharset()
     };
 
+    // Bug 1282660 - We use the system principal to load the default internal
+    // stylesheets instead of the content principal since such stylesheets
+    // require system principal to load. At meanwhile, we strip the loadGroup
+    // for preventing the assertion of the userContextId mismatching.
+    // The default internal stylesheets load from the 'resource:' URL.
+    // Bug 1287607 - The 'chrome:' URL will be also loaded from here, so we do
+    // the same thing for such URLs as well.
+    if (!/^resource:\/\//.test(this.href) &&
+        !/^chrome:\/\//.test(this.href)) {
+      options.window = this.window;
+      options.principal = this.document.nodePrincipal;
+    }
+
     return fetch(this.href, options).then(({ content }) => {
       this.text = content;
       return content;
@@ -690,17 +478,12 @@ var StyleSheetActor = protocol.ActorClass({
    * Protocol method to get the original source (actors) for this
    * stylesheet if it has uses source maps.
    */
-  getOriginalSources: method(function() {
+  getOriginalSources: function () {
     if (this._originalSources) {
       return promise.resolve(this._originalSources);
     }
     return this._fetchOriginalSources();
-  }, {
-    request: {},
-    response: {
-      originalSources: RetVal("nullable:array:originalsource")
-    }
-  }),
+  },
 
   /**
    * Fetch the original sources (actors) for this style sheet using its
@@ -709,7 +492,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {Promise}
    *         Promise that resolves with an array of OriginalSourceActors
    */
-  _fetchOriginalSources: function() {
+  _fetchOriginalSources: function () {
     this._clearOriginalSources();
     this._originalSources = [];
 
@@ -724,7 +507,7 @@ var StyleSheetActor = protocol.ActorClass({
         this._originalSources.push(actor);
       }
       return this._originalSources;
-    })
+    });
   },
 
   /**
@@ -734,7 +517,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {Promise}
    *         A promise that resolves with a SourceMapConsumer, or null.
    */
-  getSourceMap: function() {
+  getSourceMap: function () {
     if (this._sourceMap) {
       return this._sourceMap;
     }
@@ -747,7 +530,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {Promise}
    *         A promise that resolves with a SourceMapConsumer, or null.
    */
-  _fetchSourceMap: function() {
+  _fetchSourceMap: function () {
     let deferred = promise.defer();
 
     this._getText().then(sheetContent => {
@@ -792,7 +575,7 @@ var StyleSheetActor = protocol.ActorClass({
   /**
    * Clear and unmanage the original source actors for this stylesheet.
    */
-  _clearOriginalSources: function() {
+  _clearOriginalSources: function () {
     for (actor in this._originalSources) {
       this.unmanage(actor);
     }
@@ -802,7 +585,10 @@ var StyleSheetActor = protocol.ActorClass({
   /**
    * Sets the source map's sourceRoot to be relative to the source map url.
    */
-  _setSourceMapRoot: function(aSourceMap, aAbsSourceMapURL, aScriptURL) {
+  _setSourceMapRoot: function (aSourceMap, aAbsSourceMapURL, aScriptURL) {
+    if (aScriptURL.startsWith("blob:")) {
+      aScriptURL = aScriptURL.replace("blob:", "");
+    }
     const base = dirname(
       aAbsSourceMapURL.startsWith("data:")
         ? aScriptURL
@@ -820,7 +606,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {string}
    *         Url of source map.
    */
-  _extractSourceMapUrl: function(content) {
+  _extractSourceMapUrl: function (content) {
     var matches = /sourceMappingURL\=([^\s\*]*)/.exec(content);
     if (matches) {
       return matches[1];
@@ -833,7 +619,7 @@ var StyleSheetActor = protocol.ActorClass({
    * line, column pair in this stylesheet, if its source mapped, otherwise
    * a promise of the same location.
    */
-  getOriginalLocation: method(function(line, column) {
+  getOriginalLocation: function (line, column) {
     return this.getSourceMap().then((sourceMap) => {
       if (sourceMap) {
         return sourceMap.originalPositionFor({ line: line, column: column });
@@ -845,29 +631,14 @@ var StyleSheetActor = protocol.ActorClass({
         column: column
       };
     });
-  }, {
-    request: {
-      line: Arg(0, "number"),
-      column: Arg(1, "number")
-    },
-    response: RetVal(types.addDictType("originallocationresponse", {
-      source: "string",
-      line: "number",
-      column: "number"
-    }))
-  }),
+  },
 
   /**
    * Protocol method to get the media rules for the stylesheet.
    */
-  getMediaRules: method(function() {
+  getMediaRules: function () {
     return this._getMediaRules();
-  }, {
-    request: {},
-    response: {
-      mediaRules: RetVal("nullable:array:mediarule")
-    }
-  }),
+  },
 
   /**
    * Get all the @media rules in this stylesheet.
@@ -875,7 +646,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @return {promise}
    *         A promise that resolves with an array of MediaRuleActors.
    */
-  _getMediaRules: function() {
+  _getMediaRules: function () {
     return this.getCSSRules().then((rules) => {
       let mediaRules = [];
       for (let i = 0; i < rules.length; i++) {
@@ -899,7 +670,7 @@ var StyleSheetActor = protocol.ActorClass({
    * @param string channelCharset
    *        Charset of the source string if set by the HTTP channel.
    */
-  _getCSSCharset: function(channelCharset)
+  _getCSSCharset: function (channelCharset)
   {
     // StyleSheet's charset can be specified from multiple sources
     if (channelCharset && channelCharset.length > 0) {
@@ -952,7 +723,7 @@ var StyleSheetActor = protocol.ActorClass({
    *         'transition' - whether to do CSS transition for change.
    *         'kind' - either UPDATE_PRESERVING_RULES or UPDATE_GENERAL
    */
-  update: method(function(text, transition, kind = UPDATE_GENERAL) {
+  update: function (text, transition, kind = UPDATE_GENERAL) {
     DOMUtils.parseStyleSheet(this.rawSheet, text);
 
     modifiedStyleSheets.set(this.rawSheet, text);
@@ -971,19 +742,14 @@ var StyleSheetActor = protocol.ActorClass({
     this._getMediaRules().then((rules) => {
       events.emit(this, "media-rules-changed", rules);
     });
-  }, {
-    request: {
-      text: Arg(0, "string"),
-      transition: Arg(1, "boolean")
-    }
-  }),
+  },
 
   /**
    * Insert a catch-all transition rule into the document. Set a timeout
    * to remove the rule after a certain time.
    */
-  _insertTransistionRule: function(kind) {
-    this.document.documentElement.classList.add(TRANSITION_CLASS);
+  _insertTransistionRule: function (kind) {
+    addPseudoClassLock(this.document.documentElement, TRANSITION_PSEUDO_CLASS);
 
     // We always add the rule since we've just reset all the rules
     this.rawSheet.insertRule(TRANSITION_RULE, this.rawSheet.cssRules.length);
@@ -999,9 +765,10 @@ var StyleSheetActor = protocol.ActorClass({
    * This cleans up class and rule added for transition effect and then
    * notifies that the style has been applied.
    */
-  _onTransitionEnd: function(kind)
+  _onTransitionEnd: function (kind)
   {
-    this.document.documentElement.classList.remove(TRANSITION_CLASS);
+    this._transitionTimeout = null;
+    removePseudoClassLock(this.document.documentElement, TRANSITION_PSEUDO_CLASS);
 
     let index = this.rawSheet.cssRules.length - 1;
     let rule = this.rawSheet.cssRules[index];
@@ -1011,180 +778,200 @@ var StyleSheetActor = protocol.ActorClass({
 
     events.emit(this, "style-applied", kind, this);
   }
-})
+});
+
+exports.StyleSheetActor = StyleSheetActor;
 
 /**
- * StyleSheetFront is the client-side counterpart to a StyleSheetActor.
+ * Creates a StyleSheetsActor. StyleSheetsActor provides remote access to the
+ * stylesheets of a document.
  */
-var StyleSheetFront = protocol.FrontClass(StyleSheetActor, {
-  initialize: function(conn, form) {
-    protocol.Front.prototype.initialize.call(this, conn, form);
-
-    this._onPropertyChange = this._onPropertyChange.bind(this);
-    events.on(this, "property-change", this._onPropertyChange);
-  },
-
-  destroy: function() {
-    events.off(this, "property-change", this._onPropertyChange);
-    protocol.Front.prototype.destroy.call(this);
-  },
-
-  _onPropertyChange: function(property, value) {
-    this._form[property] = value;
-  },
-
-  form: function(form, detail) {
-    if (detail === "actorid") {
-      this.actorID = form;
-      return;
-    }
-    this.actorID = form.actor;
-    this._form = form;
-  },
-
-  get href() {
-    return this._form.href;
-  },
-  get nodeHref() {
-    return this._form.nodeHref;
-  },
-  get disabled() {
-    return !!this._form.disabled;
-  },
-  get title() {
-    return this._form.title;
-  },
-  get isSystem() {
-    return this._form.system;
-  },
-  get styleSheetIndex() {
-    return this._form.styleSheetIndex;
-  },
-  get ruleCount() {
-    return this._form.ruleCount;
+var StyleSheetsActor = protocol.ActorClassWithSpec(styleSheetsSpec, {
+  /**
+   * The window we work with, taken from the parent actor.
+   */
+  get window() {
+    return this.parentActor.window;
   },
 
   /**
-   * Get the indentation to use for edits to this style sheet.
-   *
-   * @return {Promise} A promise that will resolve to a string that
-   * should be used to indent a block in this style sheet.
+   * The current content document of the window we work with.
    */
-  guessIndentation: function() {
-    let prefIndent = getIndentationFromPrefs();
-    if (prefIndent) {
-      let {indentUnit, indentWithTabs} = prefIndent;
-      return promise.resolve(indentWithTabs ? "\t" : " ".repeat(indentUnit));
-    }
+  get document() {
+    return this.window.document;
+  },
 
-    return Task.spawn(function*() {
-      let longStr = yield this.getText();
-      let source = yield longStr.string();
+  form: function ()
+  {
+    return { actor: this.actorID };
+  },
 
-      let {indentUnit, indentWithTabs} = getIndentationFromString(source);
-
-      return indentWithTabs ? "\t" : " ".repeat(indentUnit);
-    }.bind(this));
-  }
-});
-
-/**
- * Actor representing an original source of a style sheet that was specified
- * in a source map.
- */
-var OriginalSourceActor = protocol.ActorClass({
-  typeName: "originalsource",
-
-  initialize: function(aUrl, aSourceMap, aParentActor) {
+  initialize: function (conn, tabActor) {
     protocol.Actor.prototype.initialize.call(this, null);
 
-    this.url = aUrl;
-    this.sourceMap = aSourceMap;
-    this.parentActor = aParentActor;
-    this.conn = this.parentActor.conn;
-
-    this.text = null;
-  },
-
-  form: function() {
-    return {
-      actor: this.actorID, // actorID is set when it's added to a pool
-      url: this.url,
-      relatedStyleSheet: this.parentActor.form()
-    };
-  },
-
-  _getText: function() {
-    if (this.text) {
-      return promise.resolve(this.text);
-    }
-    let content = this.sourceMap.sourceContentFor(this.url);
-    if (content) {
-      this.text = content;
-      return promise.resolve(content);
-    }
-    let options = {
-      policy: Ci.nsIContentPolicy.TYPE_INTERNAL_STYLESHEET,
-      window: this.window
-    };
-    return fetch(this.url, options).then(({content}) => {
-      this.text = content;
-      return content;
-    });
+    this.parentActor = tabActor;
   },
 
   /**
-   * Protocol method to get the text of this source.
+   * Protocol method for getting a list of StyleSheetActors representing
+   * all the style sheets in this document.
    */
-  getText: method(function() {
-    return this._getText().then((text) => {
-      return new LongStringActor(this.conn, text || "");
-    });
-  }, {
-    response: {
-      text: RetVal("longstring")
+  getStyleSheets: Task.async(function* () {
+    // Iframe document can change during load (bug 1171919). Track their windows
+    // instead.
+    let windows = [this.window];
+    let actors = [];
+
+    for (let win of windows) {
+      let sheets = yield this._addStyleSheets(win);
+      actors = actors.concat(sheets);
+
+      // Recursively handle style sheets of the documents in iframes.
+      for (let iframe of win.document.querySelectorAll("iframe, browser, frame")) {
+        if (iframe.contentDocument && iframe.contentWindow) {
+          // Sometimes, iframes don't have any document, like the
+          // one that are over deeply nested (bug 285395)
+          windows.push(iframe.contentWindow);
+        }
+      }
     }
-  })
-})
+    return actors;
+  }),
 
-/**
- * The client-side counterpart for an OriginalSourceActor.
- */
-var OriginalSourceFront = protocol.FrontClass(OriginalSourceActor, {
-  initialize: function(client, form) {
-    protocol.Front.prototype.initialize.call(this, client, form);
-
-    this.isOriginalSource = true;
-  },
-
-  form: function(form, detail) {
-    if (detail === "actorid") {
-      this.actorID = form;
-      return;
+  /**
+   * Check if we should be showing this stylesheet.
+   *
+   * @param {Document} doc
+   *        Document for which we're checking
+   * @param {DOMCSSStyleSheet} sheet
+   *        Stylesheet we're interested in
+   *
+   * @return boolean
+   *         Whether the stylesheet should be listed.
+   */
+  _shouldListSheet: function (doc, sheet) {
+    // Special case about:PreferenceStyleSheet, as it is generated on the
+    // fly and the URI is not registered with the about: handler.
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=935803#c37
+    if (sheet.href && sheet.href.toLowerCase() == "about:preferencestylesheet") {
+      return false;
     }
-    this.actorID = form.actor;
-    this._form = form;
+
+    return true;
   },
 
-  get href() {
-    return this._form.url;
+  /**
+   * Add all the stylesheets for the document in this window to the map and
+   * create an actor for each one if not already created.
+   *
+   * @param {Window} win
+   *        Window for which to add stylesheets
+   *
+   * @return {Promise}
+   *         Promise that resolves to an array of StyleSheetActors
+   */
+  _addStyleSheets: function (win)
+  {
+    return Task.spawn(function* () {
+      let doc = win.document;
+      // readyState can be uninitialized if an iframe has just been created but
+      // it has not started to load yet.
+      if (doc.readyState === "loading" || doc.readyState === "uninitialized") {
+        // Wait for the document to load first.
+        yield listenOnce(win, "DOMContentLoaded", true);
+
+        // Make sure we have the actual document for this window. If the
+        // readyState was initially uninitialized, the initial dummy document
+        // was replaced with the actual document (bug 1171919).
+        doc = win.document;
+      }
+
+      let isChrome = Services.scriptSecurityManager.isSystemPrincipal(doc.nodePrincipal);
+      let styleSheets = isChrome ? DOMUtils.getAllStyleSheets(doc) : doc.styleSheets;
+      let actors = [];
+      for (let i = 0; i < styleSheets.length; i++) {
+        let sheet = styleSheets[i];
+        if (!this._shouldListSheet(doc, sheet)) {
+          continue;
+        }
+
+        let actor = this.parentActor.createStyleSheetActor(sheet);
+        actors.push(actor);
+
+        // Get all sheets, including imported ones
+        let imports = yield this._getImported(doc, actor);
+        actors = actors.concat(imports);
+      }
+      return actors;
+    }.bind(this));
   },
-  get url() {
-    return this._form.url;
+
+  /**
+   * Get all the stylesheets @imported from a stylesheet.
+   *
+   * @param  {Document} doc
+   *         The document including the stylesheet
+   * @param  {DOMStyleSheet} styleSheet
+   *         Style sheet to search
+   * @return {Promise}
+   *         A promise that resolves with an array of StyleSheetActors
+   */
+  _getImported: function (doc, styleSheet) {
+    return Task.spawn(function* () {
+      let rules = yield styleSheet.getCSSRules();
+      let imported = [];
+
+      for (let i = 0; i < rules.length; i++) {
+        let rule = rules[i];
+        if (rule.type == Ci.nsIDOMCSSRule.IMPORT_RULE) {
+          // Associated styleSheet may be null if it has already been seen due
+          // to duplicate @imports for the same URL.
+          if (!rule.styleSheet || !this._shouldListSheet(doc, rule.styleSheet)) {
+            continue;
+          }
+          let actor = this.parentActor.createStyleSheetActor(rule.styleSheet);
+          imported.push(actor);
+
+          // recurse imports in this stylesheet as well
+          let children = yield this._getImported(doc, actor);
+          imported = imported.concat(children);
+        }
+        else if (rule.type != Ci.nsIDOMCSSRule.CHARSET_RULE) {
+          // @import rules must precede all others except @charset
+          break;
+        }
+      }
+
+      return imported;
+    }.bind(this));
+  },
+
+
+  /**
+   * Create a new style sheet in the document with the given text.
+   * Return an actor for it.
+   *
+   * @param  {object} request
+   *         Debugging protocol request object, with 'text property'
+   * @return {object}
+   *         Object with 'styelSheet' property for form on new actor.
+   */
+  addStyleSheet: function (text) {
+    let parent = this.document.documentElement;
+    let style = this.document.createElementNS("http://www.w3.org/1999/xhtml", "style");
+    style.setAttribute("type", "text/css");
+
+    if (text) {
+      style.appendChild(this.document.createTextNode(text));
+    }
+    parent.appendChild(style);
+
+    let actor = this.parentActor.createStyleSheetActor(style.sheet);
+    return actor;
   }
 });
 
-
-XPCOMUtils.defineLazyGetter(this, "DOMUtils", function () {
-  return Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);
-});
-
 exports.StyleSheetsActor = StyleSheetsActor;
-exports.StyleSheetsFront = StyleSheetsFront;
-
-exports.StyleSheetActor = StyleSheetActor;
-exports.StyleSheetFront = StyleSheetFront;
-
 
 /**
  * Normalize multiple relative paths towards the base paths on the right.
