@@ -13,6 +13,7 @@ Cc["@mozilla.org/psm;1"].getService(Ci.nsISupports);
 
 var Services = require("Services");
 var promise = require("promise");
+var defer = require("devtools/shared/defer");
 var DevToolsUtils = require("devtools/shared/DevToolsUtils");
 var { dumpn, dumpv } = DevToolsUtils;
 loader.lazyRequireGetter(this, "DebuggerTransport",
@@ -27,8 +28,6 @@ loader.lazyRequireGetter(this, "Authenticators",
   "devtools/shared/security/auth", true);
 loader.lazyRequireGetter(this, "AuthenticationResult",
   "devtools/shared/security/auth", true);
-loader.lazyRequireGetter(this, "setTimeout", "Timer", true);
-loader.lazyRequireGetter(this, "clearTimeout", "Timer", true);
 
 DevToolsUtils.defineLazyGetter(this, "nsFile", () => {
   return CC("@mozilla.org/file/local;1", "nsIFile", "initWithPath");
@@ -49,8 +48,7 @@ DevToolsUtils.defineLazyGetter(this, "nssErrorsService", () => {
          .getService(Ci.nsINSSErrorsService);
 });
 
-DevToolsUtils.defineLazyModuleGetter(this, "Task",
-  "resource://gre/modules/Task.jsm");
+const { Task } = require("devtools/shared/task");
 
 var DebuggerSocket = {};
 
@@ -71,7 +69,7 @@ var DebuggerSocket = {};
  * @return promise
  *         Resolved to a DebuggerTransport instance.
  */
-DebuggerSocket.connect = Task.async(function*(settings) {
+DebuggerSocket.connect = Task.async(function* (settings) {
   // Default to PROMPT |Authenticator| instance if not supplied
   if (!settings.authenticator) {
     settings.authenticator = new (Authenticators.get().Client)();
@@ -111,7 +109,7 @@ DebuggerSocket.connect = Task.async(function*(settings) {
  * @return s nsISocketTransport
  *         Underlying socket transport, in case more details are needed.
  */
-var _getTransport = Task.async(function*(settings) {
+var _getTransport = Task.async(function* (settings) {
   let { host, port, encryption } = settings;
   let attempt = yield _attemptTransport(settings);
   if (attempt.transport) {
@@ -159,7 +157,7 @@ var _getTransport = Task.async(function*(settings) {
  * @return s nsISocketTransport
  *         Underlying socket transport, in case more details are needed.
  */
-var _attemptTransport = Task.async(function*(settings) {
+var _attemptTransport = Task.async(function* (settings) {
   let { authenticator } = settings;
   // _attemptConnect only opens the streams.  Any failures at that stage
   // aborts the connection process immedidately.
@@ -172,7 +170,7 @@ var _attemptTransport = Task.async(function*(settings) {
     let results = yield _isInputAlive(input);
     alive = results.alive;
     certError = results.certError;
-  } catch(e) {
+  } catch (e) {
     // For other unexpected errors, like NS_ERROR_CONNECTION_REFUSED, we reach
     // this block.
     input.close();
@@ -217,7 +215,7 @@ var _attemptTransport = Task.async(function*(settings) {
  * @return output nsIAsyncOutputStream
  *         The socket's output stream.
  */
-var _attemptConnect = Task.async(function*({ host, port, encryption }) {
+var _attemptConnect = Task.async(function* ({ host, port, encryption }) {
   let s;
   if (encryption) {
     s = socketTransportService.createTransport(["ssl"], 1, host, port, null);
@@ -236,7 +234,7 @@ var _attemptConnect = Task.async(function*({ host, port, encryption }) {
     clientCert = yield cert.local.getOrCreate();
   }
 
-  let deferred = promise.defer();
+  let deferred = defer();
   let input;
   let output;
   // Delay opening the input stream until the transport has fully connected.
@@ -258,7 +256,7 @@ var _attemptConnect = Task.async(function*({ host, port, encryption }) {
       }
       try {
         input = s.openInputStream(0, 0, 0);
-      } catch(e) {
+      } catch (e) {
         deferred.reject(e);
       }
       deferred.resolve({ s, input, output });
@@ -270,7 +268,7 @@ var _attemptConnect = Task.async(function*({ host, port, encryption }) {
   // the call to this method.
   try {
     output = s.openOutputStream(0, 0, 0);
-  } catch(e) {
+  } catch (e) {
     deferred.reject(e);
   }
 
@@ -293,7 +291,7 @@ var _attemptConnect = Task.async(function*({ host, port, encryption }) {
  * first connection to a new host because the cert is self-signed.
  */
 function _isInputAlive(input) {
-  let deferred = promise.defer();
+  let deferred = defer();
   input.asyncWait({
     onInputStreamReady(stream) {
       try {
@@ -373,7 +371,7 @@ SocketListener.prototype = {
   /**
    * Validate that all options have been set to a supported configuration.
    */
-  _validateOptions: function() {
+  _validateOptions: function () {
     if (this.portOrPath === null) {
       throw new Error("Must set a port / path to listen on.");
     }
@@ -386,7 +384,7 @@ SocketListener.prototype = {
   /**
    * Listens on the given port or socket file for remote debugger connections.
    */
-  open: function() {
+  open: function () {
     this._validateOptions();
     DebuggerServer._addListener(this);
 
@@ -397,7 +395,7 @@ SocketListener.prototype = {
     }
 
     let self = this;
-    return Task.spawn(function*() {
+    return Task.spawn(function* () {
       let backlog = 4;
       self._socket = self._createSocketInstance();
       if (self.isPortBased) {
@@ -422,7 +420,7 @@ SocketListener.prototype = {
     });
   },
 
-  _advertise: function() {
+  _advertise: function () {
     if (!this.discoverable || !this.port) {
       return;
     }
@@ -437,7 +435,7 @@ SocketListener.prototype = {
     discovery.addService("devtools", advertisement);
   },
 
-  _createSocketInstance: function() {
+  _createSocketInstance: function () {
     if (this.encryption) {
       return Cc["@mozilla.org/network/tls-server-socket;1"]
              .createInstance(Ci.nsITLSServerSocket);
@@ -446,7 +444,7 @@ SocketListener.prototype = {
            .createInstance(Ci.nsIServerSocket);
   },
 
-  _setAdditionalSocketOptions: Task.async(function*() {
+  _setAdditionalSocketOptions: Task.async(function* () {
     if (this.encryption) {
       this._socket.serverCert = yield cert.local.getOrCreate();
       this._socket.setSessionCache(false);
@@ -461,7 +459,7 @@ SocketListener.prototype = {
    * Closes the SocketListener.  Notifies the server to remove the listener from
    * the set of active SocketListeners.
    */
-  close: function() {
+  close: function () {
     if (this.discoverable && this.port) {
       discovery.removeService("devtools");
     }
@@ -512,11 +510,11 @@ SocketListener.prototype = {
   // nsIServerSocketListener implementation
 
   onSocketAccepted:
-  DevToolsUtils.makeInfallible(function(socket, socketTransport) {
+  DevToolsUtils.makeInfallible(function (socket, socketTransport) {
     new ServerSocketConnection(this, socketTransport);
   }, "SocketListener.onSocketAccepted"),
 
-  onStopListening: function(socket, status) {
+  onStopListening: function (socket, status) {
     dumpn("onStopListening, status: " + status);
   }
 
@@ -596,7 +594,7 @@ ServerSocketConnection.prototype = {
   _handle() {
     dumpn("Debugging connection starting authentication on " + this.address);
     let self = this;
-    Task.spawn(function*() {
+    Task.spawn(function* () {
       self._listenForTLSHandshake();
       self._createTransport();
       yield self._awaitTLSHandshake();
@@ -641,7 +639,7 @@ ServerSocketConnection.prototype = {
    * |onHandshakeDone|.
    */
   _listenForTLSHandshake() {
-    this._handshakeDeferred = promise.defer();
+    this._handshakeDeferred = defer();
     if (!this._listener.encryption) {
       this._handshakeDeferred.resolve();
       return;
@@ -685,7 +683,7 @@ ServerSocketConnection.prototype = {
     this._handshakeDeferred.resolve();
   },
 
-  _authenticate: Task.async(function*() {
+  _authenticate: Task.async(function* () {
     let result = yield this._listener.authenticator.authenticate({
       client: this.client,
       server: this.server,
@@ -746,7 +744,7 @@ ServerSocketConnection.prototype = {
 
 };
 
-DebuggerSocket.createListener = function() {
+DebuggerSocket.createListener = function () {
   return new SocketListener();
 };
 
