@@ -6,12 +6,15 @@
 
 #include "nsDOMMutationObserver.h"
 
+#include "mozilla/AnimationTarget.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/OwningNonNull.h"
 
 #include "mozilla/dom/Animation.h"
 #include "mozilla/dom/KeyframeEffect.h"
 
 #include "nsContentUtils.h"
+#include "nsCSSPseudoElements.h"
 #include "nsError.h"
 #include "nsIDOMMutationEvent.h"
 #include "nsIScriptGlobalObject.h"
@@ -19,8 +22,12 @@
 #include "nsTextFragment.h"
 #include "nsThreadUtils.h"
 
+using mozilla::Maybe;
+using mozilla::Move;
+using mozilla::NonOwningAnimationTarget;
 using mozilla::dom::TreeOrderComparator;
 using mozilla::dom::Animation;
+using mozilla::dom::Element;
 
 AutoTArray<RefPtr<nsDOMMutationObserver>, 4>*
   nsDOMMutationObserver::sScheduledMutationObservers = nullptr;
@@ -393,14 +400,14 @@ nsAnimationReceiver::RecordAnimationMutation(Animation* aAnimation,
     return;
   }
 
-  dom::Element* elem = animationTarget->mElement;
+  Element* elem = animationTarget->mElement;
   if (!Animations() || !(Subtree() || elem == Target()) ||
       elem->ChromeOnlyAccess()) {
     return;
   }
 
   // Record animations targeting to a pseudo element only when subtree is true.
-  if (animationTarget->mPseudoType != CSSPseudoElementType::NotPseudo &&
+  if (animationTarget->mPseudoType != mozilla::CSSPseudoElementType::NotPseudo &&
       !Subtree()) {
     return;
   }
@@ -623,15 +630,11 @@ nsDOMMutationObserver::Observe(nsINode& aTarget,
   bool attributeOldValue =
     aOptions.mAttributeOldValue.WasPassed() &&
     aOptions.mAttributeOldValue.Value();
-  bool nativeAnonymousChildList = aOptions.mNativeAnonymousChildList &&
-    nsContentUtils::ThreadsafeIsCallerChrome();
+  bool nativeAnonymousChildList = aOptions.mNativeAnonymousChildList;
   bool characterDataOldValue =
     aOptions.mCharacterDataOldValue.WasPassed() &&
     aOptions.mCharacterDataOldValue.Value();
-  bool animations =
-    aOptions.mAnimations.WasPassed() &&
-    aOptions.mAnimations.Value() &&
-    nsContentUtils::ThreadsafeIsCallerChrome();
+  bool animations = aOptions.mAnimations;
 
   if (!aOptions.mAttributes.WasPassed() &&
       (aOptions.mAttributeOldValue.WasPassed() ||
@@ -683,8 +686,7 @@ nsDOMMutationObserver::Observe(nsINode& aTarget,
     filters.SetCapacity(len);
 
     for (uint32_t i = 0; i < len; ++i) {
-      nsCOMPtr<nsIAtom> a = NS_Atomize(filtersAsString[i]);
-      filters.AppendObject(a);
+      filters.AppendElement(NS_Atomize(filtersAsString[i]));
     }
   }
 
@@ -696,7 +698,7 @@ nsDOMMutationObserver::Observe(nsINode& aTarget,
   r->SetAttributeOldValue(attributeOldValue);
   r->SetCharacterDataOldValue(characterDataOldValue);
   r->SetNativeAnonymousChildList(nativeAnonymousChildList);
-  r->SetAttributeFilter(filters);
+  r->SetAttributeFilter(Move(filters));
   r->SetAllAttributes(allAttrs);
   r->SetAnimations(animations);
   r->RemoveClones();
@@ -757,7 +759,7 @@ nsDOMMutationObserver::GetObservingInfo(
     info.mAttributeOldValue.Construct(mr->AttributeOldValue());
     info.mCharacterDataOldValue.Construct(mr->CharacterDataOldValue());
     info.mNativeAnonymousChildList = mr->NativeAnonymousChildList();
-    info.mAnimations.Construct(mr->Animations());
+    info.mAnimations = mr->Animations();
     nsCOMArray<nsIAtom>& filters = mr->AttributeFilter();
     if (filters.Count()) {
       info.mAttributeFilter.Construct();
@@ -883,6 +885,11 @@ nsDOMMutationObserver::HandleMutationsInternal()
     return;
   }
 
+  // We need the AutoSafeJSContext to ensure the slow script dialog is
+  // triggered. AutoSafeJSContext does that by pushing JSAutoRequest to stack.
+  // This needs to be outside the while loop.
+  AutoSafeJSContext cx;
+
   nsTArray<RefPtr<nsDOMMutationObserver> >* suppressedObservers = nullptr;
 
   while (sScheduledMutationObservers) {
@@ -903,6 +910,7 @@ nsDOMMutationObserver::HandleMutationsInternal()
       }
     }
     delete observers;
+    JS_CheckForInterrupt(cx);
   }
 
   if (suppressedObservers) {
