@@ -250,6 +250,11 @@ class nsObjectLoadingContent : public nsImageLoadingContent
       return runID;
     }
 
+    bool IsRewrittenYoutubeEmbed() const
+    {
+      return mRewrittenYoutubeEmbed;
+    }
+
   protected:
     /**
      * Begins loading the object when called
@@ -440,6 +445,11 @@ class nsObjectLoadingContent : public nsImageLoadingContent
      */
     nsresult CloseChannel();
 
+    /**
+     * If this object should be tested against blocking list.
+     */
+    bool ShouldBlockContent();
+
     /**
      * If this object is allowed to play plugin content, or if it would display
      * click-to-play instead.
@@ -577,6 +587,9 @@ class nsObjectLoadingContent : public nsImageLoadingContent
                                       JS::MutableHandle<JSObject*> plugin_obj,
                                       JS::MutableHandle<JSObject*> plugin_proto);
 
+    // Utility for firing an error event, if we're an <object>.
+    void MaybeFireErrorEvent();
+
     // The final listener for mChannel (uriloader, pluginstreamlistener, etc.)
     nsCOMPtr<nsIStreamListener> mFinalListener;
 
@@ -625,7 +638,7 @@ class nsObjectLoadingContent : public nsImageLoadingContent
     FallbackType                mFallbackType : 8;
 
     uint32_t                    mRunID;
-    bool                        mHasRunID;
+    bool                        mHasRunID : 1;
 
     // If true, we have opened a channel as the listener and it has reached
     // OnStartRequest. Does not get set for channels that are passed directly to
@@ -645,6 +658,9 @@ class nsObjectLoadingContent : public nsImageLoadingContent
     // activated by PlayPlugin(). (see ShouldPlay())
     bool                        mActivated : 1;
 
+    // Whether content blocking is enabled or not for this object.
+    bool                        mContentBlockingDisabled : 1;
+
     // Protects DoStopPlugin from reentry (bug 724781).
     bool                        mIsStopping : 1;
 
@@ -655,6 +671,12 @@ class nsObjectLoadingContent : public nsImageLoadingContent
     // whether content js has tried to access the plugin script object.
     bool                        mScriptRequested : 1;
 
+    // True if object represents an object/embed tag pointing to a flash embed
+    // for a youtube video. When possible (see IsRewritableYoutubeEmbed function
+    // comments for details), we change these to try to load HTML5 versions of
+    // videos.
+    bool                        mRewrittenYoutubeEmbed : 1;
+
     nsWeakFrame                 mPrintFrame;
 
     RefPtr<nsPluginInstanceOwner> mInstanceOwner;
