@@ -570,7 +570,7 @@ WebGLContext::GetBufferSlotByTarget(GLenum target)
         return mBoundUniformBuffer;
 
     default:
-        MOZ_CRASH("Should not get here.");
+        MOZ_CRASH("GFX: Should not get here.");
     }
 }
 
@@ -588,7 +588,7 @@ WebGLContext::GetBufferSlotByTargetIndexed(GLenum target, GLuint index)
         return mBoundUniformBuffers[index];
 
     default:
-        MOZ_CRASH("Should not get here.");
+        MOZ_CRASH("GFX: Should not get here.");
     }
 }
 
