@@ -138,7 +138,6 @@ public:
     return NS_OK;
   }
 
-  virtual JSContext* GetJSContextForEventHandlers() override;
   virtual nsIPrincipal* GetPrincipal() override;
   virtual JSObject* GetGlobalJSObject() override;
 
@@ -194,12 +193,12 @@ public:
   // Get the Document for the top-level window in this tab.
   already_AddRefed<nsIDocument> GetDocument() const;
 
-protected:
-  virtual ~TabChildBase();
-
   // Get the pres-shell of the document for the top-level window in this tab.
   already_AddRefed<nsIPresShell> GetPresShell() const;
 
+protected:
+  virtual ~TabChildBase();
+
   // Wraps up a JSON object as a structured clone and sends it to the browser
   // chrome script.
   //
@@ -228,7 +227,8 @@ class TabChild final : public TabChildBase,
                        public nsITabChild,
                        public nsIObserver,
                        public TabContext,
-                       public nsITooltipListener
+                       public nsITooltipListener,
+                       public mozilla::ipc::IShmemAllocator
 {
   typedef mozilla::dom::ClonedMessageData ClonedMessageData;
   typedef mozilla::layout::RenderFrameChild RenderFrameChild;
@@ -265,8 +265,6 @@ public:
   Create(nsIContentChild* aManager, const TabId& aTabId,
          const TabContext& aContext, uint32_t aChromeFlags);
 
-  bool IsRootContentDocument() const;
-
   // Let managees query if it is safe to send messages.
   bool IsDestroyed() const{ return mDestroyed; }
 
@@ -287,6 +285,8 @@ public:
   NS_DECL_NSIOBSERVER
   NS_DECL_NSITOOLTIPLISTENER
 
+  FORWARD_SHMEM_ALLOCATOR_TO(PBrowserChild)
+
   /**
    * MessageManagerCallback methods that we override.
    */
@@ -310,7 +310,6 @@ public:
                           const Maybe<ZoomConstraints>& aConstraints) override;
 
   virtual bool RecvLoadURL(const nsCString& aURI,
-                           const BrowserConfiguration& aConfiguration,
                            const ShowInfo& aInfo) override;
 
   virtual bool RecvCacheFileDescriptor(const nsString& aPath,
@@ -452,6 +451,10 @@ public:
 
   virtual bool DeallocPColorPickerChild(PColorPickerChild* aActor) override;
 
+    virtual PDatePickerChild*
+    AllocPDatePickerChild(const nsString& title, const nsString& initialDate) override;
+    virtual bool DeallocPDatePickerChild(PDatePickerChild* actor) override;
+
   virtual PFilePickerChild*
   AllocPFilePickerChild(const nsString& aTitle, const int16_t& aMode) override;
 
@@ -476,6 +479,8 @@ public:
 
   void GetDefaultScale(double *aScale);
 
+  bool IsTransparent() const { return mIsTransparent; }
+
   void GetMaxTouchPoints(uint32_t* aTouchPoints);
 
   ScreenOrientationInternal GetOrientation() const { return mOrientation; }
@@ -566,8 +571,8 @@ public:
   virtual bool
   RecvThemeChanged(nsTArray<LookAndFeelInt>&& aLookAndFeelIntCache) override;
 
-  virtual bool RecvHandleAccessKey(nsTArray<uint32_t>&& aCharCodes,
-                                   const bool& aIsTrusted,
+  virtual bool RecvHandleAccessKey(const WidgetKeyboardEvent& aEvent,
+                                   nsTArray<uint32_t>&& aCharCodes,
                                    const int32_t& aModifierMask) override;
 
   virtual bool RecvAudioChannelChangeNotification(const uint32_t& aAudioChannel,
@@ -580,6 +585,9 @@ public:
                  const mozilla::NativeEventData& aKeyEventData,
                  const bool& aIsConsumed) override;
 
+  virtual bool RecvPrint(const uint64_t& aOuterWindowID,
+                         const PrintData& aPrintData) override;
+
   /**
    * Native widget remoting protocol for use with windowed plugins with e10s.
    */
@@ -614,17 +622,12 @@ public:
                                  bool aPreventDefault) const;
   void SetTargetAPZC(uint64_t aInputBlockId,
                     const nsTArray<ScrollableLayerGuid>& aTargets) const;
-  void HandleDoubleTap(const CSSPoint& aPoint,
-                       const Modifiers& aModifiers,
-                       const mozilla::layers::ScrollableLayerGuid& aGuid);
-  void HandleSingleTap(const CSSPoint& aPoint,
-                       const Modifiers& aModifiers,
-                       const mozilla::layers::ScrollableLayerGuid& aGuid,
-                       bool aCallTakeFocusForClickFromTap);
-  void HandleLongTap(const CSSPoint& aPoint,
-                     const Modifiers& aModifiers,
-                     const mozilla::layers::ScrollableLayerGuid& aGuid,
-                     const uint64_t& aInputBlockId);
+  void HandleTap(layers::GeckoContentController::TapType aType,
+                 const LayoutDevicePoint& aPoint,
+                 const Modifiers& aModifiers,
+                 const mozilla::layers::ScrollableLayerGuid& aGuid,
+                 const uint64_t& aInputBlockId,
+                 bool aCallTakeFocusForClickFromTap);
   void SetAllowedTouchBehavior(uint64_t aInputBlockId,
                                const nsTArray<TouchBehaviorFlags>& aFlags) const;
 
@@ -666,6 +669,9 @@ protected:
 
   virtual bool RecvParentActivated(const bool& aActivated) override;
 
+  virtual bool RecvSetKeyboardIndicators(const UIStateChangeType& aShowAccelerators,
+                                         const UIStateChangeType& aShowFocusRings) override;
+
   virtual bool RecvStopIMEStateManagement() override;
 
   virtual bool RecvMenuKeyboardListenerInstalled(
@@ -676,13 +682,19 @@ protected:
 #endif
 
 private:
+  void HandleDoubleTap(const CSSPoint& aPoint, const Modifiers& aModifiers,
+                       const ScrollableLayerGuid& aGuid);
+
   // Notify others that our TabContext has been updated.  (At the moment, this
-  // sets the appropriate app-id and is-browser flags on our docshell.)
+  // sets the appropriate origin attributes on our docshell.)
   //
   // You should call this after calling TabContext::SetTabContext().  We also
   // call this during Init().
   void NotifyTabContextUpdated();
 
+  // Update the frameType on our docshell.
+  void UpdateFrameType();
+
   void ActorDestroy(ActorDestroyReason why) override;
 
   enum FrameScriptLoading { DONT_LOAD_SCRIPTS, DEFAULT_LOAD_SCRIPTS };
@@ -751,6 +763,8 @@ private:
   float mDPI;
   double mDefaultScale;
 
+  bool mIsTransparent;
+
   bool mIPCOpen;
   bool mParentIsActive;
   bool mAsyncPanZoomEnabled;
