@@ -112,9 +112,9 @@ PresentationNetworkHelper::OnGetWifiIPAddress(const nsACString& aIPAddress)
   MOZ_ASSERT(mFunc);
 
   NS_DispatchToMainThread(
-    NS_NewRunnableMethodWithArg<nsCString>(mInfo,
-                                           mFunc,
-                                           aIPAddress));
+    NewRunnableMethod<nsCString>(mInfo,
+                                 mFunc,
+                                 aIPAddress));
   return NS_OK;
 }
 
@@ -226,7 +226,7 @@ PresentationSessionInfo::Shutdown(nsresult aReason)
 
   // Close the control channel if any.
   if (mControlChannel) {
-    NS_WARN_IF(NS_FAILED(mControlChannel->Close(aReason)));
+    NS_WARN_IF(NS_FAILED(mControlChannel->Disconnect(aReason)));
   }
 
   // Close the data transport channel if any.
@@ -236,13 +236,18 @@ PresentationSessionInfo::Shutdown(nsresult aReason)
   }
 
   mIsResponderReady = false;
+  mIsOnTerminating = false;
 
-  mBuilder = nullptr;
+  SetBuilder(nullptr);
 }
 
 nsresult
 PresentationSessionInfo::SetListener(nsIPresentationSessionListener* aListener)
 {
+  if (mListener && aListener) {
+    NS_WARN_IF(NS_FAILED(mListener->NotifyReplaced()));
+  }
+
   mListener = aListener;
 
   if (mListener) {
@@ -256,7 +261,7 @@ PresentationSessionInfo::SetListener(nsIPresentationSessionListener* aListener)
 
     // The transport might become ready, or might become un-ready again, before
     // the listener has registered. So notify the listener of the state change.
-    return mListener->NotifyStateChange(mSessionId, mState);
+    return mListener->NotifyStateChange(mSessionId, mState, mReason);
   }
 
   return NS_OK;
@@ -284,22 +289,49 @@ PresentationSessionInfo::Close(nsresult aReason,
     return NS_ERROR_DOM_INVALID_STATE_ERR;
   }
 
-  SetState(aState);
+  // Do nothing if session is already terminated.
+  if (nsIPresentationSessionListener::STATE_TERMINATED == mState) {
+    return NS_OK;
+  }
+
+  SetStateWithReason(aState, aReason);
+
+  switch (aState) {
+    case nsIPresentationSessionListener::STATE_CLOSED: {
+      Shutdown(aReason);
+      break;
+    }
+    case nsIPresentationSessionListener::STATE_TERMINATED: {
+      if (!mControlChannel) {
+        nsCOMPtr<nsIPresentationControlChannel> ctrlChannel;
+        nsresult rv = mDevice->EstablishControlChannel(getter_AddRefs(ctrlChannel));
+        if (NS_SUCCEEDED(rv)) {
+          SetControlChannel(ctrlChannel);
+        }
+        return rv;
+      }
+
+      return mControlChannel->Terminate(mSessionId);
+    }
+  }
+
+  return NS_OK;
+}
+
+nsresult
+PresentationSessionInfo::OnTerminate(nsIPresentationControlChannel* aControlChannel)
+{
+  mIsOnTerminating = true; // Mark for terminating transport channel
+  SetStateWithReason(nsIPresentationSessionListener::STATE_TERMINATED, NS_OK);
+  SetControlChannel(aControlChannel);
 
-  Shutdown(aReason);
   return NS_OK;
 }
 
 nsresult
 PresentationSessionInfo::ReplySuccess()
 {
-  SetState(nsIPresentationSessionListener::STATE_CONNECTED);
-
-  if (mCallback) {
-    NS_WARN_IF(NS_FAILED(mCallback->NotifySuccess()));
-    SetCallback(nullptr);
-  }
-
+  SetStateWithReason(nsIPresentationSessionListener::STATE_CONNECTED, NS_OK);
   return NS_OK;
 }
 
@@ -308,11 +340,6 @@ PresentationSessionInfo::ReplyError(nsresult aError)
 {
   Shutdown(aError);
 
-  if (mCallback) {
-    NS_WARN_IF(NS_FAILED(mCallback->NotifyError(aError)));
-    SetCallback(nullptr);
-  }
-
   // Remove itself since it never succeeds.
   return UntrackFromService();
 }
@@ -353,6 +380,18 @@ PresentationSessionInfo::IsAccessible(base::ProcessId aProcessId)
   return true;
 }
 
+void
+PresentationSessionInfo::ContinueTermination()
+{
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(mControlChannel);
+
+  if (NS_WARN_IF(NS_FAILED(mControlChannel->Terminate(mSessionId)))
+      || mIsOnTerminating) {
+    Shutdown(NS_OK);
+  }
+}
+
 // nsIPresentationSessionTransportCallback
 NS_IMETHODIMP
 PresentationSessionInfo::NotifyTransportReady()
@@ -361,6 +400,11 @@ PresentationSessionInfo::NotifyTransportReady()
 
   mIsTransportReady = true;
 
+  // Established RTCDataChannel implies responder is ready.
+  if (mTransportType == nsIPresentationChannelDescription::TYPE_DATACHANNEL) {
+    mIsResponderReady = true;
+  }
+
   // At sender side, session might not be ready at this point (waiting for
   // receiver's answer). Yet at receiver side, session must be ready at this
   // point since the data transport channel is created after the receiver page
@@ -381,7 +425,8 @@ PresentationSessionInfo::NotifyTransportClosed(nsresult aReason)
   // potential subsequent |Shutdown| calls.
   mTransport = nullptr;
 
-  if (NS_WARN_IF(!IsSessionReady())) {
+  if (NS_WARN_IF(!IsSessionReady() &&
+                 mState == nsIPresentationSessionListener::STATE_CONNECTING)) {
     // It happens before the session is ready. Reply the callback.
     return ReplyError(NS_ERROR_DOM_OPERATION_ERR);
   }
@@ -391,7 +436,7 @@ PresentationSessionInfo::NotifyTransportClosed(nsresult aReason)
 
   if (mState == nsIPresentationSessionListener::STATE_CONNECTED) {
     // The transport channel is closed unexpectedly (not caused by a |Close| call).
-    SetState(nsIPresentationSessionListener::STATE_CLOSED);
+    SetStateWithReason(nsIPresentationSessionListener::STATE_CLOSED, aReason);
   }
 
   Shutdown(aReason);
@@ -424,6 +469,13 @@ PresentationSessionInfo::NotifyData(const nsACString& aData)
 NS_IMETHODIMP
 PresentationSessionInfo::OnSessionTransport(nsIPresentationSessionTransport* transport)
 {
+  SetBuilder(nullptr);
+
+  // The session transport is managed by content process
+  if (!transport) {
+    return NS_OK;
+  }
+
   mTransport = transport;
 
   nsresult rv = mTransport->SetCallback(this);
@@ -441,9 +493,34 @@ PresentationSessionInfo::OnSessionTransport(nsIPresentationSessionTransport* tra
 NS_IMETHODIMP
 PresentationSessionInfo::OnError(nsresult reason)
 {
+  SetBuilder(nullptr);
   return ReplyError(reason);
 }
 
+NS_IMETHODIMP
+PresentationSessionInfo::SendOffer(nsIPresentationChannelDescription* aOffer)
+{
+  return mControlChannel->SendOffer(aOffer);
+}
+
+NS_IMETHODIMP
+PresentationSessionInfo::SendAnswer(nsIPresentationChannelDescription* aAnswer)
+{
+  return mControlChannel->SendAnswer(aAnswer);
+}
+
+NS_IMETHODIMP
+PresentationSessionInfo::SendIceCandidate(const nsAString& candidate)
+{
+  return mControlChannel->SendIceCandidate(candidate);
+}
+
+NS_IMETHODIMP
+PresentationSessionInfo::Close(nsresult reason)
+{
+  return mControlChannel->Disconnect(reason);
+}
+
 /**
  * Implementation of PresentationControllingInfo
  *
@@ -452,7 +529,7 @@ PresentationSessionInfo::OnError(nsresult reason)
  * 4 is not guaranteed.)
  * 1. |Init| is called to open a socket |mServerSocket| for data transport
  *    channel.
- * 2. |NotifyOpened| of |nsIPresentationControlChannelListener| is called to
+ * 2. |NotifyConnected| of |nsIPresentationControlChannelListener| is called to
  *    indicate the control channel is ready to use. Then send the offer to the
  *    receiver via the control channel.
  * 3.1 |OnSocketAccepted| of |nsIServerSocketListener| is called to indicate the
@@ -550,7 +627,7 @@ PresentationControllingInfo::GetAddress()
   // To make consistent code sequence, following function call is dispatched
   // into main thread instead of calling it directly.
   NS_DispatchToMainThread(
-    NS_NewRunnableMethodWithArg<nsCString>(
+    NewRunnableMethod<nsCString>(
       this,
       &PresentationControllingInfo::OnGetAddress,
       NS_ConvertUTF16toUTF8(ip)));
@@ -570,7 +647,7 @@ PresentationControllingInfo::GetAddress()
 #elif defined(MOZ_MULET)
   // In simulator,we need to use the "127.0.0.1" as target address.
   NS_DispatchToMainThread(
-    NS_NewRunnableMethodWithArg<nsCString>(
+    NewRunnableMethod<nsCString>(
       this,
       &PresentationControllingInfo::OnGetAddress,
       "127.0.0.1"));
@@ -579,7 +656,7 @@ PresentationControllingInfo::GetAddress()
   // TODO Get host IP via other platforms.
 
   NS_DispatchToMainThread(
-    NS_NewRunnableMethodWithArg<nsCString>(
+    NewRunnableMethod<nsCString>(
       this,
       &PresentationControllingInfo::OnGetAddress,
       EmptyCString()));
@@ -588,13 +665,6 @@ PresentationControllingInfo::GetAddress()
   return NS_OK;
 }
 
-NS_IMETHODIMP
-PresentationControllingInfo::OnIceCandidate(const nsAString& aCandidate)
-{
-  MOZ_ASSERT(false, "Should not receive ICE candidates.");
-  return NS_ERROR_FAILURE;
-}
-
 nsresult
 PresentationControllingInfo::OnGetAddress(const nsACString& aAddress)
 {
@@ -613,6 +683,23 @@ PresentationControllingInfo::OnGetAddress(const nsACString& aAddress)
 }
 
 // nsIPresentationControlChannelListener
+NS_IMETHODIMP
+PresentationControllingInfo::OnIceCandidate(const nsAString& aCandidate)
+{
+  if (mTransportType != nsIPresentationChannelDescription::TYPE_DATACHANNEL) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+    builder = do_QueryInterface(mBuilder);
+
+  if (NS_WARN_IF(!builder)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return builder->OnIceCandidate(aCandidate);
+}
+
 NS_IMETHODIMP
 PresentationControllingInfo::OnOffer(nsIPresentationChannelDescription* aDescription)
 {
@@ -623,10 +710,21 @@ PresentationControllingInfo::OnOffer(nsIPresentationChannelDescription* aDescrip
 NS_IMETHODIMP
 PresentationControllingInfo::OnAnswer(nsIPresentationChannelDescription* aDescription)
 {
+  if (mTransportType == nsIPresentationChannelDescription::TYPE_DATACHANNEL) {
+    nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+      builder = do_QueryInterface(mBuilder);
+
+    if (NS_WARN_IF(!builder)) {
+      return NS_ERROR_FAILURE;
+    }
+
+    return builder->OnAnswer(aDescription);
+  }
+
   mIsResponderReady = true;
 
   // Close the control channel since it's no longer needed.
-  nsresult rv = mControlChannel->Close(NS_OK);
+  nsresult rv = mControlChannel->Disconnect(NS_OK);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return ReplyError(NS_ERROR_DOM_OPERATION_ERR);
   }
@@ -641,37 +739,115 @@ PresentationControllingInfo::OnAnswer(nsIPresentationChannelDescription* aDescri
 }
 
 NS_IMETHODIMP
-PresentationControllingInfo::NotifyOpened()
+PresentationControllingInfo::NotifyConnected()
 {
   MOZ_ASSERT(NS_IsMainThread());
 
+  switch (mState) {
+    case nsIPresentationSessionListener::STATE_CONNECTING: {
+      nsresult rv = mControlChannel->Launch(GetSessionId(), GetUrl());
+      if (NS_WARN_IF(NS_FAILED(rv))) {
+        return rv;
+      }
+      Unused << NS_WARN_IF(NS_FAILED(BuildTransport()));
+      break;
+    }
+    case nsIPresentationSessionListener::STATE_TERMINATED: {
+      ContinueTermination();
+      break;
+    }
+    default:
+      break;
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PresentationControllingInfo::NotifyReconnected()
+{
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(mReconnectCallback);
+
+  if (NS_WARN_IF(mState == nsIPresentationSessionListener::STATE_TERMINATED)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  SetStateWithReason(nsIPresentationSessionListener::STATE_CONNECTING, NS_OK);
+  return mReconnectCallback->NotifySuccess();
+}
+
+nsresult
+PresentationControllingInfo::BuildTransport()
+{
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (mState != nsIPresentationSessionListener::STATE_CONNECTING) {
+    return NS_OK;
+  }
+
   if (!Preferences::GetBool("dom.presentation.session_transport.data_channel.enable")) {
     // Build TCP session transport
     return GetAddress();
   }
 
-  nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder> builder =
-    do_CreateInstance("@mozilla.org/presentation/datachanneltransportbuilder;1");
-
-  if (NS_WARN_IF(!builder)) {
-    return NS_ERROR_NOT_AVAILABLE;
+  nsPIDOMWindowInner* window = nullptr;
+  /**
+   * Generally transport is maintained by the chrome process. However, data
+   * channel should be live with the DOM , which implies RTCDataChannel in an OOP
+   * page should be establish in the content process.
+   *
+   * In OOP data channel transport case, |mBuilder| is hooked when the content
+   * process is ready to build a data channel transport, trigger by:
+   * 1. PresentationIPCService::StartSession (sender)
+   * 2. PresentationIPCService::NotifyReceiverReady (receiver).
+   *
+   * In this case, |mBuilder| would be an object of |PresentationBuilderParent|
+   * and set previously. Therefore, |BuildDataChannelTransport| triggers an IPC
+   * call to make content process establish a RTCDataChannel transport.
+   */
+  // in-process case
+  if (!mBuilder) {
+    nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder> builder =
+      do_CreateInstance("@mozilla.org/presentation/datachanneltransportbuilder;1");
+    if (NS_WARN_IF(!builder)) {
+      return NS_ERROR_NOT_AVAILABLE;
+    }
+    SetBuilder(builder);
+    // OOP window would be set from content process
+    window = GetWindow();
   }
-
-  mBuilder = builder;
+  // OOP case
   mTransportType = nsIPresentationChannelDescription::TYPE_DATACHANNEL;
 
-  return builder->BuildDataChannelTransport(nsIPresentationService::ROLE_CONTROLLER,
-                                            GetWindow(),
-                                            mControlChannel,
-                                            this);
-
+  nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+    dataChannelBuilder(do_QueryInterface(mBuilder));
+  if (NS_WARN_IF(!dataChannelBuilder)) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  nsresult rv = dataChannelBuilder->
+         BuildDataChannelTransport(nsIPresentationService::ROLE_CONTROLLER,
+                                   window,
+                                   this);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
+  }
+  return NS_OK;
 }
 
 NS_IMETHODIMP
-PresentationControllingInfo::NotifyClosed(nsresult aReason)
+PresentationControllingInfo::NotifyDisconnected(nsresult aReason)
 {
   MOZ_ASSERT(NS_IsMainThread());
 
+  if (mTransportType == nsIPresentationChannelDescription::TYPE_DATACHANNEL) {
+    nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+      builder = do_QueryInterface(mBuilder);
+    if (builder) {
+      NS_WARN_IF(NS_FAILED(builder->NotifyDisconnected(aReason)));
+    }
+  }
+
   // Unset control channel here so it won't try to re-close it in potential
   // subsequent |Shutdown| calls.
   SetControlChannel(nullptr);
@@ -679,7 +855,7 @@ PresentationControllingInfo::NotifyClosed(nsresult aReason)
   if (NS_WARN_IF(NS_FAILED(aReason) || !mIsResponderReady)) {
     // The presentation session instance may already exist.
     // Change the state to TERMINATED since it never succeeds.
-    SetState(nsIPresentationSessionListener::STATE_TERMINATED);
+    SetStateWithReason(nsIPresentationSessionListener::STATE_TERMINATED, aReason);
 
     // Reply error for an abnormal close.
     return ReplyError(NS_ERROR_DOM_OPERATION_ERR);
@@ -730,7 +906,42 @@ PresentationControllingInfo::OnStopListening(nsIServerSocket* aServerSocket,
   }
 
   // It happens after the session is ready. Change the state to CLOSED.
-  SetState(nsIPresentationSessionListener::STATE_CLOSED);
+  SetStateWithReason(nsIPresentationSessionListener::STATE_CLOSED, aStatus);
+
+  return NS_OK;
+}
+
+nsresult
+PresentationControllingInfo::Reconnect(nsIPresentationServiceCallback* aCallback)
+{
+  if (!aCallback) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  mReconnectCallback = aCallback;
+
+  if (NS_WARN_IF(mState == nsIPresentationSessionListener::STATE_TERMINATED)) {
+    return mReconnectCallback->NotifyError(NS_ERROR_DOM_INVALID_STATE_ERR);
+  }
+
+  nsresult rv = NS_OK;
+  if (!mControlChannel) {
+    nsCOMPtr<nsIPresentationControlChannel> ctrlChannel;
+    rv = mDevice->EstablishControlChannel(getter_AddRefs(ctrlChannel));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return mReconnectCallback->NotifyError(NS_ERROR_DOM_OPERATION_ERR);
+    }
+
+    rv = Init(ctrlChannel);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return mReconnectCallback->NotifyError(NS_ERROR_DOM_OPERATION_ERR);
+    }
+  }
+
+  rv = mControlChannel->Reconnect(mSessionId, GetUrl());
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return mReconnectCallback->NotifyError(rv);
+  }
 
   return NS_OK;
 }
@@ -795,6 +1006,7 @@ PresentationPresentingInfo::Shutdown(nsresult aReason)
   mDevice = nullptr;
   mLoadingCallback = nullptr;
   mRequesterDescription = nullptr;
+  mPendingCandidates.Clear();
   mPromise = nullptr;
 }
 
@@ -808,6 +1020,11 @@ PresentationPresentingInfo::OnSessionTransport(nsIPresentationSessionTransport*
     return rv;
   }
 
+  // The session transport is managed by content process
+  if (!transport) {
+    return NS_OK;
+  }
+
   // send answer for TCP session transport
   if (mTransportType == nsIPresentationChannelDescription::TYPE_TCP) {
     // Prepare and send the answer.
@@ -834,10 +1051,26 @@ PresentationPresentingInfo::OnSessionTransport(nsIPresentationSessionTransport*
   return NS_OK;
 }
 
+// Delegate the pending offer and ICE candidates to builder.
 NS_IMETHODIMP
-PresentationPresentingInfo::OnError(nsresult reason)
+PresentationPresentingInfo::FlushPendingEvents(nsIPresentationDataChannelSessionTransportBuilder* builder)
 {
-  return PresentationSessionInfo::OnError(reason);
+  if (NS_WARN_IF(!builder)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  mHasFlushPendingEvents = true;
+
+  if (mRequesterDescription) {
+    builder->OnOffer(mRequesterDescription);
+  }
+  mRequesterDescription = nullptr;
+
+  for (size_t i = 0; i < mPendingCandidates.Length(); ++i) {
+    builder->OnIceCandidate(mPendingCandidates[i]);
+  }
+  mPendingCandidates.Clear();
+  return NS_OK;
 }
 
 nsresult
@@ -860,7 +1093,7 @@ PresentationPresentingInfo::InitTransportAndSendAnswer()
       return NS_ERROR_NOT_AVAILABLE;
     }
 
-    mBuilder = builder;
+    SetBuilder(builder);
     mTransportType = nsIPresentationChannelDescription::TYPE_TCP;
     return builder->BuildTCPReceiverTransport(mRequesterDescription, this);
   }
@@ -869,31 +1102,57 @@ PresentationPresentingInfo::InitTransportAndSendAnswer()
     if (!Preferences::GetBool("dom.presentation.session_transport.data_channel.enable")) {
       return NS_ERROR_NOT_IMPLEMENTED;
     }
-    nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder> builder =
-      do_CreateInstance("@mozilla.org/presentation/datachanneltransportbuilder;1");
+    nsPIDOMWindowInner* window = nullptr;
 
-    if (NS_WARN_IF(!builder)) {
+    /**
+     * Generally transport is maintained by the chrome process. However, data
+     * channel should be live with the DOM , which implies RTCDataChannel in an OOP
+     * page should be establish in the content process.
+     *
+     * In OOP data channel transport case, |mBuilder| is hooked when the content
+     * process is ready to build a data channel transport, trigger by:
+     * 1. PresentationIPCService::StartSession (sender)
+     * 2. PresentationIPCService::NotifyReceiverReady (receiver).
+     *
+     * In this case, |mBuilder| would be an object of |PresentationBuilderParent|
+     * and set previously. Therefore, |BuildDataChannelTransport| triggers an IPC
+     * call to make content process establish a RTCDataChannel transport.
+     */
+    // in-process case
+    if (!mBuilder) {
+      nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder> builder =
+        do_CreateInstance("@mozilla.org/presentation/datachanneltransportbuilder;1");
+
+      if (NS_WARN_IF(!builder)) {
+        return NS_ERROR_NOT_AVAILABLE;
+      }
+
+      SetBuilder(builder);
+
+      // OOP window would be set from content process
+      window = GetWindow();
+    }
+    mTransportType = nsIPresentationChannelDescription::TYPE_DATACHANNEL;
+
+    nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+      dataChannelBuilder(do_QueryInterface(mBuilder));
+    if (NS_WARN_IF(!dataChannelBuilder)) {
       return NS_ERROR_NOT_AVAILABLE;
     }
-
-    mBuilder = builder;
-    mTransportType = nsIPresentationChannelDescription::TYPE_DATACHANNEL;
-    rv = builder->BuildDataChannelTransport(nsIPresentationService::ROLE_RECEIVER,
-                                            GetWindow(),
-                                            mControlChannel,
-                                            this);
+    rv = dataChannelBuilder->
+           BuildDataChannelTransport(nsIPresentationService::ROLE_RECEIVER,
+                                     window,
+                                     this);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
 
-    // delegate |onOffer| to builder
-    nsCOMPtr<nsIPresentationControlChannelListener> listener(do_QueryInterface(builder));
 
-    if (NS_WARN_IF(!listener)) {
-      return NS_ERROR_NOT_AVAILABLE;
+    rv = this->FlushPendingEvents(dataChannelBuilder);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
     }
-
-    return listener->OnOffer(mRequesterDescription);
+    return NS_OK;
   }
 
   MOZ_ASSERT(false, "Unknown nsIPresentationChannelDescription type!");
@@ -954,6 +1213,10 @@ PresentationPresentingInfo::NotifyResponderReady()
 NS_IMETHODIMP
 PresentationPresentingInfo::OnOffer(nsIPresentationChannelDescription* aDescription)
 {
+  if (NS_WARN_IF(mHasFlushPendingEvents)) {
+    return ReplyError(NS_ERROR_DOM_OPERATION_ERR);
+  }
+
   if (NS_WARN_IF(!aDescription)) {
     return ReplyError(NS_ERROR_DOM_OPERATION_ERR);
   }
@@ -982,22 +1245,51 @@ PresentationPresentingInfo::OnAnswer(nsIPresentationChannelDescription* aDescrip
 NS_IMETHODIMP
 PresentationPresentingInfo::OnIceCandidate(const nsAString& aCandidate)
 {
-  MOZ_ASSERT(false, "Should not receive ICE candidates.");
-  return NS_ERROR_FAILURE;
+  if (!mBuilder && !mHasFlushPendingEvents) {
+    mPendingCandidates.AppendElement(nsString(aCandidate));
+    return NS_OK;
+  }
+
+  if (NS_WARN_IF(!mBuilder && mHasFlushPendingEvents)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+    builder = do_QueryInterface(mBuilder);
+
+  return builder->OnIceCandidate(aCandidate);
 }
 
 NS_IMETHODIMP
-PresentationPresentingInfo::NotifyOpened()
+PresentationPresentingInfo::NotifyConnected()
 {
-  // Do nothing.
+  if (nsIPresentationSessionListener::STATE_TERMINATED == mState) {
+    ContinueTermination();
+  }
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
-PresentationPresentingInfo::NotifyClosed(nsresult aReason)
+PresentationPresentingInfo::NotifyReconnected()
+{
+  MOZ_ASSERT(false, "NotifyReconnected should not be called at receiver side.");
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PresentationPresentingInfo::NotifyDisconnected(nsresult aReason)
 {
   MOZ_ASSERT(NS_IsMainThread());
 
+  if (mTransportType == nsIPresentationChannelDescription::TYPE_DATACHANNEL) {
+    nsCOMPtr<nsIPresentationDataChannelSessionTransportBuilder>
+      builder = do_QueryInterface(mBuilder);
+    if (builder) {
+      NS_WARN_IF(NS_FAILED(builder->NotifyDisconnected(aReason)));
+    }
+  }
+
   // Unset control channel here so it won't try to re-close it in potential
   // subsequent |Shutdown| calls.
   SetControlChannel(nullptr);
@@ -1005,7 +1297,7 @@ PresentationPresentingInfo::NotifyClosed(nsresult aReason)
   if (NS_WARN_IF(NS_FAILED(aReason))) {
     // The presentation session instance may already exist.
     // Change the state to TERMINATED since it never succeeds.
-    SetState(nsIPresentationSessionListener::STATE_TERMINATED);
+    SetStateWithReason(nsIPresentationSessionListener::STATE_TERMINATED, aReason);
 
     // Reply error for an abnormal close.
     return ReplyError(NS_ERROR_DOM_OPERATION_ERR);
