@@ -7,7 +7,6 @@
 #include "QuotaManagerService.h"
 
 #include "ActorsChild.h"
-#include "mozIApplicationClearPrivateDataParams.h"
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/Hal.h"
@@ -25,7 +24,7 @@
 #include "QuotaManager.h"
 #include "QuotaRequests.h"
 
-#define PROFILE_BEFORE_CHANGE_OBSERVER_ID "profile-before-change"
+#define PROFILE_BEFORE_CHANGE_QM_OBSERVER_ID "profile-before-change-qm"
 
 namespace mozilla {
 namespace dom {
@@ -258,7 +257,7 @@ QuotaManagerService::NoteLiveManager(QuotaManager* aManager)
 }
 
 void
-QuotaManagerService::NoteFinishedManager()
+QuotaManagerService::NoteShuttingDownManager()
 {
   MOZ_ASSERT(XRE_IsParentProcess());
   MOZ_ASSERT(NS_IsMainThread());
@@ -295,9 +294,10 @@ QuotaManagerService::Init()
       return NS_ERROR_FAILURE;
     }
 
-    nsresult rv = observerService->AddObserver(this,
-                                               PROFILE_BEFORE_CHANGE_OBSERVER_ID,
-                                               false);
+    nsresult rv =
+      observerService->AddObserver(this,
+                                   PROFILE_BEFORE_CHANGE_QM_OBSERVER_ID,
+                                   false);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -638,39 +638,20 @@ QuotaManagerService::Observe(nsISupports* aSubject,
   MOZ_ASSERT(XRE_IsParentProcess());
   MOZ_ASSERT(NS_IsMainThread());
 
-  if (!strcmp(aTopic, PROFILE_BEFORE_CHANGE_OBSERVER_ID)) {
+  if (!strcmp(aTopic, PROFILE_BEFORE_CHANGE_QM_OBSERVER_ID)) {
     RemoveIdleObserver();
     return NS_OK;
   }
 
-  if (!strcmp(aTopic, TOPIC_WEB_APP_CLEAR_DATA)) {
-    nsCOMPtr<mozIApplicationClearPrivateDataParams> params =
-      do_QueryInterface(aSubject);
-    if (NS_WARN_IF(!params)) {
-      return NS_ERROR_UNEXPECTED;
-    }
-
-    uint32_t appId;
-    nsresult rv = params->GetAppId(&appId);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return rv;
-    }
-
-    bool browserOnly;
-    rv = params->GetBrowserOnly(&browserOnly);
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return rv;
-    }
-
+  if (!strcmp(aTopic, "clear-origin-data")) {
     RefPtr<Request> request = new Request();
 
-    ClearAppParams requestParams;
-    requestParams.appId() = appId;
-    requestParams.browserOnly() = browserOnly;
+    ClearOriginsParams requestParams;
+    requestParams.pattern() = nsDependentString(aData);
 
     nsAutoPtr<PendingRequestInfo> info(new RequestInfo(request, requestParams));
 
-    rv = InitiateRequest(info);
+    nsresult rv = InitiateRequest(info);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
