@@ -14,12 +14,14 @@
 #if defined(XP_UNIX)
 #include "unistd.h"
 #include "dirent.h"
+#include "poll.h"
 #include "sys/stat.h"
 #if defined(ANDROID)
 #include <sys/vfs.h>
 #define statvfs statfs
 #else
 #include "sys/statvfs.h"
+#include "sys/wait.h"
 #include <spawn.h>
 #endif // defined(ANDROID)
 #endif // defined(XP_UNIX)
@@ -405,6 +407,9 @@ static const dom::ConstantSpec gLibcProperties[] =
 {
   // Arguments for open
   INT_CONSTANT(O_APPEND),
+#if defined(O_CLOEXEC)
+  INT_CONSTANT(O_CLOEXEC),
+#endif // defined(O_CLOEXEC)
   INT_CONSTANT(O_CREAT),
 #if defined(O_DIRECTORY)
   INT_CONSTANT(O_DIRECTORY),
@@ -442,6 +447,10 @@ static const dom::ConstantSpec gLibcProperties[] =
   INT_CONSTANT(O_TRUNC),
   INT_CONSTANT(O_WRONLY),
 
+#if defined(FD_CLOEXEC)
+  INT_CONSTANT(FD_CLOEXEC),
+#endif // defined(FD_CLOEXEC)
+
 #if defined(AT_EACCESS)
   INT_CONSTANT(AT_EACCESS),
 #endif //defined(AT_EACCESS)
@@ -483,16 +492,44 @@ static const dom::ConstantSpec gLibcProperties[] =
   INT_CONSTANT(SEEK_END),
   INT_CONSTANT(SEEK_SET),
 
- // fcntl command values
 #if defined(XP_UNIX)
+  // poll
+  INT_CONSTANT(POLLERR),
+  INT_CONSTANT(POLLHUP),
+  INT_CONSTANT(POLLIN),
+  INT_CONSTANT(POLLNVAL),
+  INT_CONSTANT(POLLOUT),
+
+  // wait
+#if defined(WNOHANG)
+  INT_CONSTANT(WNOHANG),
+#endif // defined(WNOHANG)
+
+  // fcntl command values
   INT_CONSTANT(F_GETLK),
+  INT_CONSTANT(F_SETFD),
+  INT_CONSTANT(F_SETFL),
   INT_CONSTANT(F_SETLK),
   INT_CONSTANT(F_SETLKW),
 
- // flock type values
+  // flock type values
   INT_CONSTANT(F_RDLCK),
   INT_CONSTANT(F_WRLCK),
   INT_CONSTANT(F_UNLCK),
+
+  // splice
+#if defined(SPLICE_F_MOVE)
+  INT_CONSTANT(SPLICE_F_MOVE),
+#endif // defined(SPLICE_F_MOVE)
+#if defined(SPLICE_F_NONBLOCK)
+  INT_CONSTANT(SPLICE_F_NONBLOCK),
+#endif // defined(SPLICE_F_NONBLOCK)
+#if defined(SPLICE_F_MORE)
+  INT_CONSTANT(SPLICE_F_MORE),
+#endif // defined(SPLICE_F_MORE)
+#if defined(SPLICE_F_GIFT)
+  INT_CONSTANT(SPLICE_F_GIFT),
+#endif // defined(SPLICE_F_GIFT)
 #endif // defined(XP_UNIX)
   // copyfile
 #if defined(COPYFILE_DATA)
