@@ -560,9 +560,9 @@ Telephony::HandleAudioAgentState()
     }
   } else if (!activeCall.IsNull() && !mIsAudioStartPlaying) {
     mIsAudioStartPlaying = true;
-    float volume;
-    bool muted;
-    rv = mAudioAgent->NotifyStartedPlaying(&volume, &muted);
+    AudioPlaybackConfig config;
+    rv = mAudioAgent->NotifyStartedPlaying(&config,
+                                           AudioChannelService::AudibleState::eAudible);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -576,9 +576,12 @@ Telephony::HandleAudioAgentState()
     // because the modem have not changed the call state yet. It causes that
     // the telephony can't be resumed. Therefore, we don't mute the telephony
     // at the beginning.
-    volume = 1.0;
-    muted = false;
-    rv = WindowVolumeChanged(volume, muted);
+    rv = WindowVolumeChanged(1.0, false);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+
+    rv = WindowSuspendChanged(config.mSuspend);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -710,6 +713,13 @@ Telephony::WindowVolumeChanged(float aVolume, bool aMuted)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+Telephony::WindowSuspendChanged(nsSuspendedTypes aSuspend)
+{
+  // Not support yet.
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 Telephony::WindowAudioCaptureChanged(bool aCapture)
 {
