@@ -828,7 +828,7 @@ nsEditingSession::OnLocationChange(nsIWebProgress *aWebProgress,
   nsIDocShell *docShell = piWindow->GetDocShell();
   NS_ENSURE_TRUE(docShell, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsICommandManager> commandManager = do_GetInterface(docShell);
+  nsCOMPtr<nsICommandManager> commandManager = docShell->GetCommandManager();
   nsCOMPtr<nsPICommandUpdater> commandUpdater =
                                   do_QueryInterface(commandManager);
   NS_ENSURE_TRUE(commandUpdater, NS_ERROR_FAILURE);
@@ -912,17 +912,6 @@ nsEditingSession::StartDocumentLoad(nsIWebProgress *aWebProgress,
 
   NS_ENSURE_ARG_POINTER(aWebProgress);
 
-  // If we have an editor here, then we got a reload after making the editor.
-  // We need to blow it away and make a new one at the end of the load.
-  nsCOMPtr<mozIDOMWindowProxy> domWindow;
-  aWebProgress->GetDOMWindow(getter_AddRefs(domWindow));
-  if (domWindow)
-  {
-    nsIDocShell *docShell = nsPIDOMWindowOuter::From(domWindow)->GetDocShell();
-    NS_ENSURE_TRUE(docShell, NS_ERROR_FAILURE);
-    docShell->DetachEditorFromWindow();
-  }
-
   if (aIsToBeMadeEditable)
     mEditorStatus = eEditorCreationInProgress;
 
@@ -1041,7 +1030,7 @@ nsEditingSession::TimerCallback(nsITimer* aTimer, void* aClosure)
   {
     nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(docShell));
     if (webNav)
-      webNav->LoadURI(MOZ_UTF16("about:blank"),
+      webNav->LoadURI(u"about:blank",
                       0, nullptr, nullptr, nullptr);
   }
 }
