@@ -25,10 +25,11 @@
 #include "mozilla/layers/LayersSurfaces.h"  // for SurfaceDescriptor
 #include "mozilla/mozalloc.h"           // for operator delete
 #include "mozilla/gfx/CriticalSection.h"
-#include "nsAutoPtr.h"                  // for nsRefPtr
 #include "nsCOMPtr.h"                   // for already_AddRefed
 #include "nsISupportsImpl.h"            // for TextureImage::AddRef, etc
 #include "GfxTexturesReporter.h"
+#include "pratom.h"
+#include "nsThreadUtils.h"
 
 class gfxImageSurface;
 
@@ -47,6 +48,7 @@ class SharedSurface_Gralloc;
 namespace layers {
 
 class AsyncTransactionWaiter;
+class BufferTextureData;
 class CompositableForwarder;
 class GrallocTextureData;
 class ClientIPCAllocator;
@@ -63,6 +65,7 @@ class ITextureClientRecycleAllocator;
 #ifdef GFX_DEBUG_TRACK_CLIENTS_IN_POOL
 class TextureClientPool;
 #endif
+class TextureForwarder;
 class KeepAlive;
 
 /**
@@ -176,6 +179,57 @@ struct MappedYCbCrTextureData {
   }
 };
 
+class ReadLockDescriptor;
+
+// A class to help implement copy-on-write semantics for shared textures.
+//
+// A TextureClient/Host pair can opt into using a ReadLock by calling
+// TextureClient::EnableReadLock. This will equip the TextureClient with a
+// ReadLock object that will be automatically ReadLock()'ed by the texture itself
+// when it is written into (see TextureClient::Unlock).
+// A TextureReadLock's counter starts at 1 and is expected to be equal to 1 when the
+// lock is destroyed. See ShmemTextureReadLock for explanations about why we use
+// 1 instead of 0 as the initial state.
+// TextureReadLock is mostly internally managed by the TextureClient/Host pair,
+// and the compositable only has to forward it during updates. If an update message
+// contains a null_t lock, it means that the texture was not written into on the
+// content side, and there is no synchronization required on the compositor side
+// (or it means that the texture pair did not opt into using ReadLocks).
+// On the compositor side, the TextureHost can receive a ReadLock during a
+// transaction, and will both ReadUnlock() it and drop it as soon as the shared
+// data is available again for writing (the texture upload is done, or the compositor
+// not reading the texture anymore). The lock is dropped to make sure it is
+// ReadUnlock()'ed only once.
+class TextureReadLock {
+protected:
+  virtual ~TextureReadLock() {}
+
+public:
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(TextureReadLock)
+
+  virtual int32_t ReadLock() = 0;
+  virtual int32_t ReadUnlock() = 0;
+  virtual int32_t GetReadCount() = 0;
+  virtual bool IsValid() const = 0;
+
+  static already_AddRefed<TextureReadLock>
+  Create(ClientIPCAllocator* aAllocator);
+
+  static already_AddRefed<TextureReadLock>
+  Deserialize(const ReadLockDescriptor& aDescriptor, ISurfaceAllocator* aAllocator);
+
+  virtual bool Serialize(ReadLockDescriptor& aOutput) = 0;
+
+  enum LockType {
+    TYPE_MEMORY,
+    TYPE_SHMEM
+  };
+  virtual LockType GetType() = 0;
+
+protected:
+  NS_DECL_OWNINGTHREAD
+};
+
 #ifdef XP_WIN
 class D3D11TextureData;
 #endif
@@ -246,6 +300,8 @@ public:
 #endif
 
   virtual GrallocTextureData* AsGrallocTextureData() { return nullptr; }
+
+  virtual BufferTextureData* AsBufferTextureData() { return nullptr; }
 };
 
 /**
@@ -284,6 +340,16 @@ public:
 
   // Creates and allocates a TextureClient usable with Moz2D.
   static already_AddRefed<TextureClient>
+  CreateForDrawing(TextureForwarder* aAllocator,
+                   gfx::SurfaceFormat aFormat,
+                   gfx::IntSize aSize,
+                   LayersBackend aLayersBackend,
+                   BackendSelector aSelector,
+                   TextureFlags aTextureFlags,
+                   TextureAllocationFlags aAllocFlags = ALLOC_DEFAULT);
+
+  // TODO: remove this one and use the one above instead.
+  static already_AddRefed<TextureClient>
   CreateForDrawing(CompositableForwarder* aAllocator,
                    gfx::SurfaceFormat aFormat,
                    gfx::IntSize aSize,
@@ -291,6 +357,14 @@ public:
                    TextureFlags aTextureFlags,
                    TextureAllocationFlags flags = ALLOC_DEFAULT);
 
+  static already_AddRefed<TextureClient>
+  CreateFromSurface(TextureForwarder* aAllocator,
+                    gfx::SourceSurface* aSurface,
+                    LayersBackend aLayersBackend,
+                    BackendSelector aSelector,
+                    TextureFlags aTextureFlags,
+                    TextureAllocationFlags aAllocFlags);
+
   // Creates and allocates a TextureClient supporting the YCbCr format.
   static already_AddRefed<TextureClient>
   CreateForYCbCr(ClientIPCAllocator* aAllocator,
@@ -314,7 +388,6 @@ public:
   // providing format and sizes could let us do more optimization.
   static already_AddRefed<TextureClient>
   CreateForYCbCrWithBufferSize(ClientIPCAllocator* aAllocator,
-                               gfx::SurfaceFormat aFormat,
                                size_t aSize,
                                TextureFlags aTextureFlags);
 
@@ -452,24 +525,9 @@ public:
 
   void RemoveFlags(TextureFlags aFlags);
 
-  // The TextureClient must not be locked when calling this method.
+  // Must not be called when TextureClient is in use by CompositableClient.
   void RecycleTexture(TextureFlags aFlags);
 
-  /**
-   * valid only for TextureFlags::RECYCLE TextureClient.
-   * When called this texture client will grab a strong reference and release
-   * it once the compositor notifies that it is done with the texture.
-   * NOTE: In this stage the texture client can no longer be used by the
-   * client in a transaction.
-   */
-  void WaitForCompositorRecycle();
-
-  /**
-   * Should only be called when dying. We no longer care whether the compositor
-   * has finished with the texture.
-   */
-  void CancelWaitForCompositorRecycle();
-
   /**
    * After being shared with the compositor side, an immutable texture is never
    * modified, it can only be read. It is safe to not Lock/Unlock immutable
@@ -499,12 +557,22 @@ public:
   bool IsAddedToCompositableClient() const { return mAddedToCompositableClient; }
 
   /**
-   * Create and init the TextureChild/Parent IPDL actor pair.
+  * Create and init the TextureChild/Parent IPDL actor pair
+  * with a CompositableForwarder.
+  *
+  * Should be called only once per TextureClient.
+  * The TextureClient must not be locked when calling this method.
+  */
+  bool InitIPDLActor(CompositableForwarder* aForwarder);
+
+  /**
+   * Create and init the TextureChild/Parent IPDL actor pair
+   * with a TextureForwarder.
    *
    * Should be called only once per TextureClient.
    * The TextureClient must not be locked when calling this method.
    */
-  bool InitIPDLActor(CompositableForwarder* aForwarder);
+  bool InitIPDLActor(TextureForwarder* aForwarder, LayersBackend aBackendType);
 
   /**
    * Return a pointer to the IPDLActor.
@@ -547,11 +615,6 @@ public:
     return mAcquireFenceHandle;
   }
 
-  /**
-   * Set AsyncTransactionTracker of RemoveTextureFromCompositableAsync() transaction.
-   */
-  virtual void SetRemoveFromCompositableWaiter(AsyncTransactionWaiter* aWaiter);
-
   /**
    * This function waits until the buffer is no longer being used.
    *
@@ -588,8 +651,51 @@ public:
   TextureData* GetInternalData() { return mData; }
   const TextureData* GetInternalData() const { return mData; }
 
-  virtual void RemoveFromCompositable(CompositableClient* aCompositable,
-                                      AsyncTransactionWaiter* aWaiter = nullptr);
+  uint64_t GetSerial() const { return mSerial; }
+
+  bool NeedsFenceHandle()
+  {
+#if defined(MOZ_WIDGET_GONK) && ANDROID_VERSION >= 17
+    if (!mData) {
+      return false;
+    }
+    return !!mData->AsGrallocTextureData();
+#else
+    return false;
+#endif
+  }
+
+  void WaitFenceHandleOnImageBridge(Mutex& aMutex);
+  void ClearWaitFenceHandleOnImageBridge(Mutex& aMutex);
+  void CancelWaitFenceHandleOnImageBridge();
+
+  void CancelWaitForRecycle();
+
+  /**
+   * Set last transaction id of CompositableForwarder.
+   * 
+   * Called when TextureClient has TextureFlags::RECYCLE flag.
+   * When CompositableForwarder forwards the TextureClient with
+   * TextureFlags::RECYCLE, it holds TextureClient's ref until host side
+   * releases it. The host side sends TextureClient release message.
+   * The id is used to check if the message is for the last TextureClient
+   * forwarding.
+   */
+  void SetLastFwdTransactionId(uint64_t aTransactionId)
+  {
+    MOZ_ASSERT(mFwdTransactionId <= aTransactionId);
+    mFwdTransactionId = aTransactionId;
+  }
+
+  uint64_t GetLastFwdTransactionId() { return mFwdTransactionId; }
+
+  void EnableReadLock();
+
+  TextureReadLock* GetReadLock() { return mReadLock; }
+
+  bool IsReadLocked() const;
+
+  void SerializeReadLock(ReadLockDescriptor& aDescriptor);
 
 private:
   static void TextureClientRecycleCallback(TextureClient* aClient, void* aClosure);
@@ -624,7 +730,7 @@ protected:
   RefPtr<ClientIPCAllocator> mAllocator;
   RefPtr<TextureChild> mActor;
   RefPtr<ITextureClientRecycleAllocator> mRecycleAllocator;
-  RefPtr<AsyncTransactionWaiter> mRemoveFromCompositableWaiter;
+  RefPtr<TextureReadLock> mReadLock;
 
   TextureData* mData;
   RefPtr<gfx::DrawTarget> mBorrowedDrawTarget;
@@ -632,6 +738,8 @@ protected:
   TextureFlags mFlags;
   FenceHandle mReleaseFenceHandle;
   FenceHandle mAcquireFenceHandle;
+  RefPtr<AsyncTransactionWaiter> mFenceHandleWaiter;
+
   gl::GfxTextureWasteTracker mWasteTracker;
 
   OpenMode mOpenMode;
@@ -639,13 +747,26 @@ protected:
   uint32_t mExpectedDtRefs;
 #endif
   bool mIsLocked;
+  // This member tracks that the texture was written into until the update
+  // is sent to the compositor. We need this remember to lock mReadLock on
+  // behalf of the compositor just before sending the notification.
+  bool mUpdated;
 
+  // Used when TextureClient is recycled with TextureFlags::RECYCLE flag.
   bool mAddedToCompositableClient;
+
   bool mWorkaroundAnnoyingSharedSurfaceLifetimeIssues;
   bool mWorkaroundAnnoyingSharedSurfaceOwnershipIssues;
 
   RefPtr<TextureReadbackSink> mReadbackSink;
 
+  uint64_t mFwdTransactionId;
+
+  // Serial id of TextureClient. It is unique in current process.
+  const uint64_t mSerial;
+  // Used to assign serial ids of TextureClient.
+  static mozilla::Atomic<uint64_t> sSerialCounter;
+
   friend class TextureChild;
   friend class RemoveTextureFromCompositableTracker;
   friend void TestTextureClientSurface(TextureClient*, gfxImageSurface*);
