@@ -600,6 +600,8 @@ gfxUserFontEntry::SetLoadState(UserFontLoadState aLoadState)
     mUserFontLoadState = aLoadState;
 }
 
+MOZ_DEFINE_MALLOC_SIZE_OF_ON_ALLOC(UserFontMallocSizeOfOnAlloc)
+
 bool
 gfxUserFontEntry::LoadPlatformFont(const uint8_t* aFontData, uint32_t& aLength)
 {
@@ -626,6 +628,7 @@ gfxUserFontEntry::LoadPlatformFont(const uint8_t* aFontData, uint32_t& aLength)
     // if necessary. The original data in aFontData is left unchanged.
     uint32_t saneLen;
     uint32_t fontCompressionRatio = 0;
+    size_t computedSize = 0;
     const uint8_t* saneData =
         SanitizeOpenTypeData(aFontData, aLength, saneLen, fontType);
     if (!saneData) {
@@ -649,6 +652,15 @@ gfxUserFontEntry::LoadPlatformFont(const uint8_t* aFontData, uint32_t& aLength)
         // arbitrary/malicious data from the web.
         gfxFontUtils::GetFullNameFromSFNT(saneData, saneLen,
                                           originalFullName);
+
+        // Record size for memory reporting purposes. We measure this now
+        // because by the time we potentially want to collect reports, this
+        // data block may have been handed off to opaque OS font APIs that
+        // don't allow us to retrieve or measure it directly.
+        // The *OnAlloc function will also tell DMD about this block, as the
+        // OS font code may hold on to it for an extended period.
+        computedSize = UserFontMallocSizeOfOnAlloc(saneData);
+
         // Here ownership of saneData is passed to the platform,
         // which will delete it when no longer required
         fe = gfxPlatform::GetPlatform()->MakePlatformFont(mName,
@@ -663,6 +675,8 @@ gfxUserFontEntry::LoadPlatformFont(const uint8_t* aFontData, uint32_t& aLength)
     }
 
     if (fe) {
+        fe->mComputedSizeOfUserFont = computedSize;
+
         // Save a copy of the metadata block (if present) for nsIDOMFontFace
         // to use if required. Ownership of the metadata block will be passed
         // to the gfxUserFontData record below.
@@ -1149,6 +1163,13 @@ gfxUserFontSet::UserFontCache::CacheFont(gfxFontEntry* aFontEntry,
             obs->AddObserver(flusher, "last-pb-context-exited", false);
             obs->AddObserver(flusher, "xpcom-shutdown", false);
         }
+
+        // Create and register a memory reporter for sUserFonts.
+        // This reporter is never unregistered, but that's OK because
+        // the reporter checks whether sUserFonts is null, so it would
+        // be safe to call even after UserFontCache::Shutdown has deleted
+        // the cache.
+        RegisterStrongMemoryReporter(new MemoryReporter());
     }
 
     if (data->mLength) {
@@ -1275,6 +1296,89 @@ gfxUserFontSet::UserFontCache::Shutdown()
     }
 }
 
+MOZ_DEFINE_MALLOC_SIZE_OF(UserFontsMallocSizeOf)
+
+nsresult
+gfxUserFontSet::UserFontCache::Entry::ReportMemory(nsIMemoryReporterCallback* aCb,
+                                                   nsISupports* aClosure,
+                                                   bool aAnonymize)
+{
+    nsAutoCString path("explicit/gfx/user-fonts/font(");
+
+    if (aAnonymize) {
+        path.AppendPrintf("<anonymized-%p>", this);
+    } else {
+        if (mFontEntry) { // this should always be present
+            NS_ConvertUTF16toUTF8 familyName(mFontEntry->mFamilyName);
+            path.AppendPrintf("family=%s", familyName.get());
+        }
+        if (mURI) {
+            nsCString spec;
+            mURI->GetSpec(spec);
+            spec.ReplaceChar('/', '\\');
+            // Some fonts are loaded using horrendously-long data: URIs;
+            // truncate those before reporting them.
+            bool isData;
+            if (NS_SUCCEEDED(mURI->SchemeIs("data", &isData)) && isData &&
+                spec.Length() > 255) {
+                spec.Truncate(252);
+                spec.Append("...");
+            }
+            path.AppendPrintf(", url=%s", spec.get());
+        }
+        if (mPrincipal) {
+            nsCOMPtr<nsIURI> uri;
+            mPrincipal->GetURI(getter_AddRefs(uri));
+            nsCString spec;
+            uri->GetSpec(spec);
+            if (!spec.IsEmpty()) {
+                // Include a clue as to who loaded this resource. (Note that
+                // because of font entry sharing, other pages may now be using
+                // this resource, and the original page may not even be loaded
+                // any longer.)
+                spec.ReplaceChar('/', '\\');
+                path.AppendPrintf(", principal=%s", spec.get());
+            }
+        }
+    }
+    path.Append(')');
+
+    return aCb->
+        Callback(EmptyCString(), path,
+                 nsIMemoryReporter::KIND_HEAP, nsIMemoryReporter::UNITS_BYTES,
+                 mFontEntry->ComputedSizeOfExcludingThis(UserFontsMallocSizeOf),
+                 NS_LITERAL_CSTRING("Memory used by @font-face resource."),
+                 aClosure);
+}
+
+NS_IMPL_ISUPPORTS(gfxUserFontSet::UserFontCache::MemoryReporter,
+                  nsIMemoryReporter)
+
+NS_IMETHODIMP
+gfxUserFontSet::UserFontCache::MemoryReporter::CollectReports(
+    nsIMemoryReporterCallback* aCb, nsISupports* aClosure, bool aAnonymize)
+{
+    if (!sUserFonts) {
+        return NS_OK;
+    }
+
+    for (auto it = sUserFonts->Iter(); !it.Done(); it.Next()) {
+        nsresult rv = it.Get()->ReportMemory(aCb, aClosure, aAnonymize);
+        if (NS_FAILED(rv)) {
+            return rv;
+        }
+    }
+
+    return aCb->
+        Callback(EmptyCString(),
+                 NS_LITERAL_CSTRING("explicit/gfx/user-fonts/cache-overhead"),
+                 nsIMemoryReporter::KIND_HEAP, nsIMemoryReporter::UNITS_BYTES,
+                 sUserFonts->ShallowSizeOfIncludingThis(UserFontsMallocSizeOf),
+                 NS_LITERAL_CSTRING("Memory used by the @font-face cache, "
+                                    "not counting the actual font resources."),
+                 aClosure);
+}
+
 #ifdef DEBUG_USERFONT_CACHE
 
 void
@@ -1325,3 +1429,6 @@ gfxUserFontSet::UserFontCache::Dump()
 }
 
 #endif
+
+#undef LOG
+#undef LOG_ENABLED
