@@ -12,19 +12,19 @@
 
 #include "gfx2DGlue.h"
 #include "gfxPlatform.h"
+#include "gfxPrefs.h"
 #include "gfxUtils.h"
 #include "gfxAlphaRecovery.h"
 
-static bool gDisableOptimize = false;
-
 #include "GeckoProfiler.h"
-#include "mozilla/Likely.h"
 #include "MainThreadUtils.h"
-#include "mozilla/MemoryReporting.h"
-#include "nsMargin.h"
-#include "nsThreadUtils.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/gfx/Tools.h"
+#include "mozilla/Likely.h"
+#include "mozilla/MemoryReporting.h"
+#include "mozilla/Telemetry.h"
+#include "nsMargin.h"
+#include "nsThreadUtils.h"
 
 
 namespace mozilla {
@@ -33,8 +33,6 @@ using namespace gfx;
 
 namespace image {
 
-static UserDataKey kVolatileBuffer;
-
 static void
 VolatileBufferRelease(void* vbuf)
 {
@@ -57,15 +55,18 @@ CreateLockedSurface(VolatileBuffer* vbuf,
     new VolatileBufferPtr<unsigned char>(vbuf);
   MOZ_ASSERT(!vbufptr->WasBufferPurged(), "Expected image data!");
 
-  int32_t stride = VolatileSurfaceStride(size, format);
+  const int32_t stride = VolatileSurfaceStride(size, format);
+
+  // The VolatileBufferPtr is held by this DataSourceSurface.
   RefPtr<DataSourceSurface> surf =
-    Factory::CreateWrappingDataSourceSurface(*vbufptr, stride, size, format);
+    Factory::CreateWrappingDataSourceSurface(*vbufptr, stride, size, format,
+                                             &VolatileBufferRelease,
+                                             static_cast<void*>(vbufptr));
   if (!surf) {
     delete vbufptr;
     return nullptr;
   }
 
-  surf->AddUserData(&kVolatileBuffer, vbufptr, VolatileBufferRelease);
   return surf.forget();
 }
 
@@ -82,6 +83,32 @@ AllocateBufferForImage(const IntSize& size, SurfaceFormat format)
   return nullptr;
 }
 
+static bool
+ClearSurface(VolatileBuffer* aVBuf, const IntSize& aSize, SurfaceFormat aFormat)
+{
+  VolatileBufferPtr<unsigned char> vbufptr(aVBuf);
+  if (vbufptr.WasBufferPurged()) {
+    NS_WARNING("VolatileBuffer was purged");
+    return false;
+  }
+
+  int32_t stride = VolatileSurfaceStride(aSize, aFormat);
+  if (aFormat == SurfaceFormat::B8G8R8X8) {
+    // Skia doesn't support RGBX surfaces, so ensure the alpha value is set
+    // to opaque white. While it would be nice to only do this for Skia,
+    // imgFrame can run off main thread and past shutdown where
+    // we might not have gfxPlatform, so just memset everytime instead.
+    memset(vbufptr, 0xFF, stride * aSize.height);
+  } else if (aVBuf->OnHeap()) {
+    // We only need to memset it if the buffer was allocated on the heap.
+    // Otherwise, it's allocated via mmap and refers to a zeroed page and will
+    // be COW once it's written to.
+    memset(vbufptr, 0, stride * aSize.height);
+  }
+
+  return true;
+}
+
 // Returns true if an image of aWidth x aHeight is allowed and legal.
 static bool
 AllowedImageSize(int32_t aWidth, int32_t aHeight)
@@ -135,7 +162,7 @@ imgFrame::imgFrame()
   : mMonitor("imgFrame")
   , mDecoded(0, 0, 0, 0)
   , mLockCount(0)
-  , mTimeout(100)
+  , mTimeout(FrameTimeout::FromRawMilliseconds(100))
   , mDisposalMethod(DisposalMethod::NOT_SPECIFIED)
   , mBlendMethod(BlendMethod::OVER)
   , mHasNoAlpha(false)
@@ -145,16 +172,8 @@ imgFrame::imgFrame()
   , mPalettedImageData(nullptr)
   , mPaletteDepth(0)
   , mNonPremult(false)
-  , mSinglePixel(false)
   , mCompositingFailed(false)
 {
-  static bool hasCheckedOptimize = false;
-  if (!hasCheckedOptimize) {
-    if (PR_GetEnv("MOZ_DISABLE_IMAGE_OPTIMIZE")) {
-      gDisableOptimize = true;
-    }
-    hasCheckedOptimize = true;
-  }
 }
 
 imgFrame::~imgFrame()
@@ -185,8 +204,20 @@ imgFrame::InitForDecoder(const nsIntSize& aImageSize,
   }
 
   mImageSize = aImageSize;
-  mOffset.MoveTo(aRect.x, aRect.y);
-  mSize.SizeTo(aRect.width, aRect.height);
+  mFrameRect = aRect;
+
+  // We only allow a non-trivial frame rect (i.e., a frame rect that doesn't
+  // cover the entire image) for paletted animation frames. We never draw those
+  // frames directly; we just use FrameAnimator to composite them and produce a
+  // BGRA surface that we actually draw. We enforce this here to make sure that
+  // imgFrame::Draw(), which is responsible for drawing all other kinds of
+  // frames, never has to deal with a non-trivial frame rect.
+  if (aPaletteDepth == 0 &&
+      !mFrameRect.IsEqualEdges(IntRect(IntPoint(), mImageSize))) {
+    MOZ_ASSERT_UNREACHABLE("Creating a non-paletted imgFrame with a "
+                           "non-trivial frame rect");
+    return NS_ERROR_FAILURE;
+  }
 
   mFormat = aFormat;
   mPaletteDepth = aPaletteDepth;
@@ -203,30 +234,31 @@ imgFrame::InitForDecoder(const nsIntSize& aImageSize,
 
     // Use the fallible allocator here. Paletted images always use 1 byte per
     // pixel, so calculating the amount of memory we need is straightforward.
-    mPalettedImageData =
-      static_cast<uint8_t*>(malloc(PaletteDataLength() +
-                                   (mSize.width * mSize.height)));
+    size_t dataSize = PaletteDataLength() + mFrameRect.Area();
+    mPalettedImageData = static_cast<uint8_t*>(calloc(dataSize, sizeof(uint8_t)));
     if (!mPalettedImageData) {
-      NS_WARNING("malloc for paletted image data should succeed");
+      NS_WARNING("Call to calloc for paletted image data should succeed");
     }
     NS_ENSURE_TRUE(mPalettedImageData, NS_ERROR_OUT_OF_MEMORY);
   } else {
     MOZ_ASSERT(!mImageSurface, "Called imgFrame::InitForDecoder() twice?");
 
-    mVBuf = AllocateBufferForImage(mSize, mFormat);
+    mVBuf = AllocateBufferForImage(mFrameRect.Size(), mFormat);
     if (!mVBuf) {
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    if (mVBuf->OnHeap()) {
-      int32_t stride = VolatileSurfaceStride(mSize, mFormat);
-      VolatileBufferPtr<uint8_t> ptr(mVBuf);
-      memset(ptr, 0, stride * mSize.height);
-    }
-    mImageSurface = CreateLockedSurface(mVBuf, mSize, mFormat);
+
+    mImageSurface = CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
 
     if (!mImageSurface) {
-      NS_WARNING("Failed to create VolatileDataSourceSurface");
+      NS_WARNING("Failed to create ImageSurface");
+      mAborted = true;
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+
+    if (!ClearSurface(mVBuf, mFrameRect.Size(), mFormat)) {
+      NS_WARNING("Could not clear allocated buffer");
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -239,7 +271,7 @@ nsresult
 imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
                            const nsIntSize& aSize,
                            const SurfaceFormat aFormat,
-                           Filter aFilter,
+                           SamplingFilter aSamplingFilter,
                            uint32_t aImageFlags)
 {
   // Assert for properties that should be verified by decoders,
@@ -251,8 +283,7 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
   }
 
   mImageSize = aSize;
-  mOffset.MoveTo(0, 0);
-  mSize.SizeTo(aSize.width, aSize.height);
+  mFrameRect = IntRect(IntPoint(0, 0), aSize);
 
   mFormat = aFormat;
   mPaletteDepth = 0;
@@ -267,25 +298,35 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
     // get away with using volatile buffers.
     MOZ_ASSERT(!mImageSurface, "Called imgFrame::InitWithDrawable() twice?");
 
-    mVBuf = AllocateBufferForImage(mSize, mFormat);
+    mVBuf = AllocateBufferForImage(mFrameRect.Size(), mFormat);
     if (!mVBuf) {
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    int32_t stride = VolatileSurfaceStride(mSize, mFormat);
+    int32_t stride = VolatileSurfaceStride(mFrameRect.Size(), mFormat);
     VolatileBufferPtr<uint8_t> ptr(mVBuf);
     if (!ptr) {
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    if (mVBuf->OnHeap()) {
-      memset(ptr, 0, stride * mSize.height);
+
+    mImageSurface = CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
+
+    if (!mImageSurface) {
+      NS_WARNING("Failed to create ImageSurface");
+      mAborted = true;
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+
+    if (!ClearSurface(mVBuf, mFrameRect.Size(), mFormat)) {
+      NS_WARNING("Could not clear allocated buffer");
+      mAborted = true;
+      return NS_ERROR_OUT_OF_MEMORY;
     }
-    mImageSurface = CreateLockedSurface(mVBuf, mSize, mFormat);
 
     target = gfxPlatform::GetPlatform()->
-      CreateDrawTargetForData(ptr, mSize, stride, mFormat);
+      CreateDrawTargetForData(ptr, mFrameRect.Size(), stride, mFormat);
   } else {
     // We can't use data surfaces for content, so we'll create an offscreen
     // surface instead.  This means if someone later calls RawAccessRef(), we
@@ -294,7 +335,7 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
     MOZ_ASSERT(!mOptSurface, "Called imgFrame::InitWithDrawable() twice?");
 
     target = gfxPlatform::GetPlatform()->
-      CreateOffscreenContentDrawTarget(mSize, mFormat);
+      CreateOffscreenContentDrawTarget(mFrameRect.Size(), mFormat);
   }
 
   if (!target || !target->IsValid()) {
@@ -303,12 +344,11 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
   }
 
   // Draw using the drawable the caller provided.
-  nsIntRect imageRect(0, 0, mSize.width, mSize.height);
-  RefPtr<gfxContext> ctx = gfxContext::ForDrawTarget(target);
-  MOZ_ASSERT(ctx); // already checked the draw target above
-  gfxUtils::DrawPixelSnapped(ctx, aDrawable, mSize,
-                             ImageRegion::Create(ThebesRect(imageRect)),
-                             mFormat, aFilter, aImageFlags);
+  RefPtr<gfxContext> ctx = gfxContext::CreateOrNull(target);
+  MOZ_ASSERT(ctx);  // Already checked the draw target above.
+  gfxUtils::DrawPixelSnapped(ctx, aDrawable, mFrameRect.Size(),
+                             ImageRegion::Create(ThebesRect(mFrameRect)),
+                             mFormat, aSamplingFilter, aImageFlags);
 
   if (canUseDataSurface && !mImageSurface) {
     NS_WARNING("Failed to create VolatileDataSourceSurface");
@@ -334,6 +374,23 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
   return NS_OK;
 }
 
+bool
+imgFrame::CanOptimizeOpaqueImage()
+{
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(!ShutdownTracker::ShutdownHasStarted());
+  mMonitor.AssertCurrentThreadOwns();
+
+  // If we're using a surface format with alpha but the image has no alpha,
+  // change the format. This doesn't change the underlying data at all, but
+  // allows DrawTargets to avoid blending when drawing known opaque images.
+  // This optimization is free and safe, so we always do it when we can except
+  // if we have a Skia backend. Skia doesn't support RGBX so ensure we don't
+  // optimize to a RGBX surface.
+  return mHasNoAlpha && mFormat == SurfaceFormat::B8G8R8A8 && mImageSurface &&
+         (gfxPlatform::GetPlatform()->GetDefaultContentBackend() != BackendType::SKIA);
+}
+
 nsresult
 imgFrame::Optimize()
 {
@@ -342,77 +399,57 @@ imgFrame::Optimize()
   MOZ_ASSERT(mLockCount == 1,
              "Should only optimize when holding the lock exclusively");
 
+  // Check whether image optimization is disabled -- not thread safe!
+  static bool gDisableOptimize = false;
+  static bool hasCheckedOptimize = false;
+  if (!hasCheckedOptimize) {
+    if (PR_GetEnv("MOZ_DISABLE_IMAGE_OPTIMIZE")) {
+      gDisableOptimize = true;
+    }
+    hasCheckedOptimize = true;
+  }
+
   // Don't optimize during shutdown because gfxPlatform may not be available.
   if (ShutdownTracker::ShutdownHasStarted()) {
     return NS_OK;
   }
 
+  // This optimization is basically free, so we perform it even if optimization is disabled.
+  if (CanOptimizeOpaqueImage()) {
+    mFormat = SurfaceFormat::B8G8R8X8;
+    mImageSurface = CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
+  }
+
   if (!mOptimizable || gDisableOptimize) {
     return NS_OK;
   }
 
-  if (mPalettedImageData || mOptSurface || mSinglePixel) {
+  if (mPalettedImageData || mOptSurface) {
     return NS_OK;
   }
 
-  // Don't do single-color opts on non-premult data.
-  // Cairo doesn't support non-premult single-colors.
+  // XXX(seth): It's currently unclear if there's any reason why we can't
+  // optimize non-premult surfaces. We should look into removing this.
   if (mNonPremult) {
     return NS_OK;
   }
 
-  /* Figure out if the entire image is a constant color */
-
-  if (gfxPrefs::ImageSingleColorOptimizationEnabled() &&
-      mImageSurface->Stride() == mSize.width * 4) {
-    uint32_t* imgData = (uint32_t*) ((uint8_t*) mVBufPtr);
-    uint32_t firstPixel = * (uint32_t*) imgData;
-    uint32_t pixelCount = mSize.width * mSize.height + 1;
-
-    while (--pixelCount && *imgData++ == firstPixel)
-      ;
-
-    if (pixelCount == 0) {
-      // all pixels were the same
-      if (mFormat == SurfaceFormat::B8G8R8A8 ||
-          mFormat == SurfaceFormat::B8G8R8X8) {
-        mSinglePixel = true;
-        mSinglePixelColor.a = ((firstPixel >> 24) & 0xFF) * (1.0f / 255.0f);
-        mSinglePixelColor.r = ((firstPixel >> 16) & 0xFF) * (1.0f / 255.0f);
-        mSinglePixelColor.g = ((firstPixel >>  8) & 0xFF) * (1.0f / 255.0f);
-        mSinglePixelColor.b = ((firstPixel >>  0) & 0xFF) * (1.0f / 255.0f);
-        mSinglePixelColor.r /= mSinglePixelColor.a;
-        mSinglePixelColor.g /= mSinglePixelColor.a;
-        mSinglePixelColor.b /= mSinglePixelColor.a;
-
-        // blow away the older surfaces (if they exist), to release their memory
-        mVBuf = nullptr;
-        mVBufPtr = nullptr;
-        mImageSurface = nullptr;
-        mOptSurface = nullptr;
-
-        return NS_OK;
-      }
-    }
-
-    // if it's not RGB24/ARGB32, don't optimize, but we never hit this at the
-    // moment
-  }
-
 #ifdef ANDROID
   SurfaceFormat optFormat = gfxPlatform::GetPlatform()
     ->Optimal2DFormatForContent(gfxContentType::COLOR);
 
   if (mFormat != SurfaceFormat::B8G8R8A8 &&
       optFormat == SurfaceFormat::R5G6B5_UINT16) {
+    Telemetry::Accumulate(Telemetry::IMAGE_OPTIMIZE_TO_565_USED, true);
+
     RefPtr<VolatileBuffer> buf =
-      AllocateBufferForImage(mSize, optFormat);
+      AllocateBufferForImage(mFrameRect.Size(), optFormat);
     if (!buf) {
       return NS_OK;
     }
 
     RefPtr<DataSourceSurface> surf =
-      CreateLockedSurface(buf, mSize, optFormat);
+      CreateLockedSurface(buf, mFrameRect.Size(), optFormat);
     if (!surf) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -426,7 +463,7 @@ imgFrame::Optimize()
     RefPtr<DrawTarget> target =
       Factory::CreateDrawTargetForData(BackendType::CAIRO,
                                        mapping.mData,
-                                       mSize,
+                                       mFrameRect.Size(),
                                        mapping.mStride,
                                        optFormat);
 
@@ -434,7 +471,7 @@ imgFrame::Optimize()
       gfxWarning() << "imgFrame::Optimize failed in CreateDrawTargetForData";
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    Rect rect(0, 0, mSize.width, mSize.height);
+    Rect rect(0, 0, mFrameRect.width, mFrameRect.height);
     target->DrawSurface(mImageSurface, rect, rect);
     target->Flush();
     surf->Unmap();
@@ -491,69 +528,54 @@ imgFrame::SetRawAccessOnly()
 
 
 imgFrame::SurfaceWithFormat
-imgFrame::SurfaceForDrawing(bool               aDoPadding,
-                            bool               aDoPartialDecode,
+imgFrame::SurfaceForDrawing(bool               aDoPartialDecode,
                             bool               aDoTile,
-                            gfxContext*        aContext,
-                            const nsIntMargin& aPadding,
-                            gfxRect&           aImageRect,
                             ImageRegion&       aRegion,
                             SourceSurface*     aSurface)
 {
   MOZ_ASSERT(NS_IsMainThread());
   mMonitor.AssertCurrentThreadOwns();
 
-  IntSize size(int32_t(aImageRect.Width()), int32_t(aImageRect.Height()));
-  if (!aDoPadding && !aDoPartialDecode) {
-    NS_ASSERTION(!mSinglePixel, "This should already have been handled");
-    return SurfaceWithFormat(new gfxSurfaceDrawable(aSurface, size), mFormat);
+  if (!aDoPartialDecode) {
+    return SurfaceWithFormat(new gfxSurfaceDrawable(aSurface, mImageSize),
+                                                    mFormat);
   }
 
   gfxRect available = gfxRect(mDecoded.x, mDecoded.y, mDecoded.width,
                               mDecoded.height);
 
-  if (aDoTile || mSinglePixel) {
+  if (aDoTile) {
     // Create a temporary surface.
     // Give this surface an alpha channel because there are
     // transparent pixels in the padding or undecoded area
     RefPtr<DrawTarget> target =
       gfxPlatform::GetPlatform()->
-        CreateOffscreenContentDrawTarget(size, SurfaceFormat::B8G8R8A8);
+        CreateOffscreenContentDrawTarget(mImageSize, SurfaceFormat::B8G8R8A8);
     if (!target) {
       return SurfaceWithFormat();
     }
 
-    // Fill 'available' with whatever we've got
-    if (mSinglePixel) {
-      target->FillRect(ToRect(aRegion.Intersect(available).Rect()),
-                       ColorPattern(mSinglePixelColor),
-                       DrawOptions(1.0f, CompositionOp::OP_SOURCE));
-    } else {
-      SurfacePattern pattern(aSurface,
-                             aRegion.GetExtendMode(),
-                             Matrix::Translation(mDecoded.x, mDecoded.y));
-      target->FillRect(ToRect(aRegion.Intersect(available).Rect()), pattern);
-    }
+    SurfacePattern pattern(aSurface,
+                           aRegion.GetExtendMode(),
+                           Matrix::Translation(mDecoded.x, mDecoded.y));
+    target->FillRect(ToRect(aRegion.Intersect(available).Rect()), pattern);
 
     RefPtr<SourceSurface> newsurf = target->Snapshot();
-    return SurfaceWithFormat(new gfxSurfaceDrawable(newsurf, size),
+    return SurfaceWithFormat(new gfxSurfaceDrawable(newsurf, mImageSize),
                              target->GetFormat());
   }
 
   // Not tiling, and we have a surface, so we can account for
-  // padding and/or a partial decode just by twiddling parameters.
-  gfxPoint paddingTopLeft(aPadding.left, aPadding.top);
-  aRegion = aRegion.Intersect(available) - paddingTopLeft;
-  aContext->Multiply(gfxMatrix::Translation(paddingTopLeft));
-  aImageRect = gfxRect(0, 0, mSize.width, mSize.height);
-
+  // a partial decode just by twiddling parameters.
+  aRegion = aRegion.Intersect(available);
   IntSize availableSize(mDecoded.width, mDecoded.height);
+
   return SurfaceWithFormat(new gfxSurfaceDrawable(aSurface, availableSize),
                            mFormat);
 }
 
 bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
-                    Filter aFilter, uint32_t aImageFlags)
+                    SamplingFilter aSamplingFilter, uint32_t aImageFlags)
 {
   PROFILER_LABEL("imgFrame", "Draw",
     js::ProfileEntry::Category::GRAPHICS);
@@ -563,31 +585,20 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
   NS_ASSERTION(!aRegion.IsRestricted() ||
                !aRegion.Rect().Intersect(aRegion.Restriction()).IsEmpty(),
                "We must be allowed to sample *some* source pixels!");
-  NS_ASSERTION(!mPalettedImageData, "Directly drawing a paletted image!");
+  MOZ_ASSERT(mFrameRect.IsEqualEdges(IntRect(IntPoint(), mImageSize)),
+             "Directly drawing an image with a non-trivial frame rect!");
+
+  if (mPalettedImageData) {
+    MOZ_ASSERT_UNREACHABLE("Directly drawing a paletted image!");
+    return false;
+  }
 
   MonitorAutoLock lock(mMonitor);
 
-  nsIntMargin padding(mOffset.y,
-                      mImageSize.width - (mOffset.x + mSize.width),
-                      mImageSize.height - (mOffset.y + mSize.height),
-                      mOffset.x);
-
-  bool doPadding = padding != nsIntMargin(0,0,0,0);
   bool doPartialDecode = !AreAllPixelsWritten();
 
-  if (mSinglePixel && !doPadding && !doPartialDecode) {
-    if (mSinglePixelColor.a == 0.0) {
-      return true;
-    }
-    RefPtr<DrawTarget> dt = aContext->GetDrawTarget();
-    dt->FillRect(ToRect(aRegion.Rect()),
-                 ColorPattern(mSinglePixelColor),
-                 DrawOptions(1.0f, aContext->CurrentOp()));
-    return true;
-  }
-
-  RefPtr<SourceSurface> surf = GetSurfaceInternal();
-  if (!surf && !mSinglePixel) {
+  RefPtr<SourceSurface> surf = GetSourceSurfaceInternal();
+  if (!surf) {
     return false;
   }
 
@@ -596,21 +607,13 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
                 !(aImageFlags & imgIContainer::FLAG_CLAMP);
 
   ImageRegion region(aRegion);
-  // SurfaceForDrawing changes the current transform, and we need it to still
-  // be changed when we call gfxUtils::DrawPixelSnapped. We still need to
-  // restore it before returning though.
-  // XXXjwatt In general having functions require someone further up the stack
-  // to undo transform changes that they make is bad practice. We should
-  // change how this code works.
-  gfxContextMatrixAutoSaveRestore autoSR(aContext);
   SurfaceWithFormat surfaceResult =
-    SurfaceForDrawing(doPadding, doPartialDecode, doTile, aContext,
-                      padding, imageRect, region, surf);
+    SurfaceForDrawing(doPartialDecode, doTile, region, surf);
 
   if (surfaceResult.IsValid()) {
     gfxUtils::DrawPixelSnapped(aContext, surfaceResult.mDrawable,
                                imageRect.Size(), region, surfaceResult.mFormat,
-                               aFilter, aImageFlags);
+                               aSamplingFilter, aImageFlags);
   }
   return true;
 }
@@ -629,10 +632,9 @@ imgFrame::ImageUpdatedInternal(const nsIntRect& aUpdateRect)
 
   mDecoded.UnionRect(mDecoded, aUpdateRect);
 
-  // clamp to bounds, in case someone sends a bogus updateRect (I'm looking at
-  // you, gif decoder)
-  nsIntRect boundsRect(mOffset, mSize);
-  mDecoded.IntersectRect(mDecoded, boundsRect);
+  // Clamp to the frame rect to ensure that decoder bugs don't result in a
+  // decoded rect that extends outside the bounds of the frame rect.
+  mDecoded.IntersectRect(mDecoded, mFrameRect);
 
   return NS_OK;
 }
@@ -640,8 +642,10 @@ imgFrame::ImageUpdatedInternal(const nsIntRect& aUpdateRect)
 void
 imgFrame::Finish(Opacity aFrameOpacity /* = Opacity::SOME_TRANSPARENCY */,
                  DisposalMethod aDisposalMethod /* = DisposalMethod::KEEP */,
-                 int32_t aRawTimeout /* = 0 */,
-                 BlendMethod aBlendMethod /* = BlendMethod::OVER */)
+                 FrameTimeout aTimeout
+                   /* = FrameTimeout::FromRawMilliseconds(0) */,
+                 BlendMethod aBlendMethod /* = BlendMethod::OVER */,
+                 const Maybe<IntRect>& aBlendRect /* = Nothing() */)
 {
   MonitorAutoLock lock(mMonitor);
   MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
@@ -651,8 +655,9 @@ imgFrame::Finish(Opacity aFrameOpacity /* = Opacity::SOME_TRANSPARENCY */,
   }
 
   mDisposalMethod = aDisposalMethod;
-  mTimeout = aRawTimeout;
+  mTimeout = aTimeout;
   mBlendMethod = aBlendMethod;
+  mBlendRect = aBlendRect;
   ImageUpdatedInternal(GetRect());
   mFinished = true;
 
@@ -660,42 +665,17 @@ imgFrame::Finish(Opacity aFrameOpacity /* = Opacity::SOME_TRANSPARENCY */,
   mMonitor.NotifyAll();
 }
 
-nsIntRect
-imgFrame::GetRect() const
-{
-  return gfx::IntRect(mOffset, mSize);
-}
-
-int32_t
-imgFrame::GetStride() const
-{
-  mMonitor.AssertCurrentThreadOwns();
-
-  if (mImageSurface) {
-    return mImageSurface->Stride();
-  }
-
-  return VolatileSurfaceStride(mSize, mFormat);
-}
-
-SurfaceFormat
-imgFrame::GetFormat() const
-{
-  MonitorAutoLock lock(mMonitor);
-  return mFormat;
-}
-
 uint32_t
 imgFrame::GetImageBytesPerRow() const
 {
   mMonitor.AssertCurrentThreadOwns();
 
   if (mVBuf) {
-    return mSize.width * BytesPerPixel(mFormat);
+    return mFrameRect.width * BytesPerPixel(mFormat);
   }
 
   if (mPaletteDepth) {
-    return mSize.width;
+    return mFrameRect.width;
   }
 
   return 0;
@@ -704,7 +684,7 @@ imgFrame::GetImageBytesPerRow() const
 uint32_t
 imgFrame::GetImageDataLength() const
 {
-  return GetImageBytesPerRow() * mSize.height;
+  return GetImageBytesPerRow() * mFrameRect.height;
 }
 
 void
@@ -855,14 +835,6 @@ imgFrame::UnlockImageData()
       return NS_OK;
     }
 
-    // If we're using a surface format with alpha but the image has no alpha,
-    // change the format. This doesn't change the underlying data at all, but
-    // allows DrawTargets to avoid blending when drawing known opaque images.
-    if (mHasNoAlpha && mFormat == SurfaceFormat::B8G8R8A8 && mImageSurface) {
-      mFormat = SurfaceFormat::B8G8R8X8;
-      mImageSurface = CreateLockedSurface(mVBuf, mSize, mFormat);
-    }
-
     // Convert the data surface to a GPU surface or a single color if possible.
     // This will also release mImageSurface if possible.
     Optimize();
@@ -884,29 +856,15 @@ imgFrame::SetOptimizable()
   mOptimizable = true;
 }
 
-Color
-imgFrame::SinglePixelColor() const
-{
-  MOZ_ASSERT(NS_IsMainThread());
-  return mSinglePixelColor;
-}
-
-bool
-imgFrame::IsSinglePixel() const
-{
-  MOZ_ASSERT(NS_IsMainThread());
-  return mSinglePixel;
-}
-
 already_AddRefed<SourceSurface>
-imgFrame::GetSurface()
+imgFrame::GetSourceSurface()
 {
   MonitorAutoLock lock(mMonitor);
-  return GetSurfaceInternal();
+  return GetSourceSurfaceInternal();
 }
 
 already_AddRefed<SourceSurface>
-imgFrame::GetSurfaceInternal()
+imgFrame::GetSourceSurfaceInternal()
 {
   mMonitor.AssertCurrentThreadOwns();
 
@@ -933,24 +891,7 @@ imgFrame::GetSurfaceInternal()
     return nullptr;
   }
 
-  return CreateLockedSurface(mVBuf, mSize, mFormat);
-}
-
-already_AddRefed<DrawTarget>
-imgFrame::GetDrawTarget()
-{
-  MonitorAutoLock lock(mMonitor);
-
-  uint8_t* data;
-  uint32_t length;
-  GetImageDataInternal(&data, &length);
-  if (!data) {
-    return nullptr;
-  }
-
-  int32_t stride = GetStride();
-  return gfxPlatform::GetPlatform()->
-    CreateDrawTargetForData(data, mSize, stride, mFormat);
+  return CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
 }
 
 AnimationData
@@ -970,21 +911,7 @@ imgFrame::GetAnimationData() const
   bool hasAlpha = mFormat == SurfaceFormat::B8G8R8A8;
 
   return AnimationData(data, PaletteDataLength(), mTimeout, GetRect(),
-                       mBlendMethod, mDisposalMethod, hasAlpha);
-}
-
-ScalingData
-imgFrame::GetScalingData() const
-{
-  MonitorAutoLock lock(mMonitor);
-  MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
-  MOZ_ASSERT(!GetIsPaletted(), "GetScalingData can't handle paletted images");
-
-  uint8_t* data;
-  uint32_t length;
-  GetImageDataInternal(&data, &length);
-
-  return ScalingData(data, mSize, GetImageBytesPerRow(), mFormat);
+                       mBlendMethod, mBlendRect, mDisposalMethod, hasAlpha);
 }
 
 void
@@ -1032,8 +959,7 @@ bool
 imgFrame::AreAllPixelsWritten() const
 {
   mMonitor.AssertCurrentThreadOwns();
-  return mDecoded.IsEqualInterior(nsIntRect(mOffset.x, mOffset.y,
-                                            mSize.width, mSize.height));
+  return mDecoded.IsEqualInterior(mFrameRect);
 }
 
 bool imgFrame::GetCompositingFailed() const
